@@ -1,4 +1,4 @@
-# $Id: Random.pm,v 1.11 2003/05/01 16:59:27 jfontain Exp $
+# $Id: Random.pm,v 1.16 2004/10/16 21:08:40 jfontain Exp $
 
 # Perl module sample file with random data
 
@@ -11,6 +11,11 @@
 
 package Random; # must be capitalized
 
+# Note: the subroutines after(), yield(), flashMessage(), pushMessage(),
+# popMessage() and traceMessage() are defined by the core in the module
+# namespace and therefore should be seen as reserved subroutines, not to
+# be redefined in the module (this file for the Random module).
+
 
 use strict;
 use warnings; # highly recommended, makes debugging much easier
@@ -22,7 +27,7 @@ BEGIN {
     # The version number is mandatory. It must match with the
     # pkgIndex.tcl file content.
     # Example: our $VERSION = 1.18;
-    our $VERSION = qw($Revision: 1.11 $)[1];
+    our $VERSION = qw($Revision: 1.16 $)[1];
     # Note: see version number management requirements in moodss HTML
     # documentation (Module Development section)
 }
@@ -31,6 +36,7 @@ BEGIN {
 # (see update subroutine).
 our %data;
 our @data;
+our $asynchronous;
 
 # Hash members are:
 
@@ -109,6 +115,12 @@ $data{views} = [
     {indices => [0, 2, 4], sort => {2 => 'decreasing'}}
 ];
 
+# - persistent: optional (considered false if missing). A boolean
+#   value that tells whether row numbers (used by the core, along with
+#   the column numbers, to identify data cells) are identically mapped
+#   from data row keys across module instances.
+$data{persistent} = 1;
+
 # - switches: optional. A hash of boolean values indexed by switches.
 #   A switch is a single letter or a string prepended with the - or +
 #   sign. The boolean value (0 or 1) specifies whether the switch takes
@@ -118,7 +130,8 @@ $data{views} = [
 #   parsing the command line and reject any invalid switch / value
 #   combination for the module. The switches value may not be changed in
 #   the initialize subroutine.
-$data{switches} = {'-i' => 0, '--identify' => 0};
+$data{switches} =
+    {'-a' => 0, '--asynchronous' => 0, '-i' => 0, '--identify' => 0};
 
 # - identifier: optional. A string that uniquely identifies this
 #   module. It will be displayed by the core in the initial data tables
@@ -166,6 +179,13 @@ $data{helpText} = `cat Random.htm`; # load HTML formatted help
 # the core.
 sub initialize(%) {
     my %option = @_;
+    $asynchronous = ($option{'-a'} || $option{'--asynchronous'});
+    if ($asynchronous) {
+        $data{pollTimes} = [-10];
+        # use the core to call a subroutine after a number of milliseconds
+        # without blocking the user interface, as sleep() would do
+        after(3000, 'Random::update'); # boot simulation
+    }
     if ($option{'-i'} || $option{'--identify'}) {
         # generate a unique module identifier:
         my $identifier = int(rand(100));
@@ -208,7 +228,17 @@ sub update() {
     $data[6][1] = sprintf('%.1f', rand(20)); $data[6][2] = 90 + int(rand(20)); $data[6][3] = 9 + int(rand(20));
     $data[9][0] = 'Laura'; $data[9][4] = 'cc';
     $data[9][1] = sprintf('%.1f', rand(30)); $data[9][2] = 100 + int(rand(50)); $data[9][3] = 10 + int(rand(50));
+    if (int(rand(4)) == 0) {
+        # it is possible to post messages to the user:
+        flashMessage('the Random module is still at work', 2);
+        # also available: pushMessage(), popMessage() and traceMessage()
+    }
     $data{updates}++;
+    if ($asynchronous) { # simulate asynchronous operation if requested
+        # use the core to call a subroutine after a number of milliseconds
+        # without blocking the user interface, as sleep() would do
+        after(2000 + int(rand(9000)), 'Random::update');
+    }
 }
 
 END {
