@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: apache.tcl,v 2.17 2004/01/01 11:39:06 jfontain Exp $
+# $Id: apache.tcl,v 2.20 2005/01/02 00:45:07 jfontain Exp $
 
 
-package provide apache [lindex {$Revision: 2.17 $} 1]
+package provide apache [lindex {$Revision: 2.20 $} 1]
 package require miscellaneous 1
 package require apacheutilities 1
 
@@ -27,11 +27,13 @@ namespace eval apache {
         11,label kilobytes/second 11,type real 11,message {current kilobytes served per second}
         pollTimes {10 5 20 30 60 120 300}
         views {{visibleColumns {1 2 3 4 5 6 7 8 9 10 11} swap 1}}
+        persistent 1
         switches {-r 1 --remote 1 --proxyhost 1 --proxyport 1}
     }
     set file [open apache.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
+    unset file
 
     proc initialize {optionsName} {
         upvar 1 $optionsName options
