@@ -1,14 +1,14 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: canvaswm.tcl,v 2.36 2004/01/01 11:39:06 jfontain Exp $
+# $Id: canvaswm.tcl,v 2.66 2005/02/19 21:16:23 jfontain Exp $
 
 
 class canvasWindowManager {
 
     proc canvasWindowManager {this canvas} {
-        set ($this,bindings) [new bindings $canvas end]
-        bindings::set $($this,bindings) <Configure> "canvasWindowManager::rearrangeIcons $this %w %h"
+        # used for icons (minimized objects) in the canvas window manager context, also used by canvas viewers
+        set ($this,drag) [new dragSite -path $canvas -validcommand "canvasWindowManager::validateDrag $this" -grab 0]
         set ($this,canvas) $canvas
     }
 
@@ -18,7 +18,6 @@ class canvasWindowManager {
         variable ${this}handleCoordinates
         variable ${this}handleIconCoordinates
 
-        delete $($this,bindings)
         foreach {handle icon} [array get ${this}handleIcon] {
             delete $icon
         }
@@ -26,7 +25,8 @@ class canvasWindowManager {
             delete $handle
         }
         catch {unset ${this}data}
-        catch {unset ${this}handleIcon ${this}handleCoordinates ${this}handleIconCoordinates}
+        catch {unset ${this}handleIcon}; catch {unset ${this}handleCoordinates}; catch {unset ${this}handleIconCoordinates}
+        delete $($this,drag)
     }
 
     proc manage {this path viewer} {                                           ;# viewer, table or database cell histories container
@@ -45,10 +45,11 @@ class canvasWindowManager {
 
         set handle [set ${this}data(handle,$path)]
         if {[info exists ${this}handleIcon($handle)]} {                                                                 ;# minimized
-            delete [set ${this}handleIcon($handle)]                                                               ;# delete icon and
-            unset ${this}handleIcon($handle) ${this}handleCoordinates($handle)                                       ;# related data
+            delete [set ${this}handleIcon($handle)]                                                                   ;# delete icon
+            unset ${this}handleIcon($handle)
             catch {unset ${this}handleIconCoordinates($handle)}
         }
+        catch {unset ${this}handleCoordinates($handle)}
         foreach {name value} [array get ${this}data viewerHandle,*] {
             if {$value == $handle} {array unset ${this}data $name; break}
         }
@@ -115,13 +116,12 @@ class canvasWindowManager {
     proc iconCoordinates {this path} {
         variable ${this}data
         variable ${this}handleIcon
-        variable ${this}handleCoordinates
 
         set handle [set ${this}data(handle,$path)]
         if {[catch {set icon [set ${this}handleIcon($handle)]}]} {                                                  ;# not minimized
-            return {{} {}}                                                                            ;# list of 2 empty coordinates
+            return [list {} {}]                                                                       ;# list of 2 empty coordinates
         } else {
-            return [$($this,canvas) coords icon($icon)]
+            return [icon::coordinates $icon]
         }
     }
 
@@ -159,7 +159,7 @@ class canvasWindowManager {
         variable ${this}handleIcon
 
         set canvas $($this,canvas)
-        set bounds [bounds $this]                                                                                ;# for current page
+        set bounds [bounds $canvas]                                                                              ;# for current page
         set handles {}
         foreach {name handle} [array get ${this}data handle,*] {
             if {[info exists ${this}handleIcon($handle)]} continue                                       ;# ignore minimized handles
@@ -168,9 +168,12 @@ class canvasWindowManager {
             lappend handles $handle
         }
         set length [llength $handles]
-        if {$length < 2} return                                                                  ;# there can be no next or previous
+        if {$length < 2} {
+            raiseOther $this $next                                                                    ;# try icons and other objects
+            return                                                                        ;# there can be no next or previous handle
+        }
         set handles [lsort -integer $handles]                                                           ;# sort in order of creation
-        set maximum $global::integerMinimum
+        set maximum $global::32BitIntegerMinimum
         set index 0
         foreach handle $handles {
             set level [set ${this}data(relativeStackingLevel,$path($handle))]
@@ -181,13 +184,70 @@ class canvasWindowManager {
             incr index
         }
         if {$next} {
-            if {[incr top] >= $length} {set top 0}                                          ;# eventually circle around to beginning
+            if {[incr top] >= $length} {                                                               ;# circle around to beginning
+                if {[raiseOther $this 1]} return                                       ;# except if there were other objects to show
+                set top 0
+            }
         } else {
-            if {[incr top -1] < 0} {set top end}                                             ;# eventually circle around back to end
+            if {[incr top -1] < 0} {                                                                    ;# circle around back to end
+                if {[raiseOther $this 0]} return                                       ;# except if there were other objects to show
+                set top end
+            }
         }
         handles::stack [lindex $handles $top] raise
     }
 
+    proc raiseOther {this next} {
+        variable ${this}data
+        variable ${this}handleIcon
+
+        set canvas $($this,canvas)
+        set bounds [bounds $canvas]                                                                              ;# for current page
+        set handles {}
+        foreach {name handle} [array get ${this}data handle,*] {
+            if {[catch {set tag icon([set ${this}handleIcon($handle)])}]} continue                                  ;# not minimized
+            if {![intersect [$canvas bbox $tag] $bounds]} continue                                  ;# ignore objects in other pages
+            lappend handles $handle
+        }
+        set tags {}
+        foreach handle [lsort -integer $handles] {                                                      ;# sort in order of creation
+            lappend tags icon([set ${this}handleIcon($handle)])
+        }
+        foreach viewer $canvas::viewer::(list) {                                                       ;# now look at canvas viewers
+            set tag $canvas::viewer::($viewer,tag)
+            if {![intersect [$canvas bbox $tag] $bounds]} continue                                  ;# ignore viewers in other pages
+            lappend tags $tag
+        }
+        if {[llength $tags] == 0} {return 0}                                                                 ;# nothing to highlight
+        if {[info exists ($this,raiseOtherEvent)]} {                                                 ;# an icon is being highlighted
+            set command [lindex [after info $($this,raiseOtherEvent)] 0]
+            after cancel $($this,raiseOtherEvent)                                                            ;# cancel current event
+            uplevel #0 $command                                                                      ;# but still execute its script
+            set index [lsearch -exact $tags $($this,raiseOtherTag)]
+            if {$index < 0} {                                                              ;# object may have disappeared (unlikely)
+                if {$next} {set index 0} else {set index end}
+            } elseif {$next} {
+                if {[incr index] >= [llength $tags]} {unset index}                                    ;# no next object to highlight
+            } else {
+                if {[incr index -1] < 0} {unset index}                                                ;# no next object to highlight
+            }
+        } else {                                                                                          ;# start at extreme object
+            if {$next} {set index 0} else {set index end}
+        }
+        if {![info exists index]} {                                                                   ;# no next object to highlight
+            unset ($this,raiseOtherTag)
+            return 0
+        }
+        set ($this,raiseOtherTag) [lindex $tags $index]
+        foreach {left top right bottom} [$canvas bbox $($this,raiseOtherTag)] {}
+        set highlighter [new highlighter]
+        highlighter::show $highlighter\
+            [expr {[winfo rootx $canvas] + $left}] [expr {[winfo rooty $canvas] + $top}]\
+            [expr {$right - $left}] [expr {$bottom - $top}]
+        set ($this,raiseOtherEvent) [after 1000 "delete $highlighter; unset canvasWindowManager::($this,raiseOtherEvent)"]
+        return 1
+    }
+
     proc minimize {this handle title {xIcon {}} {yIcon {}} {static 0}} {
         variable ${this}handleIcon
         variable ${this}handleCoordinates
@@ -195,15 +255,18 @@ class canvasWindowManager {
 
         foreach {x y} [handles::getGeometry $handle] break                                           ;# retrieve current coordinates
         set ${this}handleCoordinates($handle) [list $x $y]                                                          ;# remember them
-        handles::move $handle $global::integerMinimum $global::integerMinimum                   ;# make window invisible to the user
-        set icon [new icon $($this,canvas) $title "canvasWindowManager::deIconify $this $handle" $static]
+        handles::moveTo $handle $global::32BitIntegerMinimum $global::32BitIntegerMinimum       ;# make window invisible to the user
+        set icon [new icon $($this,canvas) $title -command "canvasWindowManager::deIconify $this $handle"]
+        if {$static} {switched::configure $icon -state disabled} else {switched::configure $icon -state normal}
         if {[string length $xIcon] > 0} {
-            $($this,canvas) move icon($icon) $xIcon $yIcon                                                ;# pre-defined coordinates
+            icon::coordinates $icon $xIcon $yIcon                                                         ;# pre-defined coordinates
+            set ${this}handleIconCoordinates($handle) [list $xIcon $yIcon]                    ;# remember them for next minimization
         } elseif {[info exists ${this}handleIconCoordinates($handle)]} {                                            ;# if available,
-            eval $($this,canvas) move icon($icon) [set ${this}handleIconCoordinates($handle)]       ;# use previous icon coordinates
+            eval icon::coordinates $icon [set ${this}handleIconCoordinates($handle)]                ;# use previous icon coordinates
         } else {
             stowIcon $this $icon
         }
+        switched::configure $icon -color [composite::cget $handle -titlebackground]
         set ${this}handleIcon($handle) $icon
     }
 
@@ -214,10 +277,10 @@ class canvasWindowManager {
 
         set icon [set ${this}handleIcon($handle)]
         if {$icon::($icon,moved)} {                                                                        ;# icon was moved by user
-            set ${this}handleIconCoordinates($handle) [$($this,canvas) coords icon($icon)]               ;# remember its coordinates
+            set ${this}handleIconCoordinates($handle) [icon::coordinates $icon]    ;# remember its coordinates for next minimization
         }
         delete $icon
-        eval handles::move $handle [set ${this}handleCoordinates($handle)]                        ;# place back in original position
+        eval handles::moveTo $handle [set ${this}handleCoordinates($handle)]                      ;# place back in original position
         handles::stack $handle raise                                      ;# but on top of the others so that it can be easily found
         unset ${this}handleIcon($handle) ${this}handleCoordinates($handle)
     }
@@ -226,12 +289,12 @@ class canvasWindowManager {
         set canvas $($this,canvas)
         set padding $global::iconPadding
         ::update idletasks                                                                      ;# needed to get canvas correct size
-        set bounds [bounds $this]                                                                                ;# for current page
+        set bounds [bounds $canvas]                                                                              ;# for current page
         foreach {region(left) region(top) region(right) region(bottom)} $bounds {}
         foreach item [$canvas find all] {
             set index 0
             foreach tag [$canvas gettags $item] {
-                if {[scan $tag icon(%u) index]>0} break                                                             ;# found an icon
+                if {[scan $tag icon(%u) index] > 0} break                                                           ;# found an icon
             }
             if {($index == 0) || ($index == $identifier)} continue                     ;# ignore items other than icons and new icon
             if {![intersect [$canvas bbox icon($index)] $bounds]} continue                            ;# ignore icons in other pages
@@ -243,7 +306,7 @@ class canvasWindowManager {
         }
         set coordinates [lsort -integer -index 0 $coordinates]                                                  ;# order by abscissa
         foreach {left top right bottom} [$canvas bbox icon($identifier)] {}                        ;# all icons have the same height
-        set height [expr {$bottom - $top + (2 * $padding)}]  ;# icons are initially placed is slices with a reasonable padding value
+        set height [expr {$bottom - $top + (2 * $padding)}]  ;# icons are initially placed in slices with a reasonable padding value
         set width [expr {$right - $left + (2 * $padding)}]
         set maximum $region(bottom)
         while {[set minimum [expr {$maximum - $height}]] >= 0} {                 ;# look for room in slices starting from the bottom
@@ -276,59 +339,10 @@ class canvasWindowManager {
             set position(x) [expr {$region(left)}]
             set position(y) [expr {$region(bottom) - $height}]
         }
-        foreach {x y} [$canvas coords icon($identifier)] break
-        $canvas move icon($identifier) [expr {$position(x) - $x + $padding}] [expr {$position(y) - $y + $padding}]
-    }
-
-    proc rearrangeIcons {this width height} {                           ;# make sure icons are always visible when canvas is resized
-        set canvas $($this,canvas)
-        foreach {left top right bottom} [$canvas cget -scrollregion] {}
-        set width [maximum $width [expr {$right - $left}]]                     ;# in case canvas was sized pass its scrolling region
-        set height [maximum $height [expr {$bottom - $top}]]
-        foreach item [$canvas find all] {
-            set identifier 0
-            foreach tag [$canvas gettags $item] {
-                if {[scan $tag icon(%u) identifier] > 0} break                                                      ;# found an icon
-            }
-            if {$identifier == 0} continue                                                                    ;# item is not an icon
-            # find the page that the icon belongs to and calculate its bounds:
-            foreach {left top} [pages::closestPageTopLeftCorner [lindex [$canvas coords icon($identifier)] 0]] {}
-            set right [expr {$left + $width}]
-            set bottom [expr {$top + $height}]
-            foreach {icon(left) icon(top) icon(right) icon(bottom)} [$canvas bbox icon($identifier)] {}
-            # move the icon inside its page boundaries if necessary:
-            if {$icon(right) > $right} {$canvas move icon($identifier) [expr {$right - $icon(right)}] 0}
-            if {$icon(bottom) > $bottom} {$canvas move icon($identifier) 0 [expr {$bottom - $icon(bottom)}]}
-            if {$icon(left) < $left} {$canvas move icon($identifier) [expr {$left - $icon(left)}] 0}
-            if {$icon(top) < $top} {$canvas move icon($identifier) 0 [expr {$top - $icon(top)}]}
-        }
-    }
-
-    proc bounds {this} {                                            ;# returns the current canvas bounds (changes when page changes)
-        set canvas $($this,canvas)
-        foreach {left top dummy dummy} [$canvas cget -scrollregion] {}                             ;# current page upper left corner
-        # actual size may be larger than user specified size:
-        return [list\
-            $left $top\
-            [expr {$left + [maximum $global::canvasWidth [winfo width $canvas]]}]\
-            [expr {$top + [maximum $global::canvasHeight [winfo height $canvas]]}]\
-        ]
-    }
-
-    proc currentPageEmpty {this} {
-        variable ${this}data
-        variable ${this}handleIcon
-
-        set canvas $($this,canvas)
-        set bounds [bounds $this]                                                                                ;# for current page
-        foreach {name handle} [array get ${this}data handle,*] {
-            set tag $handles::($handle,item)
-            catch {set tag icon([set ${this}handleIcon($handle)])}                                        ;# handle may be minimized
-            if {[intersect [$canvas bbox $tag] $bounds]} {                                                      ;# item lies in page
-                return 0
-            }
-        }
-        return 1
+        # place top left corner of icon at calculated coordinates
+        foreach {x y} [icon::coordinates $identifier] {}
+        foreach {left top right bottom} [$canvas bbox icon($identifier)] {}
+        icon::coordinates $identifier [expr {$position(x) + ($x - $left)}] [expr {$position(y) + ($y - $top)}]
     }
 
     # return the page where the viewer or table lies or nothing if failure, such as the object not being managed
@@ -379,4 +393,74 @@ class canvasWindowManager {
         return $values
     }
 
+    proc validateDrag {this x y} {
+        variable ${this}handleIcon
+
+        set drag $($this,drag)
+        set canvas $($this,canvas)
+        # reset all formats as they are dynamically defined in order to handle different canvas object types
+        foreach format [dragSite::provide $drag] {dragSite::provide $drag $format {}}
+        foreach tag [$canvas gettags current] {
+            if {[set icon [icon::fromTag $tag]] > 0} {                                            ;# a minimized object was selected
+                set handle 0
+                foreach {name value} [array get ${this}handleIcon] {
+                    if {$value == $icon} {set handle $name; break}
+                }
+                if {$handle == 0} {error {could not find a handle for selected icon}}
+                dragSite::provide $drag MINIMIZED "dragEcho [list [list $this $handle $icon]]"     ;# allow dropping in another page
+                return 1
+            }
+        }
+        return [canvas::viewer::validateDrag $canvas $x $y]                               ;# see if a canvas viewer is being dragged
+    }
+
+    proc dragData {this format} {                                     ;# note: minimized window manager objects are handled directly
+        return [canvas::viewer::dragData $($this,canvas) $format]
+    }
+
+    proc moveIconToPage {list x y} {                                                             ;# list of manager, handle and icon
+        foreach {manager handle icon} $list {}
+        variable ${manager}handleCoordinates
+        set ${manager}handleCoordinates($handle) [list $x $y]                             ;# so that it deiconifies in the same page
+        icon::coordinates $icon $x $y
+    }
+
+    proc moveHandlesToPage {handles x y} {
+        set manager $global::windowManager
+        variable ${manager}handleCoordinates
+        variable ${manager}handleIconCoordinates
+        set ${manager}handleCoordinates($handles) [list $x $y]
+        catch {unset ${manager}handleIconCoordinates($handles)}                     ;# previous icon coordinates are no longer valid
+        handles::moveTo $handles $x $y
+    }
+
+    proc moveAll {this xMaximum} {
+        variable ${this}handleIcon
+
+        foreach handle [handles $this] {
+            if {![catch {set icon [set ${this}handleIcon($handle)]}]} {                                                 ;# minimized
+                foreach {x y} [icon::coordinates $icon] {}
+                if {$x >= $xMaximum} {
+                    moveIconToPage [list $this $handle $icon] [expr {round($x) % $xMaximum}] $y
+                }
+            } else {
+                foreach {x y} [canvasWindowManager::handles::getGeometry $handle] break
+                if {$x >= $xMaximum} {
+                    moveHandlesToPage $handle [expr {round($x) % $xMaximum}] $y
+                }
+            }
+        }
+    }
+
+    proc color {this path value} {                                                      ;# color the window manager part in some way
+        variable ${this}data
+        variable ${this}handleIcon
+
+        set handle [set ${this}data(handle,$path)]
+        composite::configure $handle -titlebackground $value
+        if {![catch {set icon [set ${this}handleIcon($handle)]}]} {                                                     ;# minimized
+            switched::configure $icon -color $value
+        }
+    }
+
 }
