@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: datapie.tcl,v 2.67 2004/01/01 11:39:06 jfontain Exp $
+# $Id: datapie.tcl,v 2.77 2005/02/07 21:51:35 jfontain Exp $
 
 
 class dataPieChart {
@@ -19,7 +19,7 @@ class dataPieChart {
         set width [$path cget -borderwidth]                          ;# place entry in top left corner, possibly over existing total
         $menu add command -label [mc Total]... -state disabled -command "
             viewer::limitEntry $this $path nw $width [expr {$width - 1}] -total total $(smallFont)\
-                {dataPieChart::totalEntered $this}
+                {dataPieChart::totalEntered $this} {} unsigned
         "
         menuContextHelp::set $($this,help) 0 [mc {set total value or automatic scaling}]
         if {!$global::readOnly} {
@@ -158,7 +158,7 @@ class dataPieChart {
                 incr ($this,nextCellIndex)                                                            ;# get ready for upcoming cell
             }
         }
-        set label [viewer::label $array $row $column]
+        foreach {label incomplete} [viewer::label $array $row $column] {}
         if {![info exists color]} {set color [viewer::getDisplayColor $cell]}
         if {$global::readOnly} {
             set slice [new slice $($this,pie) {} -background $color -label $label]
@@ -168,7 +168,7 @@ class dataPieChart {
         lappend ($this,slices) $slice
         switched::configure $slice -deletecommand "dataPieChart::deletedSlice $this $array $slice"  ;# keep track of slice existence
         set ($this,cell,$slice) $cell
-        if {[string first ? $label] >= 0} {                                                        ;# label cannot be determined yet
+        if {$incomplete} {                                                                         ;# label cannot be determined yet
             set ($this,relabel,$slice) {}
         }
         switched::configure $($this,drop) -state normal
@@ -183,9 +183,9 @@ class dataPieChart {
         if {[string length $sum] == 0} {set sum 0}
         if {![catch {set cell $($this,totalCell)}]} {
             if {[catch {set sum [set $cell]}]} {
-                composite::configure $($this,total) -text ?
+                composite::configure $($this,total) -image $viewer::(rightDarkGrayArrow) -text ?
             } else {
-                composite::configure $($this,total) -text $sum
+                composite::configure $($this,total) -image $viewer::(rightRedArrow) -text $sum
                 if {[info exists ($this,relabelTip)]} {updateTip $this}
                 if {[string equal $sum ?]} {set sum 0}                                       ;# can happen in database browsing mode
             }
@@ -203,14 +203,6 @@ class dataPieChart {
             if {[catch {set value [set $cell]}] || [string equal $value ?] || ($sum == 0)} {
                 slice::update $slice 0 ?                      ;# cell may no longer exist, be void, also catch divide by zero errors
             } else {
-                if {[info exists ($this,relabel,$slice)]} {                                ;# if label is not yet defined, update it
-                    viewer::parse $cell array row column type
-                    set label [viewer::label $array $row $column]
-                    switched::configure $slice -label $label
-                    if {[string first ? $label] < 0} {                                               ;# label now completely defined
-                        unset ($this,relabel,$slice)
-                    }
-                }
                 if {[string is double -strict $value]} {
                     set value [expr {abs($value) / $sum}]                                           ;# negative values make no sense
                     slice::update $slice $value "[format %.1f [expr {$value * 100}]] %"
@@ -218,6 +210,14 @@ class dataPieChart {
                     slice::update $slice 0 $value
                 }
             }
+            if {[info exists ($this,relabel,$slice)] && [info exists $cell]} {             ;# if label is not yet defined, update it
+                viewer::parse $cell array row column type
+                foreach {label incomplete} [viewer::label $array $row $column] {}
+                switched::configure $slice -label $label
+                if {!$incomplete} {
+                    unset ($this,relabel,$slice)                                                     ;# label now completely defined
+                }
+            }
         }
         pie::update $($this,pie)                           ;# now that all slices are updated, pie automatic update being turned off
     }
@@ -243,7 +243,7 @@ class dataPieChart {
         return $cells
     }
 
-    proc cells {this} {
+    proc cells {this} {                                                               ;# note: always return cells in the same order
         return [cellsFromSlices $this $($this,slices)]
     }
 
@@ -292,7 +292,7 @@ class dataPieChart {
 
     proc totalCellDrop {this} {
         if {[llength $dragSite::data(DATACELLS)] != 1} {
-            lifoLabel::flash $global::messenger {only one data cell can be used for total value}
+            lifoLabel::flash $global::messenger [mc {only one data cell can be used for total value}]
             bell
             return
         }
@@ -311,14 +311,12 @@ class dataPieChart {
             viewer::unregisterTrace $this $array                                                                ;# no longer monitor
             unset ($this,totalCell)
         }
+        composite::configure $($this,total) -text {} -image {}
         if {[string length $cell] == 0} {
-            composite::configure $($this,total) -text {} -image {}
             set array {}
         } else {
-            composite::configure $this -total {}
             viewer::parse $cell array ignore ignore ignore
             viewer::registerTrace $this $array
-            composite::configure $($this,total) -image $viewer::(rightRedArrow)
             set ($this,totalCell) $cell
         }
         if {[info exists ($this,pie)]} {
@@ -335,7 +333,7 @@ class dataPieChart {
     proc initializationConfiguration {this} {
         set colors {}
         foreach slice $($this,slices) {
-            lappend colors [switched::cget $slice -background]
+            lappend colors [switched::cget $slice -background]                       ;# note: always return colors in the same order
         }
         return [list -total $composite::($this,-total) -totalcell $composite::($this,-totalcell) -cellcolors $colors]
     }
@@ -350,17 +348,17 @@ class dataPieChart {
     proc updateTip {this} {
         if {[info exists ($this,totalCell)]} {
             viewer::parse $($this,totalCell) array row column type
-            set label [viewer::label $array $row $column]
-            if {[string first ? $label] < 0} {                                                           ;# label completely defined
-                catch {unset ($this,relabelTip)}
-            } else {
+            foreach {label incomplete} [viewer::label $array $row $column] {}
+            if {$incomplete} {
                 set ($this,relabelTip) {}                                                          ;# label cannot be determined yet
+            } else {                                                                                     ;# label completely defined
+                catch {unset ($this,relabelTip)}
             }
-            set text "total cell: $label"
+            set text [format [mc {total cell: %s}] $label]
         } elseif {[string length $composite::($this,-total)] > 0} {
-            set text "fixed total value set to $composite::($this,-total)"
+            set text [format [mc {fixed total value set to %s}] $composite::($this,-total)]
         } else {
-            set text {set fixed total value or drop cell as dynamic total value here}
+            set text [mc {set fixed total value or drop cell as dynamic total value here}]
         }
         foreach tip $($this,tips) {switched::configure $tip -text $text}
     }
@@ -447,7 +445,7 @@ class dataPieChart {
                 return 0                                                                                         ;# no current slice
             } else {
                 return $(this,$slice)                                                      ;# return object corresponding with slice
-            }            
+            }
         }
 
     }
@@ -476,7 +474,7 @@ class data2DPieChart {
     }
 
     proc message {this} {
-        return "2D pie chart:\ndrop data cell(s)"
+        return [mc "2D pie chart:\ndrop data cell(s)"]
     }
 
 }
@@ -503,7 +501,7 @@ class data3DPieChart {
     }
 
     proc message {this} {
-        return "3D pie chart:\ndrop data cell(s)"
+        return [mc "3D pie chart:\ndrop data cell(s)"]
     }
 
 }
