@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: dbgui.tcl,v 1.51 2004/01/01 11:39:06 jfontain Exp $
+# $Id: dbgui.tcl,v 1.55 2005/01/02 00:45:07 jfontain Exp $
 
 
 class database {
@@ -41,7 +41,7 @@ class database {
 
         proc dialog {this parentPath args} switched {$args} {
             set dialog [new dialogBox .\
-                -buttons hoc -default o -title [mc {moodss: Database Instances}] -otherbuttons SQL\
+                -buttons hoc -default o -title [mc {moodss: Database instances}] -otherbuttons SQL\
                 -helpcommand {generalHelpWindow #menus.file.database.load} -x [winfo pointerx .] -y [winfo pointery .]\
                 -grab release -command "database::dialog::validated $this" -deletecommand "delete $this"\
             ]
@@ -49,7 +49,7 @@ class database {
             composite::configure $dialog ok -state disabled                        ;# OK button disabled until proper cell selection
             lappend ($this,tips) [linkedHelpWidgetTip $composite::($dialog,help,path)]\
                 [new widgetTip -path $composite::($dialog,SQL,path)\
-                     -text {toggles the display of the SQL query that can be used to retrieve the data cell history}\
+                     -text [mc {toggles the display of the SQL query that can be used to retrieve the data cell history}]\
                 ]
             set frame [frame $widget::($dialog,path).frame]
             set scroll [new scroll tree $frame]
@@ -67,15 +67,15 @@ class database {
             lappend ($this,objects) $scroll
 
             set range [frame $frame.range]
-            set label [label $range.fromLabel -font $font::(mediumBold) -text From:]
+            set label [label $range.fromLabel -font $font::(mediumBold) -text [mc from:]]
             grid $label -row 0 -column 0 -sticky w
             set ($this,from) [label $range.from -font $font::(mediumNormal)]
             grid $($this,from) -row 0 -column 1 -sticky w -padx 2
-            set label [label $range.toLabel -font $font::(mediumBold) -text To:]
+            set label [label $range.toLabel -font $font::(mediumBold) -text [mc to:]]
             grid $label -row 1 -column 0 -sticky w
             set ($this,to) [label $range.to -font $font::(mediumNormal)]
             grid $($this,to) -row 1 -column 1 -sticky w -padx 2
-            lappend ($this,tips) [new widgetTip -path $range -text {selected item database time range}]
+            lappend ($this,tips) [new widgetTip -path $range -text [mc {selected item database time range}]]
             grid columnconfigure $range 1 -weight 1
             grid $range -row 1 -sticky ew
 
@@ -133,7 +133,7 @@ class database {
         }
 
         proc refresh {this} {
-            lifoLabel::push $global::messenger {retrieving module instances from database...}
+            lifoLabel::push $global::messenger [mc {retrieving module instances from database...}]
             busy 1 [list . $widget::($($this,dialog),path)]
             set database $global::database
             set tree $($this,tree)
@@ -191,8 +191,8 @@ class database {
                         $tree itemconfigure $new -font $font::(mediumNormal)
                     }
                     set ($this,instanceData,$new) [list $instance $module $data(identifier,$instance) $arguments]
-                    lappend ($this,nodeTips) [new widgetTip\
-                        -path $canvas -itemortag n:$new -text "instance of module $module version $data(version,$instance)"\
+                    lappend ($this,nodeTips) [new widgetTip -path $canvas -itemortag n:$new\
+                        -text [format [mc {instance of module %1$s version %2$s}] $module $data(version,$instance)]\
                     ]                                           ;# note: the node name (n:*) comes from the BWidget tree source code
                 }
             }
@@ -343,11 +343,11 @@ class database {
             }
             set singleton $this
             set dialog [new dialogBox $parentPath\
-                -buttons hoc -default o -title [mc {moodss: Database History Range}] -x [winfo pointerx .] -y [winfo pointery .]\
+                -buttons hoc -default o -title [mc {moodss: Database history range}] -x [winfo pointerx .] -y [winfo pointery .]\
                 -helpcommand {generalHelpWindow #menus.view.database.range} -deletecommand "delete $this" -grab release\
                 -command "database::rangeDialog::apply $this 1" -die 0 -otherbuttons apply\
             ]
-            composite::configure $dialog apply -text Apply -command "database::rangeDialog::apply $this 0"
+            composite::configure $dialog apply -text [mc Apply] -command "database::rangeDialog::apply $this 0"
             set ($this,tip) [linkedHelpWidgetTip $composite::($dialog,help,path)]
             set frame [frame $widget::($dialog,path).frame]
             set message [message $frame.message\
@@ -433,18 +433,6 @@ class database {
 
         class input {
 
-            variable days [mc {Sunday Monday Tuesday Wednesday Thursday Friday Saturday}]
-            variable daysWidth 0
-            foreach value $days {
-                ::set value [string length $value]
-                if {$value > $daysWidth} {::set daysWidth $value}
-            }
-            variable months [mc {January February March April May June July August September October November December}]
-            variable monthsWidth 0
-            foreach value $months {
-                ::set value [string length $value]
-                if {$value > $monthsWidth} {::set monthsWidth $value}
-            }
             variable hours
             for {::set value 0} {$value < 24} {incr value} {
                 lappend hours [format %02u $value]
@@ -468,6 +456,22 @@ class database {
                 variable minimum
                 variable maximum
 
+                if {![info exists days]} {
+                    ::set days [mc {Sunday Monday Tuesday Wednesday Thursday Friday Saturday}]
+                    ::set daysWidth 0
+                    foreach value $days {
+                        ::set value [string length $value]
+                        if {$value > $daysWidth} {::set daysWidth $value}
+                    }
+                }
+                if {![info exists months]} {
+                    ::set months [mc {January February March April May June July August September October November December}]
+                    ::set monthsWidth 0
+                    foreach value $months {
+                        ::set value [string length $value]
+                        if {$value > $monthsWidth} {::set monthsWidth $value}
+                    }
+                }
                 ::set path $widget::($this,path)
                 ::set delay 200                                                                      ;# repeat delay in milliseconds
                 if {[package vcompare $::tcl_version 8.4] < 0} {
@@ -505,7 +509,7 @@ class database {
                             -repeatdelay $delay -command "database::rangeDialog::input::increment $this %d 1"\
                         ] seconds
                     foreach entry {hours minutes seconds} {                                           ;# filter on positive integers
-                        setupEntryValidation $composite::($composite::($this,$entry),entry,path) {{checkUnsignedInteger %P}}
+                        setupEntryValidation $composite::($composite::($this,$entry),entry,path) {{check31BitUnsignedInteger %P}}
                     }
                 } else {                                                                            ;# use native widget if possible
                     composite::manage $this\
@@ -542,7 +546,7 @@ class database {
                             -repeatinterval $delay -command "database::rangeDialog::input::increment $this %d 1 %s"\
                         ] seconds
                     foreach entry {hours minutes seconds} {                                           ;# filter on positive integers
-                        setupEntryValidation $composite::($this,$entry,path) {{checkUnsignedInteger %P}}
+                        setupEntryValidation $composite::($this,$entry,path) {{check31BitUnsignedInteger %P}}
                     }
                 }
                 pack $composite::($this,dayOfWeek,path) $composite::($this,day,path) $composite::($this,month,path)\
