@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: diskstats.tcl,v 1.4 2004/01/25 22:59:17 jfontain Exp $
+# $Id: diskstats.tcl,v 1.15 2005/01/13 22:01:35 jfontain Exp $
 
 
-package provide diskstats [lindex {$Revision: 1.4 $} 1]
+package provide diskstats [lindex {$Revision: 1.15 $} 1]
 package require network 1
 package require stooop 4.1
 namespace import stooop::*
@@ -31,12 +31,11 @@ namespace eval diskstats {
         5,label writing 5,type real 5,message {time spent writing in % during the last poll period (for disks only)}
         6,label busy 6,type real 6,message {time spent doing I/O in % during the last poll period (for disks only)}
         sort {0 increasing}
-        switches {-a 0 --all 0 -C 0 --daemon 0 -i 1 --partitions 0 -r 1 --remote 1}
+        persistent 1
+        switches {-a 0 --all 0 -C 0 --daemon 0 -i 1 -p 1 --partitions 0 -r 1 --remote 1}
     }
-
-    variable nextIndex 0                                                                                ;# unique index for new rows
     set file [open diskstats.htm]
-    set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
+    set data(helpText) [::read $file]                                                         ;# initialize HTML help data from file
     close $file
     unset file
 
@@ -73,18 +72,15 @@ namespace eval diskstats {
         set data(identifier) diskstats($remote(host))
         # important: pack data in a single line using special control separator characters
         # ignore errors caused by missing diskstats file in 2.4 kernel
-        set remote(command) {[ -r /proc/diskstats ] && echo -e "$(< /proc/partitions)\n$(< /proc/diskstats)" | tr '\n' '\v' || echo "$(< /proc/partitions)" | tr '\n' '\v'}
+        set remote(command) {[ -r /proc/diskstats ] && cat /proc/partitions /proc/diskstats 2>&1 | tr '\n' '\v' || cat /proc/partitions 2>&1 | tr '\n' '\v'}
         if {[string equal $::tcl_platform(platform) unix]} {
             if {$remote(rsh)} {
                 set command "rsh -n -l $remote(user) $remote(host) {$remote(command)}"
             } else {
                 set command ssh
-                if {[info exists options(-C)]} {                                                                 ;# data compression
-                    append command { -C}
-                }
-                if {[info exists options(-i)]} {                                                                    ;# identity file
-                    append command " -i \"$options(-i)\""
-                }
+                if {[info exists options(-C)]} {append command { -C}}                                            ;# data compression
+                if {[info exists options(-i)]} {append command " -i \"$options(-i)\""}                              ;# identity file
+                if {[info exists options(-p)]} {append command " -p $options(-p)"}                                           ;# port
                 append command " -T -l $remote(user) $remote(host)"
             }
         } else {                                                                                                          ;# windows
@@ -114,20 +110,16 @@ namespace eval diskstats {
     proc update {} {
         variable remote
         variable local
-        variable disks
-        variable partitions
 
         if {[info exists remote]} {
-            if {$remote(busy)} {
-                return                                                              ;# core invocation while waiting for remote data
-            }
+            if {$remote(busy)} return                                               ;# core invocation while waiting for remote data
+            set remote(busy) 1
             if {[lineTask::end $remote(task)]} {                                                           ;# rsh or ssh daemon mode
                 lineTask::begin $remote(task)                       ;# note: for rsh, shell and command are restarted here each time
             }
             if {!$remote(rsh)} {
                 lineTask::write $remote(task) $remote(command)             ;# start data retrieval by sending command to remote side
             }
-            set remote(busy) 1
         } else {
             seek $local(partitions) 0                                                               ;# rewind before retrieving data
             if {[info exists local(diskstats)]} {                                                                      ;# 2.6 kernel
@@ -142,8 +134,6 @@ namespace eval diskstats {
     proc process {lines} {                                         ;# process partitions and diskstats data lines and update display
         variable data
         variable last
-        variable nextIndex
-        variable index
         variable disks
         variable partitions
 
@@ -178,17 +168,8 @@ namespace eval diskstats {
                 set partition [string match {*[0-9]} $name]                     ;# if names ends by a number, it must be a partition
                 if {($partition && !$partitions) || (!$partition && !$disks)} continue                             ;# skip this line
             }
-            if {[catch {set row $index($name)}]} {                                                                      ;# new entry
-                set row [set index($name) $nextIndex]
-                incr nextIndex
-                set data($row,0) $name                                                                     ;# initialize static data
-                set data($row,1) ?                                                                         ;# and yet unknown values
-                set data($row,2) ?
-                set data($row,3) ?
-                set data($row,4) ?
-                set data($row,5) ?
-                set data($row,6) ?
-            }
+            set row [format %u [expr {($major << 20) | $minor}]]
+            set data($row,0) $name
             if {[info exist blocks]} {                                                                       ;# partitions file line
                 set value [expr {$blocks / 1024.0}]                           ;# block size is always 1024 bytes in the Linux kernel
                 if {$value < 100} {                                                                       ;# less than 100 megabytes
@@ -196,7 +177,10 @@ namespace eval diskstats {
                 } else {
                     set data($row,1) [expr {round($value)}]
                 }
+            } elseif {![info exists data($row,1)]} {                             ;# do not erase existing value from partitions file
+                set data($row,1) ?
             }
+            set data($row,2) ?; set data($row,3) ?
             # notes: counters wrapping creates no problems since differences between current and last values are forced as pure
             # integers subtractions, assume 512 bytes sector size (see iostat command and source), time data is in milliseconds
             if {[info exist rsect]} {                                                 ;# line with read and write minimal statistics
@@ -207,6 +191,7 @@ namespace eval diskstats {
                 set last($row,rsect) $rsect
                 set last($row,wsect) $wsect
             }
+            set data($row,4) ?; set data($row,5) ?; set data($row,6) ?
             if {[info exist ruse]} {             ;# this is a 2.6 kernel disk entry or an enabled 2.4 kernel disk or partition entry
                 if {[info exists last($row,ruse)]} {                                                    ;# previous poll data exists
                     set data($row,4) [format %.1f [expr {int($ruse - $last($row,ruse)) / $period / 10.0}]]                   ;# in %
@@ -217,14 +202,18 @@ namespace eval diskstats {
                 set last($row,wuse) $wuse
                 set last($row,use) $use
             }
-            set current($name) {}
+            set current($row) {}
         }
-        foreach {name row} [array get index] {                                                        ;# cleanup disappeared entries
-            if {[info exists current($name)]} continue
-            array unset data $row,\[0-9\]*                                                              ;# remove obsolete data rows
-            unset index($name)
+        foreach name [array names data *,0] {                                                         ;# cleanup disappeared entries
+            set row [lindex [split $name ,] 0]
+            if {![info exists current($row)]} {array unset data $row,\[0-9\]*}
         }
         set last(clock) $clock
+        if {![info exists current] && ([string length [lindex $lines 0]] > 0)} {
+            set message "invalid data: [lindex $lines 0]"
+            if {[llength $lines] > 1} {append message "..."}
+            flashMessage $message
+        }
         incr data(updates)
     }
 
@@ -243,7 +232,7 @@ namespace eval diskstats {
                 set message "timeout on remote host: $remote(host)"
             }
         }
-        # note: is case of an unexpected event, task insures that line is empty
+        # note: in case of an unexpected event, task insures that line is empty
         if {[info exists message]} {
             flashMessage $message
         }
