@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: drag.tcl,v 2.19 2004/01/01 11:39:06 jfontain Exp $
+# $Id: drag.tcl,v 2.25 2005/01/02 00:45:07 jfontain Exp $
 
 class dragSite {                                          ;# make a Tk widget a drag site with multiple formats support for its data
 
@@ -65,15 +65,19 @@ class dragSite {                                          ;# make a Tk widget a
 
     proc set-validcommand {this value} {}       ;# command is invoked with x and y hit coordinates for widget, must return a boolean
 
-    proc provide {this format {command 0}} {               ;# public procedure to make new formats available or unavailable for data
+    # public procedure to make new formats available or unavailable for data (can be invoked up to and within the validate command)
+    proc provide {this {format {}} {command ?}} {
         variable ${this}provider
 
+        if {[string length $format] == 0} {                                 ;# return existing formats for which there is a provider
+            return [array names ${this}provider]
+        }
         switch $command {
-            0 {
+            ? {
                 return [set ${this}provider($format)]                                ;# return existing command for specified format
             }
             {} {
-                unset ${this}provider($format)                                       ;# remove existing command for specified format
+                catch {unset ${this}provider($format)}                               ;# remove existing command for specified format
             }
             default {
                 set ${this}provider($format) $command                                            ;# set command for specified format
@@ -91,7 +95,7 @@ class dragSite {                                          ;# make a Tk widget a
             grab $(grabber)                                                                       ;# drag cursor is used from now on
             update idletasks
         }
-        createFrame
+        set (highlight) [new highlighter]
         $(grabber) configure -cursor $(out)
         update idletasks                                                                   ;# note: drop code must not invoke update
         # place smaller regions first so that if several regions come from the same window, embedded regions can be selected:
@@ -123,12 +127,14 @@ class dragSite {                                          ;# make a Tk widget a
         foreach {site path left top right bottom} [dropSite $xRoot $yRoot] {}
         if {$site == $(lastSite)} {                                                           ;# in the same drop site or in no site
             return                                                                                       ;# no change, nothing to do
-        } elseif {($site == 0) || [contains $switched::($site,-path) $switched::($this,-path)]} {
-            hideFrame        ;# no longer in a drop site (if drag site itself is also a drop site, it is not considered to be valid)
+        } elseif {($site == 0) || [string equal $switched::($site,-path) $switched::($this,-path)]} {
+            # no longer in a drop site (if drag site itself is also a drop site, it is not considered to be valid)
+            highlighter::hide $(highlight)
             $(grabber) configure -cursor $(out)
             update idletasks
         } else {
-            showFrame [expr {$left - 1}] [expr {$top - 1}] [expr {$right - $left + 2}] [expr {$bottom - $top + 2}]
+            highlighter::show $(highlight)\
+                [expr {$left - 1}] [expr {$top - 1}] [expr {$right - $left + 2}] [expr {$bottom - $top + 2}]
             $(grabber) configure -cursor $(in)
             update idletasks
         }
@@ -148,13 +154,13 @@ class dragSite {                                          ;# make a Tk widget a
             bindings::set $($this,bindings) <ButtonRelease-1> {}
             bindings::set $($this,bindings) <Button1-Motion> {}
         }
-        deleteFrame
+        delete $(highlight); unset (highlight)
         $(grabber) configure -cursor $(out)
         update idletasks
         unset (lastSite)
         foreach {site path left top right bottom} [dropSite $xRoot $yRoot] {}
         unset (dropRegions)
-        if {($site == 0) || [contains $switched::($site,-path) $switched::($this,-path)]} {
+        if {($site == 0) || [string equal $switched::($site,-path) $switched::($this,-path)]} {
             return                                                               ;# no point in being able to drop data in drag site
         }
         foreach format [switched::cget $site -formats] {      ;# copy formatted data into data array so that drop site can access it
@@ -187,50 +193,12 @@ class dragSite {                                          ;# make a Tk widget a
         set (X) [winfo pointerx .]
         set (Y) [winfo pointery .]
         if {\
-            ([string length $command] > 0) && \
+            ([string length $command] > 0) &&\
             ![uplevel #0 $command [expr {$(X) - [winfo rootx $path]}] [expr {$(Y) - [winfo rooty $path]}]]\
         } return
         bindings::set $($this,bindings) <Button1-Motion> "dragSite::start $this %X %Y"
     }
 
-    proc createFrame {} {
-        # for the highlighting frame, use 4 border frames to make it hollow, with toplevel objects to make sure paths are unique
-        foreach side {left top right bottom} {
-            set ($side) [new toplevel . -background {} -highlightthickness 1 -highlightbackground black]
-            wm withdraw $widget::($($side),path)
-            wm overrideredirect $widget::($($side),path) 1
-        }
-    }
-
-    proc deleteFrame {} {
-        delete $(left) $(top) $(right) $(bottom)
-    }
-
-    proc hideFrame {} {
-        foreach side {left top right bottom} {
-            wm withdraw $widget::($($side),path)
-        }
-    }
-
-    proc showFrame {x y width height} {
-        set path $widget::($(left),path)
-        $path configure -width 1 -height $height
-        wm geometry $path +$x+$y
-        wm deiconify $path
-        set path $widget::($(top),path)
-        $path configure -width $width -height 1
-        wm geometry $path +$x+$y
-        wm deiconify $path
-        set path $widget::($(right),path)
-        $path configure -width 1 -height $height
-        wm geometry $path +[expr {$x + $width}]+$y
-        wm deiconify $path
-        set path $widget::($(bottom),path)
-        $path configure -width [expr {$width + 1}] -height 1          ;# extend width by 1 pixel to fill hole in bottom right corner
-        wm geometry $path +$x+[expr {$y + $height}]
-        wm deiconify $path
-    }
-
     proc smaller {region1 region2} {              ;# compare 2 regions (returns -1 if first smaller than second, 0 if equal, else 1)
         foreach {site container left top right bottom} $region1 {}
         set area [expr {($right - $left) * ($bottom - $top)}]
