@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: freetext.tcl,v 2.42 2004/01/01 11:39:06 jfontain Exp $
+# $Id: freetext.tcl,v 2.52 2005/01/02 00:45:07 jfontain Exp $
 
 
 class freeText {
@@ -20,27 +20,39 @@ class freeText {
         $path tag configure bolditalic -font $font::(mediumBoldItalic)
         $path tag configure underline -underline 1
         $path tag configure overstrike -overstrike 1
-        set bindings [new bindings $path 0]
-        # break from class bindings to avoid potentially harmful code and do not fail when there is no selection:
-        bindings::set $bindings <Control-b> "catch {$path tag add bold sel.first sel.last; freeText::mergeBoldItalic $path}; break"
-        bindings::set $bindings <Control-i>\
-            "catch {$path tag add italic sel.first sel.last; freeText::mergeBoldItalic $path}; break"
-        bindings::set $bindings <Control-o> "catch {$path tag add overstrike sel.first sel.last}; break"
-        bindings::set $bindings <Control-u> "catch {$path tag add underline sel.first sel.last}; break"
-        bindings::set $bindings <Control-r>\
-            "catch {foreach name {bold italic bolditalic overstrike underline} {$path tag remove \$name sel.first sel.last}}; break"
-        set ($this,bindings) $bindings
-        set ($this,tip) [new widgetTip -path $path -text\
-            {selection formatting Control keys: B(old), I(talic), U(nderline), O(verstrike), R(eset)}\
-        ]
+        if {$global::readOnly} {
+            $path configure -state disabled
+        } else {
+            set bindings [new bindings $path 0]
+            # break from class bindings to avoid potentially harmful code and do not fail when there is no selection:
+            bindings::set $bindings <Control-b>\
+                "catch {$path tag add bold sel.first sel.last; freeText::mergeBoldItalic $path}; break"
+            bindings::set $bindings <Control-i>\
+                "catch {$path tag add italic sel.first sel.last; freeText::mergeBoldItalic $path}; break"
+            bindings::set $bindings <Control-o> "catch {$path tag add overstrike sel.first sel.last}; break"
+            bindings::set $bindings <Control-u> "catch {$path tag add underline sel.first sel.last}; break"
+            bindings::set $bindings <Control-r> "
+                catch {foreach name {bold italic bolditalic overstrike underline} {$path tag remove \$name sel.first sel.last}}
+                break
+            "
+            set ($this,bindings) $bindings
+            set ($this,tip) [new widgetTip -path $path -text\
+                [mc "selection formatting Control keys:\nB(old), I(talic), U(nderline), O(verstrike), R(eset)"]\
+            ]
+        }
         composite::complete $this
         initializeTags $this
-        centerMessage $path "free text:\ndrop data cell(s), input text" $viewer::(background) $global::viewerMessageColor
-        after 2000 "centerMessage $path {}"                                                          ;# remove message after a while
+        if {[string length $composite::($this,-endtext)] == 0} {                                             ;# only in empty viewer
+            centerMessage $path [mc "free text:\ndrop data cell(s), input text"] $viewer::(background) $global::viewerMessageColor
+            set ($this,event) [after 2000 "centerMessage $path {}; unset freeText::($this,event)"]   ;# remove message after a while
+        }
     }
 
     proc ~freeText {this} {
-        delete $($this,bindings) $($this,tip)
+        catch {after cancel $($this,event)}
+        if {[info exists ($this,bindings)]} {
+            delete $($this,bindings) $($this,tip)
+        }
         if {[info exists ($this,drag)]} {
             delete $($this,drag)
         }
@@ -55,11 +67,16 @@ class freeText {
 
     proc iconData {} {
         return {
-            R0lGODdhJAAkAIQAAPj8+Hh4eHh8eAAAANjc2Dg4OJicmICEgIiQiJCYkKCkoKisqLi8uMDEwMjQyNDY0ODk4Ojs6AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
-            AAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAJAAkAAAF/iAgjmJAnuY5pqVKsiqMvu4o1MBd6y0QBIKgcEgsGoU/mQ3HUzWVpOZJGqWpBtisdsvd9n7CAWFMLpvP
-            5MFQJka73QMRFdB+29NyV/1uH5hSYEF7fHAFQyeDZViEfj56b1gGfI1QdVpjkYuYWWl/JUNtkQaLnKQDBqOhhkFsZKgEmaeps2OvlI+bWbSxkoqeV5uovLCn
-            kWa3PqDBxKliw5oDqwKtucS8klq2v4hlqJLe370E4JjbJImEx79c6X1bROjtmEQH9fXx8mr2Bwj9/fjt1PhDkKBgwS4IE2IRYDCBgocQj0iUCFHBgosYM2rc
-            yLEjg48gQ4ocSbJkg5MoK1OqXMmypYOXMGPKnEmz5oObOHPq3MmzJ4SfQIMKHUq0aISjSJMqXcq0aQgAOw==
+            R0lGODlhJAAkAMYAAAAAAAEBAQICAgMDAwQEBAUFBQYGBgcHBwgICAkJCQoKCgwMDA0NDQ4PDg8QDxAQEBISEhMTExUWFRgYGBobGh4fHiAhICIjIiMjIyYn
+            JigpKC8vLy8wLzAxMDIzMjM0MzY3Njg4ODo8Oj4/Pj9AP0BBQENEQ0VHRUZIRk5PTk9QT1lbWVpbWmZoZnBycHJ0cnZ4dnh4eHd5d3h8eHt9e3x+fH2AfYCE
+            gIKEgoaIhoeJh4iKiIiLiImMiYuNi4iQiI2PjY6RjpCTkJGUkZCYkJOWk5aZlpmcmZyfnJ6hnqCkoKSnpKisqKmtqa6yrrG0sbi8uLm8ubzAvMDEwMXIxcbK
+            xsjQyMzQzM/Tz9DU0NHV0dLW0tDY0NPX09ba1tfb19jc2ODk4Ojs6Pj8+AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAJAAkAAAH/oBjgoOCMYSHhoeDiYWKhIyKkIiPjoMzlWOXlZqNYzExM6Gio6SlpqKf
+            kpaYnIqtqoSth7KxlIcAuLm6u7y8nZ+hAJjDigCjkMLEymPJtMnLw83F0MQAhonAM8/FBtAAIaOE27fdy9ae05UA5crnqsJUKRACDidSzAYyCw8ugzkWAya8
+            EOSukChhIwBIMJEBQAdmAAhwGABgx5ghABKUiADABjNwoZCNUYEiCpgqABBABAKGBwAOYzwAMDLmyQCYBW8JKsICRIN1zAJ0AaMFwIExC8YxuzZtBQARNZIA
+            BSAUzJYA3ZIWY+rp4BgGAK6AWTIVQBAwPQBsGLMBgJAxrk4GfPiIStyYCwAwkFAQESKBDhSBjPFhtAQEADqWotPZRIOADEfAIgFQoEUCBzAG4agwgAINglzt
+            UsOUc9do0rtGKT1NkNSN1zdWszYG+8aP2z9knzaG+weR30R6CR+eawZwIkqSJz/FvLlyJUyiS59Ovbr161Cya9/Ovbv371PCix9Pvrz581bSq1/Pvr3791zi
+            y59Pv779+2Hy69/Pv7///2IEKOCABBZo4IGBAAA7
         }
     }
 
@@ -84,7 +101,11 @@ class freeText {
     }
 
     proc set-endtext {this value} {
-        $widget::($this,path) insert end $value
+        set path $widget::($this,path)
+        set state [$path cget -state]
+        $path configure -state normal
+        $path insert end $value
+        $path configure -state $state
     }
 
     proc set-deletecommand {this value} {}
@@ -155,8 +176,9 @@ class freeText {
     }
 
     proc monitorCell {this array row column} {                                                    ;# allow duplicate monitored cells
-        viewer::registerTrace $this $array
         set path $widget::($this,path)
+        if {[info exists ($this,event)]} {centerMessage $path {}}               ;# keep displaying help message only in empty viewer
+        viewer::registerTrace $this $array
         if {[info exists ($this,nextCellIndex)]} {                ;# recreate data cell labels placement from recorded configuration
             set index [lindex $composite::($this,-cellindices) $($this,nextCellIndex)]
             if {[string length $index] == 0} {                ;# indexes list exhausted: we are done initializing from recorded data
@@ -167,7 +189,7 @@ class freeText {
             }
         } else {
             set index insert                                                ;# insert cell label text and window at insertion cursor
-            $path insert $index "[viewer::label $array $row $column]: "
+            $path insert $index "[lindex [viewer::label $array $row $column] 0]: "
         }
         set label [new label $path]
         set labelPath $label::($label,path)
@@ -221,7 +243,7 @@ class freeText {
         return $cells                                                                                      ;# may contain duplicates
     }
 
-    proc cells {this} {
+    proc cells {this} {                                                               ;# note: always return cells in the same order
         return [cellsFromLabels $this $($this,labels)]
     }
 
@@ -262,7 +284,7 @@ class freeText {
         return [expr {([llength $($this,labels)] == 0) && ([string length [string trim [$widget::($this,path) get 1.0 end]]] == 0)}]
     }
 
-    proc initializationConfiguration {this} {
+    proc initializationConfiguration {this} {                                ;# note: always return configurations in the same order
         set options {}
         set text {}
         foreach {key string index} [$widget::($this,path) dump -text 1.0 end] {
