@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: htmlview.tcl,v 2.23 2004/01/19 14:51:37 jfontain Exp $
+# $Id: htmlview.tcl,v 2.28 2005/01/02 00:45:07 jfontain Exp $
 
 
 set ::htmlLibraryAdditionalCode {
@@ -30,7 +30,8 @@ set ::htmlLibraryAdditionalCode {
         if {![catch {image create photo -file $source} image]} {
             bind $label <Destroy> "image delete $image"              ;# setup binding so that image is deleted as label is destroyed
             HMgot_image $label $image                                                       ;# got the image for the specified label
-            $label configure -borderwidth 0                                                 ;# suppress relief for better visibility
+            # suppress relief for better visibility and use parent text widget background for transparent areas to really look it
+            $label configure -borderwidth 0 -background [[winfo parent $label] cget -background]
         }
     }
 
@@ -54,27 +55,24 @@ class htmlViewer {
 
         set path $composite::($composite::($this,base),scrolled,path)
         # borders are never shown on focus, padding so that lines do not start too close to the left border
-        $path configure -highlightthickness 0 -state disabled -padx 2 -cursor {}                            ;# hide insertion cursor
-
+        $path configure -highlightthickness 0 -state disabled -padx 2 -background white -cursor {}          ;# hide insertion cursor
         set interpreter [interp create]                      ;# use separate interpreter because HTML library uses global state data
         $interpreter eval "set ::auto_path [list $::auto_path]"                        ;# in case packages are needed by interpreter
         $interpreter eval $::htmlLibraryCode
         $interpreter eval $::htmlLibraryAdditionalCode
-
         $interpreter alias $path $path                                                    ;# make text widget visible in interpreter
+        $interpreter alias formulasHelpWindow formulasHelpWindow
         foreach command {bind bindtags image pack update winfo} {                                           ;# and a few Tk commands
             $interpreter alias $command $command
         }
         foreach command {button frame label scrollbar text} {                                         ;# along with a few Tk widgets
             $interpreter alias $command ::htmlViewer::widget $command $interpreter
         }
-
         $interpreter eval "HMinit_win $path"
         # make HTML text widget behave more like Netscape
         $path tag configure mark -foreground black                                                    ;# override list markers color
         $path tag configure link -borderwidth 1 -foreground blue -underline 1       ;# override hypertext links border and underline
         $interpreter eval "set ::HM${path}(S_symbols) {oooooo\xd7\xb0>:\xb7}"         ;# use simple circles for list element markers
-
         set ($this,interpreter) $interpreter
         set ($this,textPath) $path
         set interpreterFromPath($path) $interpreter                           ;# required for link hit procedure to find interpreter
@@ -118,9 +116,14 @@ class htmlViewer {
     }
 
     proc set-linkto {this viewer} {            ;# allow link hits to target another text widget (used in general help from contents)
-        if {$viewer==$this} {
+        if {$viewer == $this} {
             $($this,interpreter) eval {
                 proc HMlink_callback {widget reference} {                                          ;# supply link callback procedure
+                    switch -glob -- [string tolower [file tail $reference]] {
+                        formulas.htm - formulas-*.htm {
+                            formulasHelpWindow                             ;# formulas help can be launched from general help window
+                        }
+                    }
                     if {![string match #* $reference]} return                                 ;# can only handle internal references
                     HMgoto $widget [string trimleft $reference #]                             ;# always update help data text widget
                 }
