@@ -1,11 +1,14 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: modgui.tcl,v 2.52 2004/01/01 11:39:06 jfontain Exp $
+# $Id: modgui.tcl,v 2.57 2005/01/02 00:45:07 jfontain Exp $
 
 
 class moduleOperations {
 
+    variable internal                                                                ;# internally used options to hide in interface
+    set internal(--daemon) {}
+
     proc moduleOperations {this action} {                                                   ;# action can be display, load or manage
         if {[info exists (displayed)]} {
             delete $($(displayed),dialog)                      ;# only a single module operations dialog box can be active at a time
@@ -55,32 +58,33 @@ class moduleOperations {
         set ($this,action) $action
         set ($this,tips) {}
         lappend ($this,tips) [new widgetTip\
-            -path $composite::($dialog,help,path) -text {display help on this dialog box or on selected module}\
+            -path $composite::($dialog,help,path) -text [mc {display help on this dialog box or on selected module}]\
         ]
         switch $action {
             display {
-                composite::configure $dialog -title {moodss: Loaded modules}
+                composite::configure $dialog -title [mc {moodss: Loaded modules}]
                 loaded $this
             }
             load {
-                composite::configure $dialog -title {moodss: Load modules}
-                composite::configure $dialog load -text Load -command "moduleOperations::load $this" -state disabled
+                composite::configure $dialog -title [mc {moodss: Load modules}]
+                composite::configure $dialog load -text [mc Load] -command "moduleOperations::load $this" -state disabled
                 lappend ($this,tips)\
-                    [new widgetTip -path $composite::($dialog,load,path) -text {load selected module with specified options}]
+                    [new widgetTip -path $composite::($dialog,load,path) -text [mc {load selected module with specified options}]]
                 discover $this
             }
             manage {
-                composite::configure $dialog -title {moodss: Manage modules}
-                composite::configure $dialog unload -text Unload -command "moduleOperations::unload $this"
-                composite::configure $dialog reload -text Reload -command "moduleOperations::reload $this"
-                composite::configure $dialog new -text New -command "moduleOperations::load $this; moduleOperations::loaded $this"
+                composite::configure $dialog -title [mc {moodss: Manage modules}]
+                composite::configure $dialog unload -text [mc Unload] -command "moduleOperations::unload $this"
+                composite::configure $dialog reload -text [mc Reload] -command "moduleOperations::reload $this"
+                composite::configure $dialog new -text [mc New]\
+                    -command "moduleOperations::load $this; moduleOperations::loaded $this"
                 lappend ($this,tips)\
-                    [new widgetTip -path $composite::($dialog,unload,path) -text {unload selected module}]\
+                    [new widgetTip -path $composite::($dialog,unload,path) -text [mc {unload selected module}]]\
                     [new widgetTip -path $composite::($dialog,reload,path)\
-                        -text {unload the selected module then load a new instance using the updated parameters}\
+                        -text [mc {unload the selected module then load a new instance using the updated parameters}]\
                     ]\
                     [new widgetTip -path $composite::($dialog,new,path)\
-                        -text {load a new instance of the selected module using the updated parameters}\
+                        -text [mc {load a new instance of the selected module using the updated parameters}]\
                     ]
                 loaded $this
             }
@@ -95,29 +99,36 @@ class moduleOperations {
     }
 
     proc discover {this} {
+        variable internal
+
         if {![info exists (discoveredModules)]} {         ;# cache modules and their switches during the lifetime of the application
-            $($this,label) configure -text {Searching for modules...}
+            $($this,label) configure -text [mc {searching for modules...}]
             busy 1 $($this,frame)                                                                      ;# show user that we are busy
             update                                       ;# we could use idle tasks update here but it does not work well on windows
+            set format [mc {scanning module %s...}]
             modules::available "
                 lappend moduleOperations::(discoveredModules) %M
                 set moduleOperations::(%M,discoveredSwitches) %S                                 ;# save module switches for caching
                 scrollList::insert $($this,list) 0 %M                                        ;# display just discovered module first
                 update idletasks                                                   ;# keep refreshing while preventing interruptions
             " "
-                # show scanned module for better user feedback (make sure to clear all precedent characters on windows)
-                $($this,label) configure -text {Scanning module %M...                                                           }
+                $($this,label) configure -text \[format {$format} %M\]               ;# show scanned module for better user feedback
                 update idletasks                                                   ;# keep refreshing while preventing interruptions
             "
             busy 0 $($this,frame)
         }
         if {[info exists (discoveredModules)]} {
-            $($this,label) configure -text {Select available module (view its documentation with Help button):}
+            $($this,label) configure -text [mc {Select module (view its documentation with Help button):}]
             update idletasks
             set modules [lsort -dictionary $(discoveredModules)]                                 ;# sort modules for easy navigation
             set index 0
             foreach module $modules {
-                foreach {option argument} [set ($module,switches) $($module,discoveredSwitches)] {
+                set switches {}
+                foreach {option argument} $($module,discoveredSwitches) {
+                    if {[info exists internal($option)]} continue                                  ;# ignore internally used options
+                    lappend switches $option $argument
+                }
+                foreach {option argument} $switches {
                     if {[info exists (last,$module,$option)]} {
                         set ($this,$module,$option) $(last,$module,$option)       ;# use module options used by last loaded instance
                     } elseif {$argument} {                                                           ;# the option takes an argument
@@ -126,13 +137,14 @@ class moduleOperations {
                         set ($this,$module,$option) 0                                 ;# a boolean option is always false by default
                     }
                 }
+                set ($module,switches) $switches
                 set ($this,indexNamespace,$index) $module
                 incr index
             }
             composite::configure $($this,list) -list $modules
             bind $composite::($($this,list),listbox,path) <<ListboxSelect>> "moduleOperations::selection $this"
         } else {
-            $($this,label) configure -text {Found no modules:}
+            $($this,label) configure -text [mc {Found no modules:}]
         }
     }
 
@@ -145,7 +157,7 @@ class moduleOperations {
             # deselect next selection:
             scrollList::selection $($this,list) clear $index; scrollList::selection $($this,list) set $($this,index)
             if {![interactiveQuery\
-                $this {moodss: Module parameters} {Ignore changes and continue the selection of another module?}\
+                $this [mc {moodss: Module parameters}] [mc {Ignore changes and continue the selection of another module?}]\
             ]} return                                                                                     ;# user canceled selection
             set module $($this,indexNamespace,$($this,index))                                                ;# last selected module
             foreach {option argument} $($module,switches) value $($this,values) {
@@ -198,7 +210,7 @@ class moduleOperations {
             incr row
         }
         if {$row == 0} {                                                                               ;# no options for this module
-            set label [label $table.$row,0 -font $font::(mediumItalic) -text {no options}]
+            set label [label $table.$row,0 -font $font::(mediumItalic) -text [mc {no options}]]
             $table window configure $row,0 -window $label
             $table window configure $row,1 -window [label $table.$row,1] -sticky ew
             set width [winfo reqwidth $label]
@@ -226,19 +238,22 @@ class moduleOperations {
             set (last,$name,$option) $($this,$module,$option)                               ;# remember option value for module name
         }
         if {[catch {dynamicallyLoadModules $string} message]} {
-            tk_messageBox\
-                -title {moodss: Error loading module} -type ok -icon error -message $message -parent $widget::($($this,dialog),path)
+            tk_messageBox -title [mc {moodss: Error loading module}]\
+                 -type ok -icon error -message $message -parent $widget::($($this,dialog),path)
         }
     }
 
     proc loaded {this} {
-        $($this,label) configure -text {Select loaded module (view its documentation with Help button):}
+        variable internal
+
+        $($this,label) configure -text [mc {Select loaded module (view its documentation with Help button):}]
         cleanOptions $this
         set index 0
-        foreach {namespace identifier options} [modules::loaded] {
+        foreach {namespace identifier options} [modulesData] {
             lappend list $identifier
             set switches {}
             foreach {switch argument value} $options {
+                if {[info exists internal($switch)]} continue                                      ;# ignore internally used options
                 lappend switches $switch $argument
                 set ($this,$namespace,$switch) $value
             }
@@ -260,7 +275,7 @@ class moduleOperations {
 
     proc unload {this} {
         dynamicallyUnloadModule $($this,indexNamespace,$($this,index))
-        if {[llength $modules::(instances)] == 0} {
+        if {[llength [modulesData]] == 0} {
             # nothing to display: just vanish once module is completely unloaded (### unreproducible bug ###):
             after idle delete $($this,dialog)
         } else {
@@ -272,7 +287,7 @@ class moduleOperations {
     proc reload {this} {
         dynamicallyUnloadModule $($this,indexNamespace,$($this,index))
         load $this                ;# replace the module with a new instance but which keeps the just deleted instance number, if any
-        if {[llength $modules::(instances)] == 0} {
+        if {[llength [modulesData]] == 0} {
             # if the module failed to reload (due to bad options, for example), there may be no loaded modules left at this point
             after idle delete $($this,dialog)
         } else {
@@ -330,11 +345,21 @@ class moduleOperations {
         if {![string equal $($this,action) manage]} {return 1}                               ;# always close when not in manage mode
         if {([string length $($this,index)] == 0) || ![changed $this $($this,index)]} {          ;# no module selected or no changes
             return 1                                                                                                ;# safe to close
-        } elseif {[interactiveQuery $this {moodss: Module parameters} {Ignore changes and close the dialog box?}]} {
+        } elseif {[interactiveQuery $this [mc {moodss: Module parameters}] [mc {Ignore changes and close the dialog box?}]]} {
             return 1                                                                             ;# user chose to ignore the changes
         } else {
             return 0
         }
     }
 
+    proc modulesData {} {
+        set list {}
+        foreach {namespace identifier options} [modules::loaded] {
+            if {[string equal [lindex [modules::decoded $namespace] 0] formulas]} continue                  ;# skip internal modules
+            regsub {<0>$} $identifier {} identifier                ;# remove trailing namespace index for first instance of a module
+            lappend list $namespace $identifier $options
+        }
+        return $list
+    }
+
 }
