@@ -1,12 +1,12 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: modperl.tcl,v 1.7 2004/01/01 11:39:06 jfontain Exp $
+# $Id: modperl.tcl,v 1.9 2005/01/02 00:45:07 jfontain Exp $
 
 
 namespace eval module::perl {
 
-    variable utilityFunctions {
+    variable utilities {
 
         sub array_string {                                     # return string usable by Tcl array set command, from Perl data array
             my @data = @_;
@@ -70,6 +70,27 @@ namespace eval module::perl {
             }
             return $string;
         }
-     }
+
+        {
+            package moodss::Updated;
+            sub TIESCALAR {
+                my $class = shift;
+                my $command = shift;
+                return bless {value => undef, command => $command}, $class;
+            }
+            sub FETCH {
+                my $self = shift;
+                return $self->{value};
+            }
+            sub STORE {
+                my $self = shift;
+                my $value = shift;
+                $self->{value} = $value;
+                $Tcl::parent->eval($self->{command});
+                return $value;
+            }
+        }
+
+    }
 
 }
