@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: myerrorlog.tcl,v 1.40 2004/01/01 11:39:06 jfontain Exp $
+# $Id: myerrorlog.tcl,v 1.48 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide myerrorlog [lindex {$Revision: 1.40 $} 1]
+package provide myerrorlog [lindex {$Revision: 1.48 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -39,6 +39,7 @@ namespace eval myerrorlog {
         6,label message 6,type ascii 6,message {text of the latest message of its sort (may be empty for the 'top' type row if there were no 'new' messages)} 6,anchor left
         7,label index 7,type integer 7,message {message internal index}
         views {{indices {1 2 3 4 5 6}}}
+        persistent 0
         switches {--debug-file 1 --dsn 1 -f 1 --file 1 --password 1 --port 1 -r 1 --remote 1 --rows 1 --user 1 --whole 0}
     }
     set file [open myerrorlog.htm]
@@ -97,7 +98,7 @@ namespace eval myerrorlog {
             }
             $connection disconnect
         } else {
-            package require mysqltcl 2
+            package require mysqltcl
             if {[info exists options(--port)] && (![info exists remote(host)] || [string equal $remote(host) localhost])} {
                 error {--port option useless with local socket connection (localhost as host)}
             }
@@ -107,7 +108,7 @@ namespace eval myerrorlog {
             catch {lappend arguments -password $options(--password)}
             catch {lappend arguments -port $options(--port)}
             set connection [eval mysqlconnect $arguments]
-            set host [mysqlinfo $connection host]
+            set host [lindex [mysqlinfo $connection host] 0]               ;# work around mysqltcl 3 return value: "host via TCP/IP"
             set data(identifier) myerrorlog($host)
             mysqlsel $connection {show variables}
             while {[llength [set list [mysqlnext $connection]]] > 0} {
@@ -126,10 +127,8 @@ namespace eval myerrorlog {
         fconfigure $file -encoding $errorLog(encoding)
         set lines {}
         while {[gets $file line] >= 0} {
-            if {![string match {"*",} $line]} continue                                           ;# message format is: "message...",
-            # remove surrounding " and trailing \n and , and keep the first line only since that is all that is compared:
-            regsub {\\n.*$} [string trim [string trimright $line ,] \"] {} line
-            lappend lines $line
+            if {![string match {"*"} $line] && ![string match {"*",} $line]} continue            ;# message format is: "message...",
+            lappend lines [string trim [string trimright $line ,] \"]                       ;# remove surrounding " and , characters
         }
         close $file
         processErrorMessages $hardCodedMessages 0 $lines 1000                ;# 1000 = MySQL international error messages first code
@@ -513,6 +512,8 @@ namespace eval myerrorlog {
     # Below are the fisrt line or first part (that fits in this source code 132 columns) of error messages found directly in the
     # MySQL source code or reported once as unknown (only the first part of a message is considered necessary for a good match)
     set messages [list\
+        $info      "%s: Normal shutdown"\
+        $info      "%s: Shutdown Complete"\
         $error     "%s, errno=%d, io cache code=%d"\
         $error     "%s. Failed executing load from '%s'"\
         $error     "Aborting"\
@@ -526,7 +527,7 @@ namespace eval myerrorlog {
         $alert     "Can't create interrupt-thread (error %d, errno: %d)"\
         $alert     "Can't create thread-keys"\
         $alert     "Can't init databases"\
-        $alert     "Can't open mysql/func table"\
+        $alert     "Can't open the mysql/func table. Please run the mysql_install_db script to create it."\
         $error     "Can't start server: Bind on TCP/IP port: Address already in use"\
         $critical  "Client requested master to start replication from impossible position"\
         $critical  "Connection to master failed: %s"\
@@ -548,12 +549,15 @@ namespace eval myerrorlog {
         $error     "Error: Can't create thread to kill server"\
         $error     "Error: Couldn't repair table: %s.%s"\
         $error     "Error: Got error during commit;  Binlog is not up to date!"\
+        $error     "Error: using --replicate-same-server-id in conjunction with --log-slave-updates is impossible, it would lead"\
+        $error     "Error: when opening HEAP table, could not allocate memory to write 'DELETE FROM `%s`.`%s`' to the binary log"\
         $error     "error '%s' on query '%s'"\
         $error     "Error '%s' on query '%s'. Default database: '%s'"\
         $error     "Error '%s' running load data infile"\
         $error     "Error checking master version: %s"\
         $error     "Error counting relay log space"\
         $error     "Error deleting %s during purge"\
+        $error     "Error in Append_block event: could not open file '%s'"\
         $error     "Error in Log_event::read_log_event(): '%s', data_len: %d, event_type: %d"\
         $error     "Error initializing relay log position: %s"\
         $error     "Error on COM_BINLOG_DUMP: %d %s, will retry in %d secs"\
@@ -685,12 +689,10 @@ namespace eval myerrorlog {
         $error     "Slave I/O thread: error in mc_mysql_init()"\
         $error     "Slave I/O thread: failed dump request, reconnecting to try again, log '%s' at position %s"\
         $error     "Slave I/O thread: Failed reading log event, reconnecting to retry, log '%s' position %s"\
-        $error     "Slave SQL thread: I/O error reading event(errno: %d  cur_log->error: %d)"\
         $error     "Slave I/O thread aborted while waiting for relay log space"\
+        $error     "Slave I/O thread could not queue event from master"\
         $error     "Slave I/O thread exiting, read up to log '%s', position %s"\
-        $error     "Slave SQL thread exiting, replication stopped in log '%s' at position %s"\
         $info      "Slave I/O thread initialized"\
-        $info      "Slave SQL thread initialized, starting replication in log '%s' at position %s, relay log '%s' position: %s"\
         $error     "Slave I/O thread killed during or after a reconnect done to recover from failed read"\
         $error     "Slave I/O thread killed during or after reconnect"\
         $error     "Slave I/O thread killed while connecting to master"\
@@ -698,8 +700,12 @@ namespace eval myerrorlog {
         $error     "Slave I/O thread killed while requesting master dump"\
         $error     "Slave I/O thread killed while retrying master dump"\
         $error     "Slave I/O thread killed while waiting to reconnect after a failed read"\
+        $error     "Slave SQL thread: I/O error reading event(errno: %d  cur_log->error: %d)"\
+        $error     "Slave SQL thread exiting, replication stopped in log '%s' at position %s"\
+        $info      "Slave SQL thread initialized, starting replication in log '%s' at position %s, relay log '%s' position: %s"\
         $error     "Table %s had a open data handler in reopen_table"\
         $warning   "TCP/IP or --enable-named-pipe should be configured on NT OS"\
+        $error     "TCP/IP unavailable or disabled with --skip-networking; no available interfaces"\
         $error     "The first log %s failed to stat during purge"\
         $error     "there is an unfinished transaction in the relay log (could find neither COMMIT nor ROLLBACK in the relay log);"\
         $critical  "unknown error reading log event on the master"\
@@ -712,6 +718,7 @@ namespace eval myerrorlog {
         $warning   "Warning: Changed limits: max_connections: %ld  table_cache: %ld"\
         $warning   "Warning: Checking table:   '%s'"\
         $warning   "Warning: Could not open BDB table %s.%s after rename"\
+        $warning   "Warning: Could not remove tmp table: '%s', error: %d"\
         $warning   "Warning: DosSetRelMaxFH couldn't increase number of open files to more than %d"\
         $warning   "Warning: Failed to lock memory. Errno: %d"\
         $warning   "Warning: Got signal: %d, error: %d"\
@@ -723,11 +730,13 @@ namespace eval myerrorlog {
         $warning   "Warning: setrlimit couldn't increase number of open files to more than %lu (request: %u)"\
         $warning   "Warning: setrlimit returned ok, but didn't change limits. Max open files is %ld (request: %u)"\
         $warning   "Warning: Setting lower_case_table_names=1 because file system %s is case insensitive"\
+        $warning   "Warning: Setting lower_case_table_names=2 because file system for %s is case insensitive"\
         $warning   "Warning: The position in the binary log can't be less than %d."\
-        $warning   "Warning: Wrong use count: %u for tree at %lx"\
-        $warning   "Warning: You have enabled the binary log, but you haven't set server-id:"\
+        $warning   "Warning: Wrong use count: %u (should be %u) for tree at %lx"\
+        $warning   "Warning: You have enabled the binary log, but you haven't set server-id to a non-zero value: we force server"\
+        $warning   "Warning: You have forced lower_case_table_names to 0 through a command-line option, even though your file"\
         $warning   "Warning: you need to use --log-bin to make --log-slave-updates work. Now disabling --log-slave-updates."\
-        $warning   "Warning: You should set server-id to a non-0 value if master_host is set."\
+        $warning   "Warning: You should set server-id to a non-0 value if master_host is set; we force server id to 2, but this"\
         $error     "While trying to obtain the list of slaves from the master '%s:%d', user '%s' got the following error: '%s'"\
         $error     "Wrong record length %s of %s at %s"\
         $error     "Wrong tree: Found right == left"\
@@ -750,7 +759,8 @@ namespace eval myerrorlog {
     }
     unset code level message messages
 
-    # for international messages as found in /usr/share/mysql/english/errmsg.txt for example:
+    # for international messages as found in /usr/share/mysql/english/errmsg.txt for example, while importance level can be
+    # determined by looking at sources: include/mysqld_error.h and Docs/mysqld_error.txt
     array set codeLevel [list\
         1000 $debug 1001 $debug 1002 $debug 1003 $debug 1004 $alert 1005 $critical 1006 $critical 1007 $critical 1008 $critical\
             1009 $critical\
@@ -785,7 +795,7 @@ namespace eval myerrorlog {
             1209 $error\
         1210 $error 1211 $error 1212 $error 1213 $error 1214 $warning 1215 $error 1216 $error 1217 $error 1218 $error 1219 $error\
         1220 $error 1221 $error 1222 $error 1223 $error 1224 $error 1225 $error 1226 $error 1227 $error 1228 $error 1229 $error\
-        1230 $error 1231 $error 1232 $error 1233 $error 1234 $error 1235 $warning 1236 $critical 1237 $warning
+        1230 $error 1231 $error 1232 $error 1233 $error 1234 $error 1235 $warning 1236 $critical 1237 $warning 1238 $error\
     ]
 
     unset debug info notice warning error critical alert emergency
