@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: myhealth.tcl,v 1.22 2004/01/01 11:39:06 jfontain Exp $
+# $Id: myhealth.tcl,v 1.28 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide myhealth [lindex {$Revision: 1.22 $} 1]
+package provide myhealth [lindex {$Revision: 1.28 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -44,6 +44,7 @@ namespace eval myhealth {
         10,label {key efficiency} 10,type real 10,message {Efficiency of the key cache in %.} 1,10 ?
         pollTimes {20 10 30 60 120 300}
         switches {--dsn 1 --host 1 --password 1 --port 1 --user 1}
+        persistent 1
         views {{visibleColumns {0 1 2 3 4 5 6 7 8 9 10} swap 1}}
     }
     # variable name to column mapping for MySQL changing variables:
@@ -51,7 +52,7 @@ namespace eval myhealth {
         Uptime 2 Slow_queries 3 Select_scan 5 Select_full_join 6 Aborted_connects 7 Created_tmp_disk_tables 8 Created_tmp_files 9
     }
     # MySQL variables needed for indicators calculations
-    array set needed {max_connections {} Max_used_connections {} Key_read_requests {} Key_reads {}}
+    array set needed {max_connections {} Threads_connected {} Key_read_requests {} Key_reads {}}
     set file [open myhealth.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
@@ -98,15 +99,15 @@ namespace eval myhealth {
             catch {lappend arguments -password $options(--password)}
             catch {lappend arguments -port $options(--port)}
             if {[info exists thread]} {
-                worker::evaluate $thread(worker) {package require mysqltcl 2.1}                         ;# use a thread safe version
+                worker::evaluate $thread(worker) {package require mysqltcl}
                 set connection [worker::wait $thread(worker) "mysqlconnect $arguments"]                           ;# do not hang GUI
-                set host [worker::evaluate $thread(worker) "mysqlinfo $connection host"]
+                set host [worker::evaluate $thread(worker) "lindex \[mysqlinfo $connection host\] 0"]
                 # do not retrieve version in background as it is unlikely to fail if connection was established:
                 set list [worker::evaluate $thread(worker) "mysqlsel $connection {show variables like 'version'} -flatlist"]
             } else {
-                package require mysqltcl 2
+                package require mysqltcl
                 set connection [eval mysqlconnect $arguments]
-                set host [mysqlinfo $connection host]
+                set host [lindex [mysqlinfo $connection host] 0]           ;# work around mysqltcl 3 return value: "host via TCP/IP"
                 set list [mysqlsel $connection {show variables like 'version'} -flatlist]
             }
             set data(identifier) myhealth($host)
@@ -119,8 +120,7 @@ namespace eval myhealth {
                 if {$subMinor < 33} {error {cannot monitor a server below version 3.23.33 in the 3.23 series}}
             }
             4 {
-                if {$minor != 0} {error "cannot monitor a server version 4.$minor"}
-                if {$subMinor < 1} {error {cannot monitor a server below version 4.0.1 in the 4.0 series}}
+                if {$minor > 1} {error "cannot monitor a server version 4.$minor"}
             }
             default {
                 error "cannot monitor a server version $major"
@@ -284,7 +284,7 @@ namespace eval myhealth {
             }
         }
         if {!$error} {
-            set data(0,4) [expr {$needed(max_connections) - $needed(Max_used_connections)}]
+            set data(0,4) [expr {$needed(max_connections) - $needed(Threads_connected)}]
             if {$needed(Key_read_requests) == 0} {
                 set data(0,10) ?                                                                               ;# no keyed reads yet
             } else {
