@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: myprocs.tcl,v 2.26 2004/01/01 11:39:06 jfontain Exp $
+# $Id: myprocs.tcl,v 2.30 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide myprocs [lindex {$Revision: 2.26 $} 1]
+package provide myprocs [lindex {$Revision: 2.30 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -41,6 +41,7 @@ namespace eval myprocs {
         7,label information 7,type dictionary 7,message {query information} 7,anchor left
         sort {0 decreasing}
         pollTimes {10 5 20 30 60 120 300}
+        persistent 1
         switches {--dsn 1 --host 1 --password 1 --port 1 --user 1}
     }
     set file [open myprocs.htm]
@@ -83,7 +84,7 @@ namespace eval myprocs {
             }
         } else {
             set odbc 0
-            package require mysqltcl 2
+            package require mysqltcl
             if {[info exists options(--port)] && (![info exists options(--host)] || [string equal $options(--host) localhost])} {
                 error {--port option useless with local socket connection (localhost as host)}
             }
@@ -93,13 +94,13 @@ namespace eval myprocs {
             catch {lappend arguments -password $options(--password)}
             catch {lappend arguments -port $options(--port)}
             if {[info exists thread]} {
-                worker::evaluate $thread(worker) {package require mysqltcl 2.1}                         ;# use a thread safe version
+                worker::evaluate $thread(worker) {package require mysqltcl}
                 set connection [worker::wait $thread(worker) "mysqlconnect $arguments"]                           ;# do not hang GUI
-                set host [worker::evaluate $thread(worker) "mysqlinfo $connection host"]
+                set host [worker::evaluate $thread(worker) "lindex \[mysqlinfo $connection host\] 0"]
             } else {
-                package require mysqltcl 2
+                package require mysqltcl
                 set connection [eval mysqlconnect $arguments]
-                set host [mysqlinfo $connection host]
+                set host [lindex [mysqlinfo $connection host] 0]           ;# work around mysqltcl 3 return value: "host via TCP/IP"
             }
             set data(identifier) myprocs($host)
             if {[info exists thread]} {
