@@ -1,15 +1,15 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: myquery.tcl,v 2.25 2004/01/01 11:39:06 jfontain Exp $
+# $Id: myquery.tcl,v 2.29 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide myquery [lindex {$Revision: 2.25 $} 1]
+package provide myquery [lindex {$Revision: 2.29 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
 if {[catch {package require Thread 2.5}]} {
-    package require mysqltcl 2
+    package require mysqltcl
 } else {                                               ;# use threads so that core is not blocked on connection problems with server
     package require stooop 4.1
     namespace import stooop::*
@@ -23,7 +23,7 @@ if {[catch {package require Thread 2.5}]} {
             if {[lsearch -exact $auto_path /usr/lib] < 0} {           ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
                 lappend auto_path /usr/lib
             }
-            package require mysqltcl 2.1                                                                ;# use a thread safe version
+            package require mysqltcl                                                                    ;# use a thread safe version
         }
     }
 }
@@ -36,6 +36,7 @@ namespace eval myquery {
         0,label {} 0,type integer 0,message {row number in order of arrival}
         pollTimes {10 5 20 30 60 120 300}
         switches {--host 1 --password 1 --port 1 -q 1 --query 1 --swap 0 -t 1 --table 1 --user 1}
+        persistent 1
         sort {0 increasing}
     }
     set file [open myquery.htm]
@@ -78,10 +79,10 @@ namespace eval myquery {
         catch {lappend arguments -port $options(--port)}
         if {[info exists thread]} {
             set connection [worker::wait $thread(worker) "mysqlconnect $arguments"]                               ;# do not hang GUI
-            set host [worker::evaluate $thread(worker) "mysqlinfo $connection host"]
+            set host [worker::evaluate $thread(worker) "lindex \[mysqlinfo $connection host\] 0"]
         } else {
             set connection [eval mysqlconnect $arguments]
-            set host [mysqlinfo $connection host]
+            set host [lindex [mysqlinfo $connection host] 0]               ;# work around mysqltcl 3 return value: "host via TCP/IP"
         }
         set data(identifier) myquery($host)
         if {[info exists thread]} {
