@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: mystatus.tcl,v 2.46 2004/01/01 11:39:06 jfontain Exp $
+# $Id: mystatus.tcl,v 2.51 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide mystatus [lindex {$Revision: 2.46 $} 1]
+package provide mystatus [lindex {$Revision: 2.51 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -37,6 +37,7 @@ namespace eval mystatus {
         0,label display 0,type ascii 0,message {per second values are calculated for the last poll period using the absolute values from the server, and only for variables that make sense (otherwise a ? is displayed in the column)}
         0,0 absolute 1,0 {per second}
         pollTimes {10 5 20 30 60 120 300}
+        persistent 0
         switches {--dsn 1 --host 1 --password 1 --port 1 --user 1}
     }
     set file [open mystatus.htm]
@@ -88,15 +89,15 @@ namespace eval mystatus {
             catch {lappend arguments -password $options(--password)}
             catch {lappend arguments -port $options(--port)}
             if {[info exists thread]} {
-                worker::evaluate $thread(worker) {package require mysqltcl 2.1}                         ;# use a thread safe version
+                worker::evaluate $thread(worker) {package require mysqltcl}
                 set connection [worker::wait $thread(worker) "mysqlconnect $arguments"]                           ;# do not hang GUI
-                set host [worker::evaluate $thread(worker) "mysqlinfo $connection host"]
+                set host [worker::evaluate $thread(worker) "lindex \[mysqlinfo $connection host\] 0"]
                 # do not retrieve version in background as it is unlikely to fail if connection was established:
                 set list [worker::evaluate $thread(worker) "mysqlsel $connection {show variables like 'version'} -flatlist"]
             } else {
-                package require mysqltcl 2
+                package require mysqltcl
                 set connection [eval mysqlconnect $arguments]
-                set host [mysqlinfo $connection host]
+                set host [lindex [mysqlinfo $connection host] 0]           ;# work around mysqltcl 3 return value: "host via TCP/IP"
                 set list [mysqlsel $connection {show variables like 'version'} -flatlist]
             }
             set data(identifier) mystatus($host)
@@ -108,8 +109,7 @@ namespace eval mystatus {
                 if {$subMinor < 51} {error {cannot monitor a server below version 3.23.51 in the 3.23 series}}
             }
             4 {
-                if {$minor != 0} {error "cannot monitor a server version 4.$minor"}
-                if {$subMinor < 1} {error {cannot monitor a server below version 4.0.1 in the 4.0 series}}
+                if {$minor > 1} {error "cannot monitor a server version 4.$minor"}
             }
             default {
                 error "cannot monitor a server version $major"
