@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: myvars.tcl,v 1.52 2004/01/01 11:39:06 jfontain Exp $
+# $Id: myvars.tcl,v 1.58 2005/01/15 14:15:41 jfontain Exp $
 
 
-package provide myvars [lindex {$Revision: 1.52 $} 1]
+package provide myvars [lindex {$Revision: 1.58 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -35,6 +35,7 @@ namespace eval myvars {
         updates 0
         0,label {} 0,type ascii 0,message {} 0,0 {}
         pollTimes {60 10 20 30 60 120 300}
+        persistent 0
         switches {--dsn 1 --host 1 --password 1 --port 1 --user 1}
     }
     set file [open myvars.htm]
@@ -78,7 +79,7 @@ namespace eval myvars {
             scan [lindex [lindex $list 0] 1] %u.%u.%u major minor subMinor
         } else {
             set odbc 0
-            package require mysqltcl 2
+            package require mysqltcl
             if {[info exists options(--port)] && (![info exists options(--host)] || [string equal $options(--host) localhost])} {
                 error {--port option useless with local socket connection (localhost as host)}
             }
@@ -88,15 +89,15 @@ namespace eval myvars {
             catch {lappend arguments -password $options(--password)}
             catch {lappend arguments -port $options(--port)}
             if {[info exists thread]} {
-                worker::evaluate $thread(worker) {package require mysqltcl 2.1}                         ;# use a thread safe version
+                worker::evaluate $thread(worker) {package require mysqltcl}
                 set connection [worker::wait $thread(worker) "mysqlconnect $arguments"]                           ;# do not hang GUI
-                set host [worker::evaluate $thread(worker) "mysqlinfo $connection host"]
+                set host [worker::evaluate $thread(worker) "lindex \[mysqlinfo $connection host\] 0"]
                 # do not retrieve version in background as it is unlikely to fail if connection was established:
                 set list [worker::evaluate $thread(worker) "mysqlsel $connection {show variables like 'version'} -flatlist"]
             } else {
-                package require mysqltcl 2
+                package require mysqltcl
                 set connection [eval mysqlconnect $arguments]
-                set host [mysqlinfo $connection host]
+                set host [lindex [mysqlinfo $connection host] 0]           ;# work around mysqltcl 3 return value: "host via TCP/IP"
                 set list [mysqlsel $connection {show variables like 'version'} -flatlist]
             }
             set data(identifier) myvars($host)
@@ -108,8 +109,7 @@ namespace eval myvars {
                 if {$subMinor < 57} {error {cannot monitor a server below version 3.23.57 in the 3.23 series}}
             }
             4 {
-                if {$minor != 0} {error "cannot monitor a server version 4.$minor"}
-                if {$subMinor < 1} {error {cannot monitor a server below version 4.0.1 in the 4.0 series}}
+                if {$minor > 1} {error "cannot monitor a server version 4.$minor"}
             }
             default {
                 error "cannot monitor a server version $major"
@@ -169,6 +169,7 @@ namespace eval myvars {
         set columns {}
         set column 0
         foreach variable $variables {
+            if {[info exists nameColumn($variable)]} continue      ;# happens with 4.1.8 where innodb_max_purge_lag is listed twice!
             incr column              ;# keep column numbers even for non displayed columns in case those get enabled at a later time
             if {[info exists skip([lindex [split $variable _] 0])]} continue                                        ;# not displayed
             set nameColumn($variable) $column                                                            ;# remember variable column
