@@ -9,3 +9,5 @@ package ifneeded miscellaneous 1 [list source [file join $dir misc.tcl]]
 package ifneeded threads 1 [list source [file join $dir threads.tcl]]
 package ifneeded linetask 1 [list source [file join $dir linetask.tcl]]
 package ifneeded moodsshelp 1 [list source [file join $dir htmldata.tcl]]
+package ifneeded moodsshelp-ja 1 [list source [file join $dir htmldata-ja.tcl]]
+package ifneeded hashes 1 [list source [file join $dir hashes.tcl]]
