@@ -1,8 +1,8 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this
 # package or use the Help Copyright menu
 
-# $Id: pci.tcl,v 1.33 2004/01/25 23:10:15 jfontain Exp $
+# $Id: pci.tcl,v 1.43 2005/02/12 20:39:48 jfontain Exp $
 
 
 # This module displays information about all PCI buses on a host system and all
@@ -18,7 +18,7 @@
 # HTML documentation.
 
 
-package provide pci [lindex {$Revision: 1.33 $} 1]
+package provide pci [lindex {$Revision: 1.43 $} 1]
 # load package used for handling module remote arguments
 package require network 1
 # load object oriented package needed by task class
@@ -48,17 +48,16 @@ namespace eval pci {
         4,label description 4,type ascii 4,message {device description}
             4,anchor left
         indexColumns {0 1 2}
-        sort {0 increasing}
         pollTimes {60 10 20 30 120 300 600}
-        switches {-C 0 --daemon 0 -i 1 --path 1 -r 1 --remote 1}
+        persistent 1
+        switches {-C 0 --daemon 0 -i 1 -p 1 --path 1 -r 1 --remote 1}
     }
     # options (see pci HTML documentation for detailed information):
     #   --path: the directory where the lspci command resides
     #   -r (--remote): the remote host specification in URL format
 
-    set nextIndex 0                                             ;# new row index
     set file [open pci.htm]
-    set data(helpText) [read $file]       ;# initialize HTML help data from file
+    set data(helpText) [::read $file]     ;# initialize HTML help data from file
     close $file
     unset file
 
@@ -76,7 +75,7 @@ namespace eval pci {
         catch {set locator $options(--remote)}              ;# favor long option
         if {![info exists locator]} {                              ;# local host
             # detect errors early by attempting immediate data retrieval
-            eval exec $path                ;# a fatal error would be caught here
+            exec $path                     ;# a fatal error would be caught here
             set local(command) $path
             return                                           ;# local monitoring
         }
@@ -88,8 +87,9 @@ namespace eval pci {
         set data(identifier) pci($remote(host))
         # important: pack data in a single line using special control
         # separator characters (chosen after checking that they never are
-        # included in expected remote data)
-        set remote(command) "$path | tr '\\n' '\\v'"
+        # included in expected remote data), and redirect error output to make
+        # sure error messages are retrieved when using ssh
+        set remote(command) "$path 2>&1 | tr '\\n' '\\v'"
         if {[string equal $::tcl_platform(platform) unix]} {        ;# UNIX type
             if {$remote(rsh)} {                                           ;# rsh
                 # does not support disabling pseudo terminal allocation so
@@ -106,7 +106,10 @@ namespace eval pci {
                 if {[info exists options(-i)]} {                ;# identity file
                     append command " -i \"$options(-i)\""
                 }
-                append command " -T -l $remote(user) $remote(host) 2>@ stdout"
+                if {[info exists options(-p)]} {
+                    append command " -p $options(-p)"                    ;# port
+                }
+                append command " -T -l $remote(user) $remote(host) 2>@stdout"
                 # note: redirect standard error to pipe output in order to be
                 # able to detect remote errors
             }
@@ -119,7 +122,7 @@ namespace eval pci {
             # to the putty package documentation
             set remote(rsh) 0
             # note: host must be a putty session and pageant must be running
-            set command "plink -ssh -batch -T $remote(host) 2>@ stdout"
+            set command "plink -ssh -batch -T $remote(host) 2>@stdout"
         }
         if {$remote(rsh)} {                                               ;# rsh
             set access r                        ;# writing to pipe is not needed
@@ -153,6 +156,7 @@ namespace eval pci {
             if {$remote(busy)} {
                 return          ;# core invocation while waiting for remote data
             }
+            set remote(busy) 1                   ;# obviously busy at this point
             if {[lineTask::end $remote(task)]} {       ;# rsh or ssh daemon mode
                 # note: for rsh, shell and command are restarted here each time
                 lineTask::begin $remote(task)
@@ -161,7 +165,6 @@ namespace eval pci {
                 # start data retrieval by sending command to remote side
                 lineTask::write $remote(task) $remote(command)
             }
-            set remote(busy) 1                   ;# obviously busy at this point
         } elseif {[catch {set result [exec $local(command)]} message]} {
             # immediate retrieval failure on local host
             flashMessage "lspci error: $message"
@@ -172,35 +175,48 @@ namespace eval pci {
 
     proc process {lines} {          ;# process PCI data lines and update display
         variable data
-        variable index
-        variable nextIndex
 
         set expression\
             {^([[:xdigit:]]+):([[:xdigit:]]+)\.(\d)+\s([^:]*):\s(.*)$}
         foreach line $lines {
             if {![regexp\
                 $expression $line dummy bus device function type description\
-            ]} {
-                flashMessage "invalid line: $line"        ;# should never happen
-                continue
-            }
-            set key $bus,$device,$function             ;# uniquely defines a row
-            if {[catch {set index($key)} row]} {            ;# this is a new row
-                set row [set index($key) $nextIndex]              ;# remember it
-                incr nextIndex                     ;# get ready for next new row
-            }
+            ]} continue             ;# happens when lines contain error messages
+            # generate unique 32 unsigned integer from bus, device (5 bits) and
+            # function (3 bits) (from /usr/include/linux/pci.h)
+            set row [format %u\
+                [expr {("0x$bus" << 8) | ("0x$device" << 3) | $function}]\
+            ]
             # fill or update row
             set data($row,0) $bus
             set data($row,1) $device
             set data($row,2) $function
             set data($row,3) $type
             set data($row,4) $description
-            set current($key) {}                  ;# remember data for this poll
+            set current($row) {}                  ;# remember data for this poll
         }
-        foreach {key row} [array get index] {     ;# cleanup disappeared entries
-            if {[info exists current($key)]} continue    ;# part of current data
-            array unset data $row,\[0-9\]*          ;# remove obsolete data rows
-            unset index($key)
+        foreach name [array names data *,0] {     ;# cleanup disappeared entries
+            set row [lindex [split $name ,] 0]
+            if {![info exists current($row)]} {array unset data $row,\[0-9\]*}
+        }
+        if {![info exists current] && ([string length [lindex $lines 0]] > 0)} {
+            # There generally should be some data, so report errors when there
+            # seems to be invalid data, while ignoring empty lines, which may
+            # be the result of error messages coming via ssh.
+            # Note: when remotely monitoring, rsh and ssh behave differently in
+            # case of errors. In rsh case, error is detected in the read{}
+            # procedure when the communication pipe is closed, as is done at
+            # every poll. In ssh case, the communication pipe is never closed,
+            # so problems must be detected by looking at the actual returned
+            # data, which, when invalid, likely contains an error message, and
+            # is processed here.
+            # Only report the first line, in order to warn the user, but not
+            # clog the message and trace displays
+            set message "invalid data: [lindex $lines 0]"
+            if {[llength $lines] > 1} {
+                append message "..." ;# show that there was more data to display
+            }
+            flashMessage $message
         }
         incr data(updates)
     }
@@ -225,7 +241,7 @@ namespace eval pci {
         }
         # unpack list while removing extra last separator without copying to a
         # variable for better performance, as data could be big
-        # note: is case of an unexpected event, task insures that line is empty
+        # note: in case of an unexpected event, task insures that line is empty
         if {[info exists message]} {           ;# report unexpected event if any
             flashMessage $message
         }
