@@ -1,15 +1,16 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: ping.tcl,v 2.34 2004/01/01 11:39:06 jfontain Exp $
+# $Id: ping.tcl,v 2.41 2005/01/02 00:45:07 jfontain Exp $
 
 
-package provide ping [lindex {$Revision: 2.34 $} 1]
-if {[string equal $::tcl_platform(platform) windows]} {
+package provide ping [lindex {$Revision: 2.41 $} 1]
+if {$::tcl_platform(platform) eq "windows"} {
     package require Tnm                        ;# for host to IP address conversion since the library below is not ported to Windows
 } else {
     package require network 1
 }
+package require hashes
 package require stooop 4.1
 namespace import stooop::*
 
@@ -38,11 +39,13 @@ namespace eval ping {
             {visibleColumns {0 1 2 3 4 5 6 7} sort {0 increasing}}
             {visibleColumns {0 8 9 10 11 12 13} sort {0 increasing}}
         }
+        persistent 1 64Bits 1
         switches {-f 1 -r 1}
     }
     set file [open ping.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
+    unset file
 
     proc initialize {optionsName} {
         upvar 1 $optionsName options
@@ -54,14 +57,15 @@ namespace eval ping {
         }
         set file [open $name]
         set period 2147483647
-        set row 0
+        set rows {}
         while {[gets $file line] >= 0} {
             set line [string trim $line]
-            if {[string length $line] == 0} continue                                                             ;# skip empty lines
+            if {$line eq ""} continue                                                                            ;# skip empty lines
             if {[string match #* $line]} continue                                                                        ;# comments
             if {[llength $line] != 6} {                                                                         ;# and invalid lines
                 error "invalid line in hosts file:\n$line"
             }
+            set row [hash64::string [lindex $line 0]]                              ;# derive row number from host name or IP address
             foreach "data($row,0) data($row,3) data($row,6) data($row,4) data($row,5) data($row,7)" $line {}
             if {$data($row,3) <= 0} {
                 error "host $data($row,0) period ($data($row,3)) must be greater than 0."
@@ -85,7 +89,7 @@ namespace eval ping {
             }
             pushMessage "looking up IP address for $data($row,0)"                                        ;# looking time may be long
             set data($row,1) {}
-            if {[string equal $::tcl_platform(platform) windows]} {
+            if {$::tcl_platform(platform) eq "windows"} {
                 catch {set data($row,1) [::Tnm::netdb hosts address $data($row,0)]}
             } else {
                 catch {set data($row,1) [::network::addressfromhost $data($row,0)]}
@@ -95,19 +99,15 @@ namespace eval ping {
             if {$data($row,3) < $period} {
                 set period $data($row,3)                                           ;# keep track of the minimum period for all hosts
             }
-            incr row
+            lappend rows $row
         }
         close $file
-
         set data(pollTimes) -$period
-
         set requestsHost 127.0.0.1                                                                          ;# local host by default
         catch {set requestsHost $options(-r)}                                                   ;# may be overridden in command line
         ::session::initialize $requestsHost ::ping::received
-
-        set rows $row
-        for {set row 0} {$row < $rows} {incr row} {                                                            ;# initialize polling
-            if {[string length $data($row,1)] == 0} continue                          ;# do not attempt to ping unresolved addresses
+        foreach row $rows {                                                                                    ;# initialize polling
+            if {$data($row,1) eq ""} continue                                         ;# do not attempt to ping unresolved addresses
             launch $row
         }
     }
@@ -204,7 +204,7 @@ class session {
     proc initialize {requestsHost receiveCommand} {                             ;# command to be invoked once reception is completed
         if {[catch {set ::session::(socket) [socket $requestsHost nmicmp]} message]} {
             error \
-"could not connect to the nmcimpd server for the following reason:
+"could not connect to the nmicmpd server for the following reason:
   $message
 Please read the INSTALL file in the ping module sub-directory."
         }
