@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: random.tcl,v 1.53 2004/01/01 11:39:06 jfontain Exp $
+# $Id: random.tcl,v 1.61 2005/01/02 00:45:07 jfontain Exp $
 
 # module sample file with random data
 
@@ -9,7 +9,7 @@
 # package ifneeded random 1.45 [list source [file join $dir random.tcl]]
 
 # Note: see version number management requirements in moodss HTML documentation (Module Development section)
-package provide random [lindex {$Revision: 1.53 $} 1]
+package provide random [lindex {$Revision: 1.61 $} 1]
 
 namespace eval random {                                                                    ;# namespace name must match package name
     # Module data is arranged in a single array that can be used by many views, such as tables, graphs, pies,...
@@ -58,6 +58,10 @@ namespace eval random {
     #   example in modules that gather data from a remote host: in such a case, the identifier could be set to dataType(hostName).
     # - resizableColumns: optional. A boolean value (0 or 1) which specifies whether displayed data table(s) columns can be manually
     #   resized by the user with the mouse. Not available on a per view basis.
+    # - persistent: optional (considered false if missing). A boolean value that tells whether row numbers (used by the core, along
+    #   with the column numbers, to identify data cells) are identically mapped from data row keys across module instances.
+    # - 64Bits: optional. A boolean value (0 or 1) which specifies whether row numbers are unsigned 64 bit integers (which requires
+    #   a Tcl 8.4 or above core version). If absent or false, row numbers as unsigned 32 bit integers are assumed.
     array set data {
         updates 0
         0,label name 0,type ascii 0,message {user name}
@@ -71,9 +75,17 @@ namespace eval random {
             {indices {0 1 3 4} sort {1 decreasing}}
             {indices {0 2 4} sort {2 decreasing}}
         }
+        persistent 1
         switches {-a 0 --asynchronous 0 -i 0 --identify 0 --daemon 0}
     }
-    set file [open random.htm]                                               ;# module directory is current during the loading phase
+    if {\
+       ([info exists ::env(LC_ALL)] && [string match ja* $::env(LC_ALL)]) ||\
+       ([info exists ::env(LANG)] && [string match ja* $::env(LANG)])\
+    } {
+        set file [open random-ja.htm]
+    } else {
+        set file [open random.htm]                                           ;# module directory is current during the loading phase
+    }
     set data(helpText) [read $file]                                   ;# initialize HTML help data from file in module sub-directory
     close $file
 
@@ -86,7 +98,7 @@ namespace eval random {
     # as sole argument. The array contains the switched options values, indexed by switch. For example, if the command line was:
     # $ moodss random --asynchronous --other-option value -x 1234
     # the options array will contain:
-    #   options(--asynchronous) = 
+    #   options(--asynchronous) =
     #   options(--other-option) = value
     #   options(-x)             = 1234
     # Note that the --asynchronous member value is empty as that switch takes no argument.
@@ -109,6 +121,11 @@ namespace eval random {
         if {[info exists options(-i)] || [info exists options(--identify)]} {                 ;# generate a unique module identifier
             set data(identifier) "random [expr {int(rand() * 100)}]"
         }
+        if {[package vcompare $::tcl_version 8.4] < 0} {
+            set data(64Bits) 0
+        } else {                                                                                                ;# Tcl 8.4 and above
+            set data(64Bits) 1
+        }
     }
 
     # The terminate procedure if it exists is invoked by the core when the module is unloaded dynamically.
@@ -118,9 +135,10 @@ namespace eval random {
 
     # The dynamic data array index is the row number followed by the column number separated by a comma.
     # The column number must start from 0 up to the total number of columns minus 1 (no holes are allowed in the column sequence)
-    # The row number can take any positive integer value (between 0 and 2147483647) and be defined in any order, as long as it is
-    # unique during the lifetime of the module data. if a new row is created, it must take a value that was never used: thus, the
-    # number of a row that has disappeared is not allowed. Row numbers need not be consecutive.
+    # The row number can take any positive integer value (between 0 and 4294967295) and be defined in any order. Holes in the rows
+    # sequence are allowed as well: row numbers need not be consecutive (also see HTML developer documentation on handling unsigned
+    # 32 bit integers, database implications and row/key relationship)
+    # If a row has to disappear, all data cells for that row must disappear, that is cells for all columns.
     # Voidness for numeric data cells (integer or real type) takes the form of the ? character. Reminder: as long as a data row
     # exists, all its data cells must exist. Thus voidness cannot be expressed by non-existence.
 
@@ -139,16 +157,26 @@ namespace eval random {
                 2,3 [expr {20 + int(rand() * 30)}] 2,4 ps
             4,0 Peter\\nWard 4,1 [format %.1f [expr {rand() * 8}]] 4,2 [expr {50 + int(rand() * 10)}]
                 4,3 [expr {5 + int(rand() * 10)}] 4,4 ls
-            6,0 Laura 6,1 [format %.1f [expr {rand() * 20}]] 6,2 [expr {90 + int(rand() * 20)}]
-                6,3 [expr {9 + int(rand() * 20)}] 6,4 emacs
-            3,0 Robert 3,1 [format %.1f [expr {rand() * 10}]] 3,2 [expr {500 + int(rand() * 150)}]
-                3,3 [expr {50 + int(rand() * 150)}] 3,4 top
-            9,0 Laura 9,1 [format %.1f [expr {rand() * 30}]] 9,2 [expr {100 + int(rand() * 50)}]
-                9,3 [expr {10 + int(rand() * 50)}] 9,4 cc
+            2147483647,0 Laura 2147483647,1 [format %.1f [expr {rand() * 20}]] 2147483647,2 [expr {90 + int(rand() * 20)}]
+                2147483647,3 [expr {9 + int(rand() * 20)}] 2147483647,4 emacs
+            2147483648,0 Robert 2147483648,1 [format %.1f [expr {rand() * 10}]] 2147483648,2 [expr {500 + int(rand() * 150)}]
+                2147483648,3 [expr {50 + int(rand() * 150)}] 2147483648,4 top
         "
-        incr data(updates)                         ;# write variable so that update can be detected with an eventual tcl write trace
-        if {$asynchronous} {                                                           ;# eventually simulate asynchronous operation
-            after [expr {2000 + round(rand() * 8000)}] random::update
+        if {[package vcompare $::tcl_version 8.4] < 0} {
+            array set data "
+                4294967295,0 Laura 4294967295,1 [format %.1f [expr {rand() * 30}]] 4294967295,2 [expr {100 + int(rand() * 50)}]
+                    4294967295,3 [expr {10 + int(rand() * 50)}] 4294967295,4 cc
+            "
+        } else {                                                                                     ;# Tcl 8.4, with 64 bit support
+            array set data "
+                18446744073709551615,0 Laura 18446744073709551615,1 [format %.1f [expr {rand() * 30}]]
+                    18446744073709551615,2 [expr {100 + int(rand() * 50)}] 18446744073709551615,3 [expr {10 + int(rand() * 50)}]
+                    18446744073709551615,4 cc
+            "
+        }
+        incr data(updates)                                   ;# increment updates counter so that update can be detected by the core
+        if {$asynchronous} {                                                         ;# simulate asynchronous operation if requested
+            after [expr {2000 + round(rand() * 9000)}] random::update
         }
     }
 }
