@@ -1,14 +1,15 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: smithy.tcl,v 1.24 2004/01/01 11:39:06 jfontain Exp $
+# $Id: smithy.tcl,v 1.31 2005/01/02 00:45:07 jfontain Exp $
 
 
-package provide smithy [lindex {$Revision: 1.24 $} 1]
+package provide smithy [lindex {$Revision: 1.31 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
 package require miscellaneous
+package require hashes
 
 #rename smilib _smilib; proc smilib {args} {puts "smilib $args"; eval _smilib $args}
 #rename snmplib _snmplib; proc snmplib {args} {puts "snmplib $args"; eval _snmplib $args}
@@ -25,15 +26,13 @@ proc listRemovedDuplicates {list} {
 
 namespace eval smithy {
 
-    variable nextRow 0
-
     array set data {
         updates 0
         switches {
-            -a 1 --address 1 --authpasswd 1 --authproto 1 --community 1 --directory 1 -i 1 --identifiers 1 --keyfile 1 --mibs 1
-            --port 1 --privpasswd 1 --privproto 1 --retries 1 -t 1 --table 1 --timeout 1 --title 1 --trace 0 --trim 1 --username 1
-            --version 1
+            -a 1 --address 1 --authpasswd 1 --authproto 1 --community 1 --directory 1 -i 1 --identifiers 1 --mibs 1 --port 1
+            --privpasswd 1 --privproto 1 --retries 1 -t 1 --table 1 --timeout 1 --title 1 --trace 0 --trim 1 --username 1 --version 1
         }
+        persistent 1 64Bits 1
         pollTimes {10 5 20 30 60 120 300}
     }
     set file [open smithy.htm]
@@ -41,7 +40,7 @@ namespace eval smithy {
     close $file
 
     proc reportError {identifier message} {
-        if {[string length $identifier] > 0} {
+        if {$identifier ne ""} {
             set text "mib: [smilib get -filename [lindex [split [smilib get -fullname $identifier] !] 0]]\n"
         }
         append text $message
@@ -54,14 +53,13 @@ namespace eval smithy {
         variable session
         variable trace
 
-        catch {set ::env(SMITHY_LICENSE_FILE) $options(--keyfile)}          ;# eventually use license key in a non default directory
         if {[catch {
-            package require smilib 2.1                                                                 ;# then load needed libraries
-            package require snmplib 2.1
+            package require smilib                                                                     ;# then load needed libraries
+            package require snmplib
         }]} {
             set message {Smithy package initialization failed}
             set error [string trim [lindex [split $::errorInfo \n] 0]]                ;# only keep the first line of the error trace
-            if {[string length $error] > 0} {
+            if {$error ne ""} {
                 append message ": $error"
             }           ;# else it is probably a license problem, in which case error information goes to the standard error channel
             error $message
@@ -87,15 +85,17 @@ namespace eval smithy {
         if {[info exists table]} {
             processTableIdentifiers $table $string $trim
         } else {
-            if {[string length $string] == 0} {
+            if {$string eq ""} {
                 reportError {} {table and/or list of identifiers must be specified}
             }
             processIdentifiers $string $trim
         }
         set arguments {}
-        if {![info exists options(-a)]} {set options(-a) 127.0.0.1}                                                    ;# by default
-        catch {set arguments [list -remoteaddress $options(-a)]}
-        catch {set arguments [list -remoteaddress $options(--address)]}                                         ;# favor long option
+
+        set address 127.0.0.1                                                                                          ;# by default
+        catch {set address $options(-a)}
+        catch {set address $options(--address)}                                                                 ;# favor long option
+        set arguments [list -remoteaddress $address]                                                            ;# favor long option
         # convert from original scotty snmp switches to smithy switches:
         array set map {
             authpasswd authpass authproto authproto community readcommunity port remoteport privpasswd privpass privproto privproto
@@ -120,7 +120,6 @@ namespace eval smithy {
         set session [snmplib new]
         eval $session config $arguments
         if {[info exists options(--title)]} {
-            set address $options(-a)
             if {[string match *t* $options(--title)] && ![info exists table]} {
                 reportError {} {--title option including table name but no table was specified (-t or --table option)}
             }
@@ -137,7 +136,7 @@ namespace eval smithy {
     }
 
     proc validateIdentifier {name {exact 1}} {                          ;# report an error if identifier is not part of a loaded MIB
-        if {$exact && ([string length [smilib get -oidsuffix $name]] > 0)} {
+        if {$exact && ([smilib get -oidsuffix $name] ne "")} {
             set message "$name must not contain sub-identifiers"
             set invalid 1
         } else {
@@ -168,7 +167,7 @@ namespace eval smithy {
                     set parent $identifier
                     set invalid 1
                     foreach identifier $nodes {
-                        if {![string equal [smilib get -access $identifier] not-accessible]} {         ;# ignore tables, groups, ...
+                        if {[smilib get -access $identifier] ne "not-accessible"} {                    ;# ignore tables, groups, ...
                             lappend identifiers $identifier
                             set invalid 0
                         }
@@ -183,8 +182,7 @@ namespace eval smithy {
             lappend views $identifiers
         }
         set requestIdentifiers {}
-        # use an empty hidden column as index since there is only a single row
-        array set data {0,label {} 0,type ascii 0,message {} 0,0 {}}
+        array set data {0,label {} 0,type ascii 0,message {}}   ;# use an empty hidden column as index as there is only a single row
         set nextColumn 1
         foreach list $views {
             set columns {}
@@ -195,7 +193,7 @@ namespace eval smithy {
                     if {[regexp {\.\d+$} $identifier]} {                                                             ;# instanciated
                         lappend requestIdentifiers $identifier
                     } else {
-                        lappend requestIdentifiers $identifier.0                     ;# eventually complete with instance identifier
+                        lappend requestIdentifiers $identifier.0                       ;# possibly complete with instance identifier
                     }
                 }
                 if {[catch {set column $identifierColumn($index)}]} {         ;# identifiers thus columns may be duplicated in views
@@ -204,7 +202,7 @@ namespace eval smithy {
                     incr nextColumn
                 }
                 lappend indexColumns($index) $column                                      ;# there can be identical columns in views
-                regsub ^$trim $identifier {} data($column,label)            ;# eventually trim string from left side of column title
+                regsub ^$trim $identifier {} data($column,label)                       ;# trim string from left side of column title
                 # no strict lookup since identifier can be instantiated
                 set data($column,type) [identifierType $identifier delta($index) 0]
                 set data($column,message) [identifierMessage $identifier $delta($index)]
@@ -238,7 +236,7 @@ namespace eval smithy {
 
         # check that this is a proper table
         validateIdentifier $table
-        if {[catch {set indexes [smilib get -index $table]}] || ![string equal [smilib get -asn1type $table] {SEQUENCE OF}]} {
+        if {[catch {set indexes [smilib get -index $table]}] || ([smilib get -asn1type $table] ne "SEQUENCE OF")} {
             reportError $table "$table: not a table"                                       ;# must have an index but not be an entry
         }
         set indexLength [llength $indexes]
@@ -253,7 +251,7 @@ namespace eval smithy {
         # see whether table index is accessible as a whole
         set accessible 1
         foreach identifier $indexes {
-            if {[string equal [smilib get -access $identifier] not-accessible]} {
+            if {[smilib get -access $identifier] eq "not-accessible"} {
                 set accessible 0
                 break
             }
@@ -270,7 +268,7 @@ namespace eval smithy {
         }
 
         # generate view(s) in the form of list(s) of object identifiers and internally generated columns
-        if {[string length $format] == 0} {                                            ;# no identifiers format: display all columns
+        if {$format eq ""} {                                                           ;# no identifiers format: display all columns
             # process index(es) first
             if {$indexLength > 1} {                                                                         ;# multiple column index
                 # include a row number column so that, for example, different views can be identically sorted
@@ -323,7 +321,7 @@ namespace eval smithy {
                         set data($column,message) {row creation order}
                     }
                     default {                                                                                   ;# normal identifier
-                        regsub ^$trim $identifier {} data($column,label)    ;# eventually trim string from left side of column title
+                        regsub ^$trim $identifier {} data($column,label)               ;# trim string from left side of column title
                         set type [identifierType $identifier counter($identifier)]   ;# remember whether the identifier is a counter
                         set data($column,type) $type
                         set data($column,message) [identifierMessage $identifier $counter($identifier)]
@@ -369,8 +367,9 @@ namespace eval smithy {
         }
         set requestLength [incr index]                                                                    ;# including system uptime
 
-        # generate view(s) containing visible columns, a subset of all representation table columns
-        if {[string length $format] > 0} {                                                   ;# no views needed for whole table dump
+        if {$format eq ""} {                                                                 ;# no views needed for whole table dump
+            if {$indexLength == 1} {set data(sort) {0 increasing}}                           ;# single column index used for sorting
+        } else {                        ;# generate view(s) containing visible columns, a subset of all representation table columns
             set column 0
             foreach identifiers $views {
                 set columns {}
@@ -388,7 +387,7 @@ namespace eval smithy {
         upvar 1 $counterName counter
 
         set counter 0
-        if {$exact && ([string length [smilib get -oidsuffix $name]] > 0)} {
+        if {$exact && ([smilib get -oidsuffix $name] ne "")} {
             error "$name must not contain sub-identifiers"
         }
         set syntax [smilib get -syntax $name]
@@ -423,13 +422,14 @@ namespace eval smithy {
     proc update {} {
         variable requestIdentifiers
         variable indexLength
-        variable currentRow
         variable busy
+        variable bulkData
+        variable bulkType
 
         if {[info exist busy]} return                                        ;# wait till request is complete before sending another
         set busy {}
         if {[info exists indexLength]} {                                                                                    ;# table
-            catch {unset currentRow}
+            catch {unset bulkData bulkType}                                                                  ;# reset responses data
             getBulk $requestIdentifiers
         } else {
             get $requestIdentifiers
@@ -463,7 +463,7 @@ namespace eval smithy {
         foreach identifier $list {
             append string " [smilib get -name $identifier]"
             set suffix [smilib get -oidsuffix $identifier]
-            if {[string length $suffix] > 0} {
+            if {$suffix ne ""} {
                 append string .$suffix
             }
         }
@@ -476,7 +476,7 @@ namespace eval smithy {
             foreach {identifier type value} $object {}
             append string " [smilib get -name $identifier]"
             set suffix [smilib get -oidsuffix $identifier]
-            if {[string length $suffix] > 0} {
+            if {$suffix ne ""} {
                 append string .$suffix
             }
             append string ($value)
@@ -503,7 +503,7 @@ namespace eval smithy {
             }
             puts {}
         }
-        if {![string equal $result(-status) noError]} {
+        if {$result(-status) ne "noError"} {
             set index 0; catch {set index $result(-eindex)}
             set list {}; catch {set list $result(-varbinds)}
             processError $result(-status) $index $list
@@ -522,19 +522,20 @@ namespace eval smithy {
             foreach object $objects {                                                                     ;# now fill row data cells
                 foreach {identifier type value} $object {}
                 if {$delta($index)} {
-                    set current $value
+                    set new $value
                     if {[info exists period] && ($period > 0) && [info exists last($index)]} {
                         # display deltas per second for counters
-                        set value [format %.2f [expr {int($current - $last($index)) / $period}]]
+                        set value [format %.2f [expr {int($new - $last($index)) / $period}]]
                     } else {
                         set value ?                                                   ;# need at least 2 values to make a difference
                     }
-                    set last($index) $current
-                } elseif {[string equal $type {OBJECT IDENTIFIER}]} {
+                    set last($index) $new
+                } elseif {$type eq "OBJECT IDENTIFIER"} {
                     catch {set value [smilib get -name $value]}                              ;# attempt conversion to readable value
-                } elseif {[string equal $type TimeTicks]} {
+                } elseif {$type eq "TimeTicks"} {
                     set value [formattedTimeTicks $value]
                 }
+                set data(0,0) {}       ;# update hidden cell which may have been deleted if there was no returned data at least once
                 foreach column $indexColumns($index) {
                     set data(0,$column) $value                                                                ;# there is only 1 row
                 }
@@ -549,18 +550,12 @@ namespace eval smithy {
         variable requestLength
         variable session
         variable requestOids
-        variable indexLength
-        variable cachedIndexRow
-        variable indexRow
-        variable nextRow
-        variable currentRow
-        variable indexColumns
-        variable last
         variable data
-        variable delta
+        variable last
         variable trace
         variable busy
-        variable accessible
+        variable bulkData
+        variable bulkType
 
         array set result $args
         if {[info exists trace]} {
@@ -572,7 +567,11 @@ namespace eval smithy {
         }
         set error [string compare $result(-status) noError]
         if {$error || ([llength $result(-varbinds)] != $requestLength)} {               ;# number of responses differs from requests
-            cleanup
+            foreach name [array names data *,0] {                                                                   ;# total cleanup
+                set row [lindex [split $name ,] 0]
+                array unset data $row,\[0-9\]*
+                array unset last $row,\[0-9\]*
+            }
             incr data(updates)
             if {$error} {
                 flashMessage "error: $result(-status) from [$session cget -remoteaddress]"
@@ -581,87 +580,88 @@ namespace eval smithy {
             return                                                                                                           ;# done
         }
         set time [expr {[lindex [lindex $result(-varbinds) 0] end] / 100.0}]
-        set objects [lrange $result(-varbinds) 1 end]                                                        ;# remove system uptime
-
+        set next 0                                                                      ;# whether to continue retrieving table data
+        set identifiers {}
         set index 0
-        foreach object $objects requested $requestOids {
+        foreach object [lrange $result(-varbinds) 1 end] requested $requestOids {                              ;# skip system uptime
             catch {unset value}
             foreach {identifier type value} $object {}
-            if {($index == 0) && (([string first $requested $identifier] != 0) || ![info exists value])} {
-                # end of table passed if first oid is not a successor of the request oid or object incomplete (for resiliency sake)
-                cleanup
-                incr data(updates)
-                unset busy
-                return                                                                                                       ;# done
-            }
-            if {$index == 0} {                                                                      ;# use first oid instance as key
+            if {([string first $requested $identifier] == 0) && ($requested ne $identifier) && [info exists value]} {
+                # oid is a successor but not equal to the request oid and object complete (for resiliency sake)
                 set key [smilib get -oidsuffix $identifier]
+                set bulkData($key,$index) [list $time $value]                 ;# store time and value corresponding to key and index
+                if {![info exists bulkType($index)]} {set bulkType($index) $type}
+                set next 1                                                   ;# keep retrieving data until all columns are completed
             }
-            incr index
-        }
-        if {[catch {set row $indexRow($key)}]} {                                                                          ;# new row
-            if {[catch {set row $cachedIndexRow($key)}]} {         ;# try to use old index so eventual viewers can resume displaying
-                set row $nextRow
-                set cachedIndexRow($key) $row
-                incr nextRow
-            }
-            set indexRow($key) $row
-            if {$indexLength > 1} {
-                set value [expr {$row + 1}]                                                     ;# creation order column starts at 1
-                foreach column $indexColumns(key) {set data($row,$column) $value}
-            } elseif {!$accessible} {                                                          ;# not accessible single column index
-                foreach column $indexColumns(key) {set data($row,$column) $key}
-            }
-        } else {
-            set period [expr {$time - $last($row,time)}]
-        }
-        set currentRow($row) {}                                                            ;# keep track of current rows for cleanup
-        set index 0
-        set identifiers {}
-        foreach object $objects requested $requestOids {                                                  ;# now fill row data cells
-            foreach {identifier type value} $object {}
             lappend identifiers $identifier
-            if {([string first $requested $identifier] != 0) || ![info exists value]} {
-                # oid is not a successor of the request oid or object incomplete (for resiliency sake): use void value for type
-                switch $data([lindex $indexColumns($index) 0],type) {
-                    integer - real {set value ?}
-                    default {set value {}}
-                }
-            } elseif {$delta($index)} {
-                set current $value
-                if {[info exists period] && ($period > 0)} {                               ;# display deltas per second for counters
-                    set value [format %.2f [expr {int($current - $last($row,$index)) / $period}]]
-                } else {
-                    set value ?                                                       ;# need at least 2 values to make a difference
-                }
-                set last($row,$index) $current
-            } elseif {[string equal $type {OBJECT IDENTIFIER}]} {
-                catch {set value [smilib get -name $value]}                                  ;# attempt conversion to readable value
-            } elseif {[string equal $type TimeTicks]} {
-                set value [formattedTimeTicks $value]
-            }
-            foreach column $indexColumns($index) {
-                set data($row,$column) $value                                                ;# fill the columns for that identifier
-            }
             incr index
         }
-        set last($row,time) $time
-        getBulk $identifiers                                                                         ;# keep going till end of table
+        if {$next} {                                                                ;# at least 1 column is not completely retrieved
+            getBulk $identifiers                                                                     ;# keep going till end of table
+        } else {                                                                                                             ;# done
+            processBulkData $index
+            incr data(updates)
+            unset busy
+        }
     }
 
-    proc cleanup {} {                                                                 ;# remove disappeared rows and associated data
-        variable currentRow
-        variable indexRow
-        variable data
+    proc processBulkData {maximumIndex} {
+        variable indexLength
+        variable indexColumns
         variable last
+        variable data
+        variable delta
+        variable accessible
+        variable bulkData
+        variable bulkType
 
-        foreach {key row} [array get indexRow] {
-            if {[info exists currentRow($row)]} continue
-            unset indexRow($key)
+        foreach name [array names bulkData] {                                                                 ;# determine data keys
+            set keys([lindex [split $name ,] 0]) {}
+        }
+        foreach key [lsort -dictionary [array names keys]] {
+            set row [hash64::numbers32 [split $key .]]
+            if {![info exists data($row,0)]} {
+                if {$indexLength > 1} {
+                    set value [llength [array names data *,0]]
+                    incr value                                                                  ;# creation order column starts at 1
+                    foreach column $indexColumns(key) {set data($row,$column) $value}
+                } elseif {!$accessible} {                                                      ;# not accessible single column index
+                    foreach column $indexColumns(key) {set data($row,$column) $key}
+                }
+            }
+            set current($row) {}                                                           ;# keep track of current rows for cleanup
+            for {set index 0} {$index < $maximumIndex} {incr index} {
+                if {[catch {foreach {time value} $bulkData($key,$index) {}}]} {             ;# missing cell: use void value for type
+                    switch $data([lindex $indexColumns($index) 0],type) {
+                        integer - real {set value ?}
+                        default {set value {}}
+                    }
+                } elseif {$delta($index)} {
+                    set new $value
+                    if {![catch {set period [expr {$time - $last($row,$index,time)}]}] && ($period > 0)} {
+                        # display deltas per second for counters
+                        set value [format %.2f [expr {int($new - $last($row,$index)) / $period}]]
+                    } else {
+                        set value ?                                                   ;# need at least 2 values to make a difference
+                    }
+                    set last($row,$index) $new
+                    set last($row,$index,time) $time
+                } elseif {$bulkType($index) eq "OBJECT IDENTIFIER"} {
+                    catch {set value [smilib get -name $value]}                              ;# attempt conversion to readable value
+                } elseif {$bulkType($index) eq "TimeTicks"} {
+                    set value [formattedTimeTicks $value]
+                }
+                foreach column $indexColumns($index) {
+                    set data($row,$column) $value                                            ;# fill the columns for that identifier
+                }
+            }
+        }
+        foreach name [array names data *,0] {                                                         ;# cleanup disappeared entries
+            set row [lindex [split $name ,] 0]
+            if {[info exists current($row)]} continue
             array unset data $row,\[0-9\]*
-            array unset last $row,*
+            array unset last $row,\[0-9\]*
         }
-        catch {unset currentRow}
     }
 
     proc processError {status errorIndex objects} {
@@ -684,7 +684,7 @@ namespace eval smithy {
     }
 
     proc mibFiles {directory names} {
-        if {[string length $directory] > 0} {
+        if {$directory ne ""} {
             if {![file isdirectory $directory]} {
                 error "$directory: not a directory"
             }
@@ -694,11 +694,11 @@ namespace eval smithy {
         }
         set files {}
         foreach name $names {
-            if {[string length [file extension $name]] == 0} {
+            if {[file extension $name] eq ""} {
                 append name .mib                                           ;# complete with default file name extension if necessary
             }
-            if {([string length $directory] > 0) && [string equal [file dirname $name] .]} {
-                set name [file join $directory $name]             ;# eventually join file without directory with specified directory
+            if {($directory ne "") && ([file dirname $name] eq ".")} {
+                set name [file join $directory $name]               ;# possibly join file without directory with specified directory
             }
             lappend files $name
         }
