@@ -1,10 +1,10 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: snmp.tcl,v 2.42 2004/01/01 11:39:06 jfontain Exp $
+# $Id: snmp.tcl,v 2.52 2005/01/02 00:45:07 jfontain Exp $
 
 
-package provide snmp [lindex {$Revision: 2.42 $} 1]
+package provide snmp [lindex {$Revision: 2.52 $} 1]
 if {[lsearch -exact $auto_path /usr/lib] < 0} {                       ;# in case Tcl/Tk is somewhere else than in the /usr hierarchy
     lappend auto_path /usr/lib
 }
@@ -17,6 +17,7 @@ if {$tnm3} {
     namespace import Tnm::*
     package require miscellaneous
 }
+package require hashes
 
 proc listRemovedDuplicates {list} {
     set return {}
@@ -30,14 +31,13 @@ proc listRemovedDuplicates {list} {
 
 namespace eval snmp {
 
-    variable nextRow 0
-
     array set data {
         updates 0
         switches {
             -a 1 --address 1 --community 1 --context 1 --delay 1 --directory 1 -i 1 --identifiers 1 --mibs 1 --port 1 --password 1
             --retries 1 -t 1 --table 1 --timeout 1 --title 1 --trace 0 --trim 1 --user 1 --version 1 --window 1
         }
+        persistent 1 64Bits 1
         pollTimes {10 5 20 30 60 120 300}
     }
     set file [open snmp.htm]
@@ -47,12 +47,11 @@ namespace eval snmp {
 if {$::tnm3} {
 
     proc mib {args} {                                      ;# override original command since -exact switch has disappeared in Tnm 3
-        if {[string equal [lindex $args 0] -exact]} {
+        if {[lindex $args 0] eq "-exact"} {
             # emulate the -exact function for the following sub-command used in this module (add more if necessary):
             switch [lindex $args 1] {children - index - parent - syntax {
-                if {[string length [lindex [Tnm::mib split [lindex $args 2]] end]] > 0} {
+                if {[lindex [Tnm::mib split [lindex $args 2]] end] ne ""} {
                     error "instance identifier not allowed: $args"
-                    error {instance identifier not allowed}
                 }
             }}
             return [eval Tnm::mib [lrange $args 1 end]]
@@ -64,7 +63,7 @@ if {$::tnm3} {
 }
 
     proc reportError {identifier message} {
-        if {[string length $identifier] > 0} {
+        if {$identifier ne ""} {
             set text "mib: [mib file $identifier]\n"
         }
         append text $message
@@ -98,15 +97,16 @@ if {$::tnm3} {
         if {[info exists table]} {
             processTableIdentifiers $table $string $trim
         } else {
-            if {[string length $string] == 0} {
+            if {$string eq ""} {
                 reportError {} {table and/or list of identifiers must be specified}
             }
             processIdentifiers $string $trim
         }
         set arguments {}
-        if {![info exists options(-a)]} {set options(-a) 127.0.0.1}                                                    ;# by default
-        catch {set arguments [list -address $options(-a)]}
-        catch {set arguments [list -address $options(--address)]}                                               ;# favor long option
+        set address 127.0.0.1                                                                                          ;# by default
+        catch {set address $options(-a)}
+        catch {set address $options(--address)}                                                                 ;# favor long option
+        set arguments [list -address $address]
         foreach switch {community context delay port password retries timeout window} {
             catch {lappend arguments -$switch $options(--$switch)}
         }
@@ -126,7 +126,6 @@ if {$::tnm3} {
             set session [eval snmp session $arguments]
         }
         if {[info exists options(--title)]} {
-            set address $options(-a)
             if {[string match *t* $options(--title)] && ![info exists table]} {
                 reportError {} {--title option including table name but no table was specified (-t or --table option)}
             }
@@ -173,11 +172,11 @@ if {$::tnm3} {
                 if {[llength [mib index $identifier]] > 0} {
                     reportError $identifier "$identifier is a table: use the -t (--table) switch"
                 }
-                if {[string equal [mib access $identifier] not-accessible]} {                               ;# object may be a group
+                if {[mib access $identifier] eq "not-accessible"} {                                         ;# object may be a group
                     set parent $identifier
                     set invalid 1
                     foreach identifier [mib $command $parent] {                                  ;# lookup immediate successors only
-                        if {![string equal [mib access $identifier] not-accessible]} {                 ;# ignore tables, groups, ...
+                        if {[mib access $identifier] ne "not-accessible"} {                            ;# ignore tables, groups, ...
                             if {$::tnm3} {
                                 lappend identifiers [mib label $identifier]                                     ;# remove MIB header
                             } else {
@@ -196,8 +195,7 @@ if {$::tnm3} {
             lappend views $identifiers
         }
         set requestIdentifiers {}
-        # use an empty hidden column as index since there is only a single row
-        array set data {0,label {} 0,type ascii 0,message {} 0,0 {}}
+        array set data {0,label {} 0,type ascii 0,message {}}   ;# use an empty hidden column as index as there is only a single row
         set nextColumn 1
         foreach list $views {
             set columns {}
@@ -208,7 +206,7 @@ if {$::tnm3} {
                     if {[regexp {\.\d+$} $identifier]} {                                                             ;# instanciated
                         lappend requestIdentifiers $identifier
                     } else {
-                        lappend requestIdentifiers $identifier.0                     ;# eventually complete with instance identifier
+                        lappend requestIdentifiers $identifier.0                                ;# complete with instance identifier
                     }
                 }
                 if {[catch {set column $identifierColumn($index)}]} {         ;# identifiers thus columns may be duplicated in views
@@ -217,7 +215,7 @@ if {$::tnm3} {
                     incr nextColumn
                 }
                 lappend indexColumns($index) $column                                      ;# there can be identical columns in views
-                regsub ^$trim $identifier {} data($column,label)            ;# eventually trim string from left side of column title
+                regsub ^$trim $identifier {} data($column,label)                       ;# trim string from left side of column title
                 # no strict lookup since identifier can be instantiated
                 set data($column,type) [identifierType $identifier delta($index) 0]
                 set data($column,message) [identifierMessage $identifier $delta($index)]
@@ -258,12 +256,12 @@ if {$::tnm3} {
                 lappend list [mib label $identifier]                                                    ;# use readable name not oid
             }
             set indexes $list
-        } elseif {([llength $indexes] == 1) && [string equal [mib syntax $indexes] SEQUENCE]} {
+        } elseif {([llength $indexes] == 1) && ([mib syntax $indexes] eq "SEQUENCE")} {
             # if table is an extension, Tnm 2 returns the entry as index, so go one level deeper to get at the index(es)
             set indexes [mib -exact index $indexes]
         }
         set indexLength [llength $indexes]
-        if {($indexLength == 0) || ![string equal [mib syntax $table] {SEQUENCE OF}]} {
+        if {($indexLength == 0) || ([mib syntax $table] ne "SEQUENCE OF")} {
             reportError $table "$table: not a table"                                       ;# must have an index but not be an entry
         }
         if {$::tnm3} {set command children} else {set command successor}
@@ -275,7 +273,7 @@ if {$::tnm3} {
         # see whether table index is accessible as a whole
         set accessible 1
         foreach identifier $indexes {
-            if {[string equal [mib access $identifier] not-accessible]} {
+            if {[mib access $identifier] eq "not-accessible"} {
                 set accessible 0
                 break
             }
@@ -293,7 +291,7 @@ if {$::tnm3} {
         }
 
         # generate view(s) in the form of list(s) of object identifiers and internally generated columns
-        if {[string length $format] == 0} {                                            ;# no identifiers format: display all columns
+        if {$format eq ""} {                                                           ;# no identifiers format: display all columns
             # process index(es) first
             if {$indexLength > 1} {                                                                         ;# multiple column index
                 # include a row number column so that, for example, different views can be identically sorted
@@ -325,7 +323,7 @@ if {$::tnm3} {
                 foreach identifier [split $list ,] {                                                            ;# check identifiers
                     if {[lsearch -exact $identifiers $identifier] >= 0} continue                      ;# already in identifiers list
                     validateIdentifier $identifier
-                    if {![string equal $entry [mib -exact parent $identifier]] && ([lsearch -exact $indexes $identifier] < 0)} {
+                    if {($entry ne [mib -exact parent $identifier]) && ([lsearch -exact $indexes $identifier] < 0)} {
                         # note: index identifiers can belong to another table
                         reportError $identifier "$identifier is not a column of table $table"
                     }
@@ -347,7 +345,7 @@ if {$::tnm3} {
                         set data($column,message) {row creation order}
                     }
                     default {                                                                                   ;# normal identifier
-                        regsub ^$trim $identifier {} data($column,label)    ;# eventually trim string from left side of column title
+                        regsub ^$trim $identifier {} data($column,label)               ;# trim string from left side of column title
                         set type [identifierType $identifier counter($identifier)]   ;# remember whether the identifier is a counter
                         set data($column,type) $type
                         set data($column,message) [identifierMessage $identifier $counter($identifier)]
@@ -393,8 +391,9 @@ if {$::tnm3} {
         }
         set requestLength [incr index]                                                                    ;# including system uptime
 
-        # generate view(s) containing visible columns, a subset of all representation table columns
-        if {[string length $format] > 0} {                                                   ;# no views needed for whole table dump
+        if {$format eq ""} {                                                                 ;# no views needed for whole table dump
+            if {$indexLength == 1} {set data(sort) {0 increasing}}                           ;# single column index used for sorting
+        } else {                        ;# generate view(s) containing visible columns, a subset of all representation table columns
             set column 0
             foreach identifiers $views {
                 set columns {}
@@ -448,13 +447,14 @@ if {$::tnm3} {
     proc update {} {
         variable requestIdentifiers
         variable indexLength
-        variable currentRow
         variable busy
+        variable bulkData
+        variable bulkType
 
         if {[info exist busy]} return                                        ;# wait till request is complete before sending another
         set busy {}
         if {[info exists indexLength]} {                                                                                    ;# table
-            catch {unset currentRow}
+            catch {unset bulkData bulkType}                                                                  ;# reset responses data
             getBulk $requestIdentifiers
         } else {
             get $requestIdentifiers
@@ -523,7 +523,7 @@ if {$::tnm3} {
         if {[info exists trace]} {
             puts "<<< response($status):[formattedObjects $objects]"
         }
-        if {![string equal $status noError]} {
+        if {$status ne "noError"} {
             processError $status $errorIndex $objects
         }
         # objects list could be empty or system uptime value missing (may happen in "no such name" error cases):
@@ -540,22 +540,23 @@ if {$::tnm3} {
             foreach object $objects {                                                                     ;# now fill row data cells
                 foreach {identifier type value} $object {}
                 if {$delta($index)} {
-                    set current $value
+                    set new $value
                     if {[info exists period] && ($period > 0) && [info exists last($index)]} {
                         # display deltas per second for counters
-                        set value [format %.2f [expr {int($current - $last($index)) / $period}]]
+                        set value [format %.2f [expr {int($new - $last($index)) / $period}]]
                     } else {
                         set value ?                                                   ;# need at least 2 values to make a difference
                     }
-                    set last($index) $current
-                } elseif {[string equal $type {OBJECT IDENTIFIER}]} {
+                    set last($index) $new
+                } elseif {$type eq "OBJECT IDENTIFIER"} {
                     catch {set value [mib name $value]}                                      ;# attempt conversion to readable value
-                    if {$::tnm3 && [string equal [mib syntax $identifier] {OBJECT IDENTIFIER}]} {
+                    if {$::tnm3 && ([mib syntax $identifier] eq "OBJECT IDENTIFIER")} {
                         set value [mib label $value]                                                    ;# do not display MIB header
                     }
-                } elseif {[string equal $type TimeTicks]} {
+                } elseif {$type eq "TimeTicks"} {
                     set value [formattedTimeTicks $value]
                 }
+                set data(0,0) {}       ;# update hidden cell which may have been deleted if there was no returned data at least once
                 foreach column $indexColumns($index) {
                     set data(0,$column) $value                                                                ;# there is only 1 row
                 }
@@ -570,25 +571,23 @@ if {$::tnm3} {
         variable requestLength
         variable session
         variable requestOids
-        variable indexLength
-        variable cachedIndexRow
-        variable indexRow
-        variable nextRow
-        variable currentRow
-        variable indexColumns
-        variable last
         variable data
-        variable delta
+        variable last
         variable trace
         variable busy
-        variable accessible
+        variable bulkData
+        variable bulkType
 
         if {[info exists trace]} {
             puts "<<< response($status):[formattedObjects $objects]"
         }
         set error [string compare $status noError]
         if {$error || ([llength $objects] != $requestLength)} {                         ;# number of responses differs from requests
-            cleanup
+            foreach name [array names data *,0] {                                                                   ;# total cleanup
+                set row [lindex [split $name ,] 0]
+                array unset data $row,\[0-9\]*
+                array unset last $row,\[0-9\]*
+            }
             incr data(updates)
             if {$error} {
                 flashMessage "error: $status from [$session cget -address]"
@@ -597,90 +596,92 @@ if {$::tnm3} {
             return                                                                                                           ;# done
         }
         set time [secondsFromSystemUptime [lindex $objects 0]]
-        set objects [lrange $objects 1 end]                                                                  ;# remove system uptime
-
+        set next 0                                                                      ;# whether to continue retrieving table data
+        set identifiers {}
+        set expression {\.(.*)$}
         set index 0
-        foreach object $objects requested $requestOids {
+        foreach object [lrange $objects 1 end] requested $requestOids {                                        ;# skip system uptime
             catch {unset value}
             foreach {identifier type value} $object {}
-            if {($index == 0) && (([string first $requested $identifier] != 0) || ![info exists value])} {
-                # end of table passed if first oid is not a successor of the request oid or object incomplete (for resiliency sake)
-                cleanup
-                incr data(updates)
-                unset busy
-                return                                                                                                       ;# done
+            if {([string first $requested $identifier] == 0) && ($requested ne $identifier) && [info exists value]} {
+                # oid is a successor but not equal to the request oid and object complete (for resiliency sake)
+                regexp $expression [mib name $identifier] dummy key
+                set bulkData($key,$index) [list $time $value]                 ;# store time and value corresponding to key and index
+                if {![info exists bulkType($index)]} {set bulkType($index) $type}
+                set next 1                                                   ;# keep retrieving data until all columns are completed
             }
-            if {$index == 0} {                                                                      ;# use first oid instance as key
-                regexp {\.(.*)$} [mib name $identifier] dummy key
-            }
-            incr index
-        }
-        if {[catch {set row $indexRow($key)}]} {                                                                          ;# new row
-            if {[catch {set row $cachedIndexRow($key)}]} {         ;# try to use old index so eventual viewers can resume displaying
-                set row $nextRow
-                set cachedIndexRow($key) $row
-                incr nextRow
-            }
-            set indexRow($key) $row
-            if {$indexLength > 1} {
-                set value [expr {$row + 1}]                                                     ;# creation order column starts at 1
-                foreach column $indexColumns(key) {set data($row,$column) $value}
-            } elseif {!$accessible} {                                                          ;# not accessible single column index
-                foreach column $indexColumns(key) {set data($row,$column) $key}
-            }
-        } else {
-            set period [expr {$time - $last($row,time)}]
-        }
-        set currentRow($row) {}                                                            ;# keep track of current rows for cleanup
-        set index 0
-        set identifiers {}
-        foreach object $objects requested $requestOids {                                                  ;# now fill row data cells
-            foreach {identifier type value} $object {}
             lappend identifiers $identifier
-            if {([string first $requested $identifier] != 0) || ![info exists value]} {
-                # oid is not a successor of the request oid or object incomplete (for resiliency sake): use void value for type
-                switch $data([lindex $indexColumns($index) 0],type) {
-                    integer - real {set value ?}
-                    default {set value {}}
-                }
-            } elseif {$delta($index)} {
-                set current $value
-                if {[info exists period] && ($period > 0)} {                               ;# display deltas per second for counters
-                    set value [format %.2f [expr {int($current - $last($row,$index)) / $period}]]
-                } else {
-                    set value ?                                                       ;# need at least 2 values to make a difference
-                }
-                set last($row,$index) $current
-            } elseif {[string equal $type {OBJECT IDENTIFIER}]} {
-                catch {set value [mib name $value]}                                          ;# attempt conversion to readable value
-                if {$::tnm3 && [string equal [mib syntax $identifier] {OBJECT IDENTIFIER}]} {
-                    set value [mib label $value]                                                        ;# do not display MIB header
-                }
-            } elseif {[string equal $type TimeTicks]} {
-                set value [formattedTimeTicks $value]
-            }
-            foreach column $indexColumns($index) {
-                set data($row,$column) $value                                                ;# fill the columns for that identifier
-            }
             incr index
         }
-        set last($row,time) $time
-        getBulk $identifiers                                                                         ;# keep going till end of table
+        if {$next} {                                                                ;# at least 1 column is not completely retrieved
+            getBulk $identifiers                                                                     ;# keep going till end of table
+        } else {                                                                                                             ;# done
+            processBulkData $index
+            incr data(updates)
+            unset busy
+        }
     }
 
-    proc cleanup {} {                                                                 ;# remove disappeared rows and associated data
-        variable currentRow
-        variable indexRow
-        variable data
+    proc processBulkData {maximumIndex} {
+        variable indexLength
+        variable indexColumns
         variable last
+        variable data
+        variable delta
+        variable accessible
+        variable bulkData
+        variable bulkType
 
-        foreach {key row} [array get indexRow] {
-            if {[info exists currentRow($row)]} continue
-            unset indexRow($key)
+        foreach name [array names bulkData] {                                                                 ;# determine data keys
+            set keys([lindex [split $name ,] 0]) {}
+        }
+        foreach key [lsort -dictionary [array names keys]] {
+            set row [hash64::numbers32 [split $key .]]
+            if {![info exists data($row,0)]} {
+                if {$indexLength > 1} {
+                    set value [llength [array names data *,0]]
+                    incr value                                                                  ;# creation order column starts at 1
+                    foreach column $indexColumns(key) {set data($row,$column) $value}
+                } elseif {!$accessible} {                                                      ;# not accessible single column index
+                    foreach column $indexColumns(key) {set data($row,$column) $key}
+                }
+            }
+            set current($row) {}                                                           ;# keep track of current rows for cleanup
+            for {set index 0} {$index < $maximumIndex} {incr index} {
+                if {[catch {foreach {time value} $bulkData($key,$index) {}}]} {             ;# missing cell: use void value for type
+                    switch $data([lindex $indexColumns($index) 0],type) {
+                        integer - real {set value ?}
+                        default {set value {}}
+                    }
+                } elseif {$delta($index)} {
+                    set new $value
+                    if {![catch {set period [expr {$time - $last($row,$index,time)}]}] && ($period > 0)} {
+                        # display deltas per second for counters
+                        set value [format %.2f [expr {int($new - $last($row,$index)) / $period}]]
+                    } else {
+                        set value ?                                                   ;# need at least 2 values to make a difference
+                    }
+                    set last($row,$index) $new
+                    set last($row,$index,time) $time
+                } elseif {$bulkType($index) eq "OBJECT IDENTIFIER"} {
+                    catch {set value [mib name $value]}                                      ;# attempt conversion to readable value
+                    if {$::tnm3 && ([mib syntax $identifier] eq "OBJECT IDENTIFIER")} {
+                        set value [mib label $value]                                                    ;# do not display MIB header
+                    }
+                } elseif {$bulkType($index) eq "TimeTicks"} {
+                    set value [formattedTimeTicks $value]
+                }
+                foreach column $indexColumns($index) {
+                    set data($row,$column) $value                                            ;# fill the columns for that identifier
+                }
+            }
+        }
+        foreach name [array names data *,0] {                                                         ;# cleanup disappeared entries
+            set row [lindex [split $name ,] 0]
+            if {[info exists current($row)]} continue
             array unset data $row,\[0-9\]*
-            array unset last $row,*
+            array unset last $row,\[0-9\]*
         }
-        catch {unset currentRow}
     }
 
     proc processError {status errorIndex objects} {
@@ -699,14 +700,14 @@ if {$::tnm3} {
     }
 
 if {$::tnm3} {
- 
+
     proc formattedTimeTicks {value} {                                                                    ;# value is in centiseconds
         set integer [expr {$value / 100}]
         return [formattedTime $integer][format %02u [expr {$value - ($integer * 100)}]]
     }
 
 } else {
- 
+
     proc formattedTimeTicks {value} {
         set centiseconds 0                                                                         ;# in case it is zero and not set
         scan $value {%ud %u:%u:%u.%u} days hours minutes seconds centiseconds
@@ -716,7 +717,7 @@ if {$::tnm3} {
 }
 
     proc mibFiles {directory names} {
-        if {[string length $directory] > 0} {
+        if {$directory ne ""} {
             if {![file isdirectory $directory]} {
                 error "$directory: not a directory"
             }
@@ -726,11 +727,11 @@ if {$::tnm3} {
         }
         set files {}
         foreach name $names {
-            if {[string length [file extension $name]] == 0} {
+            if {[file extension $name] eq ""} {
                 append name .mib                                           ;# complete with default file name extension if necessary
             }
-            if {([string length $directory] > 0) && [string equal [file dirname $name] .]} {
-                set name [file join $directory $name]             ;# eventually join file without directory with specified directory
+            if {($directory ne "") && ([file dirname $name] eq ".")} {
+                set name [file join $directory $name]               ;# possibly join file without directory with specified directory
             }
             lappend files $name
         }
