@@ -1,33 +1,28 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: stagraph.tcl,v 2.49 2004/01/01 11:39:06 jfontain Exp $
+# $Id: stagraph.tcl,v 2.60 2005/02/05 21:25:47 jfontain Exp $
 
 
 class dataStackedGraph {
 
-    set (dropMessage) "stacked graph chart:\ndrop data cell(s)"
-
     proc dataStackedGraph {this parentPath args} composite {[new frame $parentPath] $args} blt2DViewer {
-        $widget::($this,path) [blt::barchart $widget::($this,path).graph\
-            -title {} -barmode stack -plotbackground $bltGraph::plotBackground -barwidth 0\
-        ]
+        $widget::($this,path) [blt::barchart $widget::($this,path).graph -title {} -barmode stack -barwidth 0] 0 1
     } {
-        set path $widget::($this,path)
-        set graphPath $widget::($this,path).graph
-        $graphPath grid off                                                                                 ;# grid is on by default
-        set graph [new bltGraph $graphPath $this]
-        # arrange for labels to remain priorly visible and the graph minimally so:
-        grid columnconfigure $path 0 -minsize 70 -weight 1
-        grid $graphPath -row 0 -column 0 -sticky nsew
+        set ($this,graphPath) $widget::($this,path).graph
+        set graph [new bltGraph $($this,graphPath) $this]
         bltGraph::hideAxisAndCrossHair $graph 1
-        updateMessage $this
+        $blt2DViewer::($this,menu) add checkbutton\
+            -label $bltGraph::(menu,grid,label) -command "composite::configure $this -grid \$dataStackedGraph::($this,-grid)"\
+            -variable dataStackedGraph::($this,-grid) -offvalue 0 -onvalue 1                         ;# add grid toggling menu entry
+        menuContextHelp::set $blt2DViewer::($this,help) [$blt2DViewer::($this,menu) index end] $bltGraph::(menu,grid,help)
+        after idle "dataStackedGraph::updateMessage $this"                        ;# delayed since it needs object to be constructed
         set ($this,graph) $graph
         composite::complete $this
     }
 
     proc ~dataStackedGraph {this} {
-        after idle delete $($this,graph)                      ;# delay deletion since axis is hidden while this object is destructed
+        delete $($this,graph)
         if {[string length $composite::($this,-deletecommand)] > 0} {
             uplevel #0 $composite::($this,-deletecommand)                                   ;# always invoke command at global level
         }
@@ -46,14 +41,17 @@ class dataStackedGraph {
         }
     }
 
-    proc options {this} {                                                                   ;# force size and X axis labels rotation
+    proc options {this} {
         set samples [expr {$global::graphNumberOfIntervals + 1}]
         return [list\
             [list -cellcolors {} {}]\
             [list -deletecommand {} {}]\
             [list -draggable 0 0]\
+            [list -grid $global::graphDisplayGrid]\
             [list -height $global::viewerHeight]\
             [list -interval 0 0]\
+            [list -labelsposition right]\
+            [list -plotbackground $global::graphPlotBackground]\
             [list -samples $samples $samples]\
             [list -width $global::viewerWidth]\
             [list -xlabelsrotation $global::graphXAxisLabelsRotation]\
@@ -71,6 +69,15 @@ class dataStackedGraph {
 
     proc set-deletecommand {this value} {}
 
+    proc set-grid {this value} {
+        if {$value} {
+            $($this,graphPath) grid configure -hide no
+        } else {
+            $($this,graphPath) grid configure -hide yes
+        }
+        set ($this,-grid) $value                                                ;# so that corresponding popup menu entry is updated
+    }
+
     foreach option {-height -width} {
         proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
     }
@@ -100,14 +107,24 @@ class dataStackedGraph {
         bltGraph::xUpdateGraduations $graph
     }
 
+    proc set-labelsposition {this value} {
+        blt2DViewer::updateLayout $this
+    }
+
+    proc set-plotbackground {this value} {
+        $($this,graphPath) configure -plotbackground $value
+        $($this,graphPath) grid configure -color [visibleForeground $value]
+    }
+
     proc set-xlabelsrotation {this value} {
         bltGraph::xRotateLabels $($this,graph) $value
     }
+
     proc set-ymaximum {this value} {
-        blt2DViewer::ySetMaximum $this $value
+        blt2DViewer::setLimit $this maximum $value
     }
     proc set-ymaximumcell {this value} {
-        blt2DViewer::ySetMaximumCell $this $value
+        blt2DViewer::setLimitCell $this maximum $value
     }
 
     proc updateTimeDisplay {this seconds} {
@@ -154,7 +171,7 @@ class dataStackedGraph {
         bltGraph::xUpdateGraduations $graph
         bltGraph::xAxisUpdateRange $graph $maximum
         if {[info exists cell]} {                                                 ;# no maximum can exist when there are no elements
-            blt2DViewer::yUpdateMaximum $this $array                           ;# note: maximum cell will take value at end of range
+            blt2DViewer::updateLimit $this maximum $array                      ;# note: maximum cell will take value at end of range
         }
     }
 
@@ -165,7 +182,8 @@ class dataStackedGraph {
 
     proc updateMessage {this} {
         if {[llength [blt2DViewer::cells $this]] == 0} {
-            centerMessage $widget::($this,path) $(dropMessage) $bltGraph::plotBackground $global::viewerMessageColor
+            centerMessage $widget::($this,path)\
+                [mc "stacked graph chart:\ndrop data cell(s)"] $composite::($this,-plotbackground) $global::viewerMessageColor
         } else {
             centerMessage $widget::($this,path) {}
         }
@@ -173,8 +191,10 @@ class dataStackedGraph {
 
     proc initializationConfiguration {this} {
         return [concat\
-            [list -ymaximum $composite::($this,-ymaximum) -ymaximumcell $composite::($this,-ymaximumcell)]\
-            [blt2DViewer::_initializationConfiguration $this]\
+            [list\
+                -ymaximum $composite::($this,-ymaximum) -ymaximumcell $composite::($this,-ymaximumcell)\
+                -labelsposition $composite::($this,-labelsposition) -grid $composite::($this,-grid)\
+            ] [blt2DViewer::_initializationConfiguration $this]\
         ]
     }
 
