@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: store.tcl,v 1.39 2004/02/06 23:19:42 jfontain Exp $
+# $Id: store.tcl,v 1.57 2005/01/02 00:45:07 jfontain Exp $
 
 # Remember cells the data of which is to be stored for history purposes, in a database, by the moomps daemon.
 # A dialog box is used with a table as a drop site for data cells.
@@ -10,11 +10,13 @@
 class store {
 
     variable number
+    variable titles {label active current comment}
     set column 0
-    foreach title {label active current comment} {                           ;# note: current column data is only for dialog box use
+    foreach title $titles {                                                  ;# note: current column data is only for dialog box use
         set number($title) $column
         incr column
     }
+    unset column
 
     proc store {this args} switched {$args} viewer {} {
         variable singleton
@@ -32,29 +34,23 @@ class store {
     proc options {this} {
         return [list\
             [list -configurations {} {}]\
-            [list -comments {} {}]\
         ]
     }
 
-    proc set-configurations {this value} {      ;# list of lists of switch/value pairs from save file (used starting at moodss 17.7)
-        set ($this,initializeIndex) 0                                        
-    }
-
-    proc set-comments {this value} {                     ;# list of comments, one per cell, from save file (used before moodss 17.7)
-        set ($this,initializeIndex) 0
-    }
+    proc set-configurations {this value} {}                                    ;# list of lists of switch/value pairs from save file
 
     proc setData {dataName row cell active comment} {
         variable number
         upvar 1 $dataName data
 
         viewer::parse $cell array cellRow cellColumn type
-        set label [viewer::label $array $cellRow $cellColumn 1]
+        foreach {label incomplete} [viewer::label $array $cellRow $cellColumn 1] {}
         set data($row,-1) $cell
         set data($row,$number(label)) $label
         set data($row,$number(active)) $active
         set data($row,$number(current)) {}                                                                  ;# updated in dialog box
         set data($row,$number(comment)) $comment
+        return $incomplete
     }
 
     proc sortedRows {dataName} {
@@ -71,32 +67,31 @@ class store {
         return $global::dataTypes                                                                                       ;# all types
     }
 
-    # invoked by core during initialization from save file only, dropped cells must be handled below by dialog box:
+    # invoked by core during initialization from save file only, dropped cells must be handled below by dialog box
     proc monitorCell {this array row column} {
         variable data
         variable number
 
+        if {[llength $switched::($this,-configurations)] == 0} return                            ;# done initializing from save file
+        set cell ${array}($row,$column)
         viewer::registerTrace $this $array
-        set index $($this,initializeIndex)
-        incr ($this,initializeIndex)
-        set length [llength $switched::($this,-configurations)]
-        if {$length > 0} {                                                                   ;# new format (starting at moodss 17.7)
-            array set option [lindex $switched::($this,-configurations) $index]
-            setData data $index ${array}($row,$column) $option(-active) $option(-comment)
-            if {$($this,initializeIndex) == $length} {                                           ;# done initializing from save file
-                switched::configure $this -configurations {}
-            }
-        } else {
-            setData data $index ${array}($row,$column) 1 [lindex $switched::($this,-comments) $index]
-            if {$($this,initializeIndex) == $length} {                                           ;# done initializing from save file
-                switched::configure $this -comments {}
-            }
+        set rowIndex [llength [array names data *,-1]]                                                         ;# next available row
+        set index 0
+        foreach configuration $switched::($this,-configurations) {
+            catch {unset option}; array set option $configuration
+            if {![info exists option(-cell)]} break                                                                    ;# old format
+            if {[string equal $option(-cell) $cell]} break          ;# new format, from 19.1, found configuration for monitored cell
+            # since cells and their configurations are recorded in the same order, skip configurations of void cells, which
+            # can happen when their module has not been loaded due to some initialization problem
+            incr index
         }
-        if {$($this,initializeIndex) == $length} {unset ($this,initializeIndex)}
-        if {[string first ? $data($index,$number(label))] >= 0} {                                  ;# label cannot be determined yet
-            set ($this,relabel,$index) {}
+        set incomplete [setData data $rowIndex $cell $option(-active) $option(-comment)]
+        # eat processed configurations. note: there cannot be several configurations fro the same cell.
+        switched::configure $this -configurations [lrange $switched::($this,-configurations) [incr index] end]
+        if {$incomplete} {                                                                         ;# label cannot be determined yet
+            set ($this,relabel,$rowIndex) {}
         }
-        set ($this,register,$index) {}                                                       ;# register with the database once only
+        set ($this,register,$rowIndex) {}                                                    ;# register with the database once only
     }
 
     proc update {this array} {     ;# if array is void, it means to eventually update database static data (invoked from dialog box)
@@ -107,14 +102,13 @@ class store {
         foreach {name cell} [array get data *,-1] {                                               ;# cells are kept in hidden column
             if {$externalUpdate && ([string first $array $cell] != 0)} continue          ;# check that cell belongs to updated array
             set row [lindex [split $name ,] 0]
-            viewer::parse $cell ignore cellRow cellColumn type
-            if {[info exists ($this,relabel,$row)]} {                                      ;# if label is not yet defined, update it
-                set label [viewer::label $array $cellRow $cellColumn 1]
-                set data($row,$number(label)) $label
-                if {[string first ? $label] < 0} {                                                   ;# label now completely defined
-                    unset ($this,relabel,$row)
-                    set ($this,register,$row) {}                                                        ;# eventually register again
+            viewer::parse $cell array cellRow cellColumn type  ;# note: array needs be (re)set here in case array parameter was void
+            if {[info exists ($this,relabel,$row)] && [info exists $cell]} {               ;# if label is not yet defined, update it
+                foreach [list data($row,$number(label)) incomplete] [viewer::label $array $cellRow $cellColumn 1] {}
+                if {!$incomplete} {
+                    unset ($this,relabel,$row)                                                       ;# label now completely defined
                 }
+                set ($this,register,$row) {}                                                            ;# eventually register again
             }
             set database $global::database
             if {$database == 0} continue                                                      ;# no history to be stored in database
@@ -131,7 +125,8 @@ if {$global::withGUI} {
                 set ($this,databaseInstance,$array) $instance
             }
             if {[info exists ($this,register,$row)]} {       ;# note: do not prefix label with module identifier to avoid redundancy
-                database::monitor $database $instance $cellRow $cellColumn [viewer::label $array $cellRow $cellColumn 0] $comment
+                database::monitor\
+                    $database $instance $cellRow $cellColumn [lindex [viewer::label $array $cellRow $cellColumn 0] 0] $comment
                 unset ($this,register,$row)
             }
             if {$externalUpdate} {                                                ;# only update static data when invoked internally
@@ -147,7 +142,7 @@ if {$global::withGUI} {
                 set ($this,databaseInstance,$array) $instance
             }
             if {[info exists ($this,register,$row)]} {       ;# note: do not prefix label with module identifier to avoid redundancy
-                $database monitor $instance $cellRow $cellColumn [viewer::label $array $cellRow $cellColumn 0] $comment
+                $database monitor $instance $cellRow $cellColumn [lindex [viewer::label $array $cellRow $cellColumn 0] 0] $comment
                 unset ($this,register,$row)
             }
             # note: there are only external (core) updates in daemon mode
@@ -157,7 +152,7 @@ if {$global::withGUI} {
         }
     }
 
-    proc cells {this} {
+    proc cells {this} {                                                               ;# note: always return cells in the same order
         variable data
 
         set cells {}
@@ -167,21 +162,21 @@ if {$global::withGUI} {
         return $cells
     }
 
+    proc manageable {this} {return 0}                                           ;# dialog box is displayed and managed locally below
+
+if {$global::withGUI} {
+
     proc initializationConfiguration {this} {        ;# return a list of comments, one for each stored cell, in the cells list order
         variable number
         variable data
 
         set arguments {}
-        foreach row [sortedRows data] {
-            lappend arguments [list -active $data($row,$number(active)) -comment $data($row,$number(comment))]
+        foreach row [sortedRows data] {                                     ;# note: -cell option added from 19.1: see monitorCell{}
+            lappend arguments [list -cell $data($row,-1) -active $data($row,$number(active)) -comment $data($row,$number(comment))]
         }
-        return [list -configurations $arguments]
+        return [list -configurations $arguments]                             ;# note: always return configurations in the same order
     }
 
-    proc manageable {this} {return 0}                                           ;# dialog box is displayed and managed locally below
-
-if {$global::withGUI} {
-
     proc reload {dataName} {                                  ;# data comes from dialog box table that the user edited and validated
         variable data
         variable singleton
@@ -208,6 +203,19 @@ if {$global::withGUI} {
         return 0
     }
 
+    proc anyActiveCells {this} {
+        variable data
+        variable number
+
+        foreach name [array names data *,-1] {                                                    ;# cells are kept in hidden column
+            set row [lindex [split $name ,] 0]
+            if {$data($row,$number(active))} {return 1}
+        }
+        return 0
+    }
+
+}
+
     proc reset {this} {                                                       ;# return to original state when singleton was created
         variable data
 
@@ -216,10 +224,20 @@ if {$global::withGUI} {
             viewer::unregisterTrace $this $array
         }
         catch {unset data}
-        switched::configure $this -comments {}
     }
 
-}
+    proc active {options} {                                                  ;# public procedure: returns the number of active cells
+        array set value $options
+        if {![info exists value(-configurations)]} {
+            return 0                                                                                                     ;# no cells
+        }
+        set cells 0
+        foreach options $value(-configurations) {                                              ;# list of switch, value, switch, ...
+            array set option $options
+            if {$option(-active)} {incr cells}
+        }
+        return $cells
+    }
 
 }
 
@@ -248,22 +266,18 @@ class store {
 
     class dialog {
 
-        variable help
-        set help(label) {data cell identification}
-        set help(active) {whether data cell history should be recorded in database}
-        set help(current) {current value of data cell}
-        set help(comment) {user editable comment}
-
         proc dialog {this parentPath writable {deleteCommand {}}} viewer {} {
             variable ${this}data
 
             set dialog [new dialogBox .\
-                -buttons hoc -default o -title [mc {moodss: Database History Cells}]\
+                -buttons hoc -default o -title [mc {moodss: Database archiving}]\
                 -helpcommand {generalHelpWindow #menus.edit.database} -x [winfo pointerx .] -y [winfo pointery .]\
                 -grab release -otherbuttons delete -command "set store::dialog::($this,valid) 1" -deletecommand "delete $this"\
             ]
             lappend ($this,tips) [linkedHelpWidgetTip $composite::($dialog,help,path)]
-            composite::configure $dialog delete -text Delete -underline 0 -command "store::dialog::delete $this" -state disabled
+            foreach {string underline} [underlineAmpersand [mc &Delete]] {}
+            composite::configure $dialog delete -text $string -underline $underline -command "store::dialog::delete $this"\
+                -state disabled
             set frame [frame $widget::($dialog,path).frame]
             set table [createTable $this $frame]
             set ($this,drop) [new dropSite -path $selectTable::($table,tablePath) -formats DATACELLS\
@@ -284,7 +298,7 @@ class store {
             set ($this,deleteCommand) $deleteCommand
         }
 
-        proc ~dialog {this} {
+        proc ~dialog {this} {                                ;# note: all data trace unregistering occurs in viewer layer destructor
             variable ${this}data
 
             if {$($this,valid)} {                                                                      ;# user validated its choices
@@ -299,26 +313,23 @@ class store {
 
         proc createTable {this parentPath} {
             variable ${this}data
-            variable help
 
+            set help(label) [mc {data cell identification}]
+            set help(active) [mc {whether data cell history should be recorded in database}]
+            set help(current) [mc {current value of data cell}]
+            set help(comment) [mc {user editable comment}]
             set table [new selectTable $parentPath\
                 -selectcommand "store::dialog::select $this" -variable store::dialog::${this}data -titlerows 1 -roworigin -1\
-            ]
+                -columns [llength $store::titles]\
+            ]                                                                   ;# set number of columns according to title row data
             set path $selectTable::($table,tablePath)
             set column 0
-            foreach title {label active current comment} {                                                           ;# table titles
-                set ${this}data(-1,$column) $title
+            foreach title $store::titles {
+                set label [label $path.$column -font $font::(mediumBold) -text [mc $title]]
+                selectTable::windowConfigure $table -1,$column -window $label -padx 1 -pady 1 -sticky nsew
+                lappend ($this,tips) [new widgetTip -path $label -text $help($title)]
                 incr column
             }
-            # set number of columns according to title row data
-            composite::configure $table -columns [llength [array names ${this}data -1,*]]
-            foreach {cell title} [array get ${this}data -1,*] {
-                set label [label $path.$cell -font $font::(mediumBold) -text $title]
-                selectTable::windowConfigure $table $cell -window $label -padx 1 -pady 1 -sticky nsew
-                lappend ($this,tips) [new widgetTip -path $label -text $help($title)]
-            }
-            selectTable::tag $table configure disabled -state disabled
-            selectTable::tag $table col disabled $store::number(label)
             return $table
         }
 
@@ -343,12 +354,12 @@ class store {
                 viewer::parse $cell array ignore ignore ignore
                 set module [modules::identifier $array]
                 if {[string length $module] == 0} {                                ;# ignore cells not attached to a module instance
-                    lifoLabel::flash $global::messenger {data does not belong do an original module table}
+                    lifoLabel::flash $global::messenger [mc {data does not belong to an original module table}]
                     bell
                     continue
                 }
                 if {[string equal $module trace]} {                                                     ;# ignore trace module cells
-                    lifoLabel::flash $global::messenger {cannot monitor cells from trace module}
+                    lifoLabel::flash $global::messenger [mc {cannot monitor cells from trace module}]
                     bell
                     continue
                 }
@@ -395,7 +406,7 @@ class store {
             unset ${this}data
             set row 0; set rows {}
             foreach index [store::sortedRows data] {
-                set ${this}data($row,-1) $data($index,-1) 
+                set ${this}data($row,-1) $data($index,-1)
                 set column $store::number(label); set ${this}data($row,$column) $data($index,$column)
                 set column $store::number(active); set ${this}data($row,$column) $data($index,$column)
                 set column $store::number(comment); set ${this}data($row,$column) $data($index,$column)
