@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: sumtable.tcl,v 2.64 2004/01/01 11:39:06 jfontain Exp $
+# $Id: sumtable.tcl,v 2.76 2005/02/21 21:35:47 jfontain Exp $
 
 
 class summaryTable {
@@ -26,15 +26,21 @@ if {$global::withGUI} {
         set dataName ::summaryTable::$(nextDataIndex)data        ;# generate unique name based on index (available after completion)
         incr (nextDataIndex)
         catch {unset $dataName}
-        array set $dataName {
-            updates 0
-            0,label data 0,type ascii 0,message {data cell description}
-            1,label current 1,type real 1,message {current value}
-            2,label average 2,type real 2,message {average value since viewer creation}
-            3,label minimum 3,type real 3,message {minimum value since viewer creation}
-            4,label maximum 4,type real 4,message {maximum value since viewer creation}
-            indexColumns 0
-        }
+        array set $dataName [list\
+            updates 0\
+            0,label [mc data] 0,type ascii 0,message [mc {data cell description}]\
+            1,label [mc current] 1,type real 1,message [mc {current value}]\
+            2,label [mc average] 2,type real\
+                2,message [mc {average value (since viewer creation in real time mode or for range in database mode)}]\
+            3,label [mc minimum] 3,type real\
+                3,message [mc {minimum value (since viewer creation in real time mode or for range in database mode)}]\
+            4,label [mc maximum] 4,type real\
+                4,message [mc {maximum value (since viewer creation in real time mode or for range in database mode)}]\
+            5,label [mc deviation] 5,type real\
+                5,message [mc {standard deviation (since viewer creation in real time mode or for range in database mode)}]\
+            indexColumns 0\
+            sort {0 increasing}\
+        ]
 if {$global::withGUI} {
         viewTable::createTable $this $dataName "summaryTable::dragData $this"
         updateMessage $this
@@ -82,15 +88,12 @@ if {$global::withGUI} {
 } else {
         set font {}
 }
-        set columns {current average minimum maximum}
         return [list\
             [list -cellrows {} {}]\
-            [list -columns $columns $columns]\
             [list -dataindex {}]\
             [list -deletecommand {} {}]\
             [list -draggable 0 0]\
             [list -interval 0 0]\
-            [list -titlefont $font $font]\
         ]
     }
 
@@ -106,8 +109,6 @@ if {$global::withGUI} {
         viewTable::setCellRows $this $value
     }
 
-    proc set-columns {this value} {}
-
     set (nextDataIndex) 0                            ;# used when data array index is not specified as an option when creating table
     proc reset {} {                                  ;# reset generated counter (invoker must insure that there are no viewers left)
         set (nextDataIndex) 0
@@ -133,17 +134,15 @@ if {$global::withGUI} {
 
     proc set-deletecommand {this value} {}
 
-    foreach option {-draggable -titlefont} {
-        proc set$option {this value} "
+    proc set-draggable {this value} {
 if {$global::withGUI} {
-            set complete \$composite::(\$this,complete)
+        set complete $composite::($this,complete)
 } else {
-            set complete \$switched::(\$this,complete)
+        set complete $switched::($this,complete)
 }
-            if {\$complete} {
-                error {option $option cannot be set dynamically}
-            }
-        "
+        if {$complete} {
+            error {option -draggable cannot be set dynamically}
+        }
     }
 
     proc set-interval {this value} {}                                                   ;# only so that history mode can be detected
@@ -160,29 +159,32 @@ if {$global::withGUI} {
         set cell ${array}($row,$column)
         if {[string length [viewTable::row $this $cell]] > 0} return                                     ;# already displayed, abort
 if {$global::withGUI} {
-        # in history mode, ignore any cell not from an instance module
-        if {($composite::($this,-interval) == 0) && ![string equal [lindex [modules::decoded $array] 0] instance]} return
+        if {($composite::($this,-interval) == 0) && ![string equal [lindex [modules::decoded $array] 0] instance]} {
+            lifoLabel::flash $global::messenger $viewTable::(monitorInstanceCellsMessage)
+            return                                                   ;# in history mode, ignore any cell not from an instance module
+        }
 }
-        set label [viewer::label $array $row $column]
+        foreach {label incomplete} [viewer::label $array $row $column] {}
         set row [viewTable::register $this $cell $array]                                                           ;# view table row
         set dataName $viewTable::($this,dataName)
         set ${dataName}($row,0) $label
         set current ?
         catch {set current [set $cell]}                                                                        ;# cell may not exist
         set ${dataName}($row,1) $current
-        array set $dataName [list $row,2 ? $row,3 ? $row,4 ?]                             ;# initialize average, minimum and maximum
+        array set $dataName [list $row,2 ? $row,3 ? $row,4 ? $row,5 ?]         ;# initialize average, minimum, maximum and deviation
         set ${dataName}($row,updates) 0
         set ${dataName}($row,sum) 0.0
+        set ${dataName}($row,squares) 0.0                                ;# sum of squared values for standard deviation calculation
 if {$global::withGUI} {
         set ${this}cellRange($cell,start) 0                                                                      ;# for history mode
         set ${this}cellRange($cell,end) 0
         # setup action when a row is deleted through a cell drop in eraser site
         set ($this,rowLastWish,$row) [new lastWish "summaryTable::deleteRow $this $cell"]
 }
-        if {[string first ? $label] >= 0} {                                                        ;# label cannot be determined yet
+        if {$incomplete} {                                                                         ;# label cannot be determined yet
             set ($this,relabel,$row) {}
         }
-        set ${dataName}(updates) 0                      ;# let data table update itself (so colors can be set on cells, for example)
+        incr ${dataName}(updates)                       ;# let data table update itself (so colors can be set on cells, for example)
 if {$global::withGUI} {
         updateMessage $this
 }
@@ -190,6 +192,7 @@ if {$global::withGUI} {
 
     proc update {this array} {
         set dataName $viewTable::($this,dataName)
+        set updated 0
         foreach {cell row} [viewTable::cellsAndRows $this] {
             if {[string first $array $cell] != 0} continue             ;# no need to update if cell does not belong to updated array
             if {[catch {set current [set $cell]}] || [string equal $current ?]} {                  ;# cell does not exist or is void
@@ -198,19 +201,19 @@ if {$global::withGUI} {
                     processHistory $this $row $cell                                       ;# last value may be void but others valid
                 }
             } else {                                                                                                ;# data is valid
-                if {[info exists ($this,relabel,$row)]} {                                  ;# if label is not yet defined, update it
-                    viewer::parse $cell ignore cellRow cellColumn type
-                    set label [viewer::label $array $cellRow $cellColumn]
-                    set ${dataName}($row,0) $label
-                    if {[string first ? $label] < 0} {                                               ;# label now completely defined
-                        unset ($this,relabel,$row)
-                    }
-                }
                 set ${dataName}($row,1) $current
                 if {[string is double -strict $current]} {updateCalculations $this $row $cell}          ;# process numeric data part
             }
+            if {[info exists ($this,relabel,$row)] && [info exists $cell]} {               ;# if label is not yet defined, update it
+                viewer::parse $cell ignore cellRow cellColumn type
+                foreach [list ${dataName}($row,0) incomplete] [viewer::label $array $cellRow $cellColumn] {}
+                if {!$incomplete} {                                                                  ;# label now completely defined
+                    unset ($this,relabel,$row)
+                }
+            }
+            set updated 1
         }
-        incr ${dataName}(updates)                                                                    ;# let data table update itself
+        if {$updated} {incr ${dataName}(updates)}                                                    ;# let data table update itself
     }
 
     proc cells {this} {           ;# return cells in creation order (increasing row) so that restoring from save file works properly
@@ -224,16 +227,23 @@ if {$global::withGUI} {
             set dataName $viewTable::($this,dataName)
             set current [set ${dataName}($row,1)]
             set sum [expr {[set ${dataName}($row,sum)] + $current}]
-            set ${dataName}($row,2) [format %.2f [expr {$sum / [incr ${dataName}($row,updates)]}]]
+            set updates [incr ${dataName}($row,updates)]
+            set average [expr {$sum / $updates}]
+            set ${dataName}($row,2) [format %.2f $average]
             set value [set ${dataName}($row,3)]
-            if {[string equal $value ?] || ($current < $value)} {                                   ;# eventually initialize minimum
+            if {[string equal $value ?] || ($current < $value)} {                                     ;# possibly initialize minimum
                 set ${dataName}($row,3) $current
             }
             set value [set ${dataName}($row,4)]
-            if {[string equal $value ?] || ($current > $value)} {                                   ;# eventually initialize maximum
+            if {[string equal $value ?] || ($current > $value)} {                                     ;# possibly initialize maximum
                 set ${dataName}($row,4) $current
             }
+            set squares [expr {[set ${dataName}($row,squares)] + ($current * $current)}]
+            set value 0                                                           ;# for the first sample as division by zero occurs
+            catch {set value [expr {sqrt(($squares + ($updates * $average * $average) - (2 * $average * $sum)) / ($updates - 1))}]}
+            set ${dataName}($row,5) [format %.2f $value]
             set ${dataName}($row,sum) $sum
+            set ${dataName}($row,squares) $squares
         }
     }
 
@@ -250,6 +260,7 @@ if {$global::withGUI} {
             set ${dataName}($row,2) ?
             set ${dataName}($row,3) ?
             set ${dataName}($row,4) ?
+            set ${dataName}($row,5) ?
             return
         }
         if {([set ${this}cellRange($cell,start)] == $start) && ([set ${this}cellRange($cell,end)] == $end)} {
@@ -274,11 +285,14 @@ if {$global::withGUI} {
             regsub {\.0$} [result index 0] {} ${dataName}($row,3)                           ;# strip .0 trailer if result is integer
             result expr {max(values)}
             regsub {\.0$} [result index 0] {} ${dataName}($row,4)                           ;# strip .0 trailer if result is integer
+            result expr {sdev(values)}                                                                         ;# standard deviation
+            set ${dataName}($row,5) [format %.2f [result index 0]]
             blt::vector destroy result
         } else {                                                                                            ;# there were only voids
             set ${dataName}($row,2) ?
             set ${dataName}($row,3) ?
             set ${dataName}($row,4) ?
+            set ${dataName}($row,5) ?
         }
         blt::vector destroy values
     }
@@ -308,20 +322,16 @@ if {$global::withGUI} {
         set dataName $viewTable::($this,dataName)
         set row [viewTable::deleteRow $this $cell]
         unset ${dataName}($row,0) ${dataName}($row,1) ${dataName}($row,2) ${dataName}($row,3) ${dataName}($row,4)\
-            ($this,rowLastWish,$row) ${dataName}($row,updates) ${dataName}($row,sum)
+            ${dataName}($row,5) ($this,rowLastWish,$row) ${dataName}($row,updates) ${dataName}($row,sum) ${dataName}($row,squares)
         catch {unset ${this}cellRange($cell,start) ${this}cellRange($cell,end)}                                  ;# for history mode
         viewTable::update $this
         updateMessage $this
     }
 
-}
-
     proc initializationConfiguration {this} {
         return [viewTable::initializationConfiguration $this]
     }
 
-if {$global::withGUI} {
-
     proc monitored {this cell} {
         return [viewTable::monitored $this $cell]
     }
@@ -335,7 +345,7 @@ if {$global::withGUI} {
             centerMessage $widget::($this,path) {}
         } else {
             centerMessage $widget::($this,path)\
-                "statistics table:\ndrop data cell(s)" $viewer::(background) $global::viewerMessageColor
+                [mc "statistics table:\ndrop data cell(s)"] $viewer::(background) $global::viewerMessageColor
         }
     }
 
