@@ -1,27 +1,31 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: trace.tcl,v 1.17 2004/01/01 11:39:06 jfontain Exp $
+# $Id: trace.tcl,v 1.19 2005/01/02 00:45:07 jfontain Exp $
 
 
-package provide trace [lindex {$Revision: 1.17 $} 1]
+package provide trace [lindex {$Revision: 1.19 $} 1]
+package require msgcat
+namespace import msgcat::*
+package require internationalization
+
 
 namespace eval trace {
 
     variable nextRow 0
     variable numberOfRows 10                                                                                           ;# by default
 
-    array set data {
-        updates 0
-        0,label {} 0,type integer 0,message {row creation order}
-        1,label date 1,type clock 1,message {date of message occurrence}
-        2,label time 2,type clock 2,message {time of message occurrence}
-        3,label module 3,type dictionary 3,message {module identification} 3,anchor left
-        4,label message 4,type ascii 4,message {message from module} 4,anchor left
-        switches {-m 1 --modules 1 --rows 1}
-        pollTimes -10
-        sort {0 decreasing}
-    }
+    array set data [list\
+        updates 0\
+        0,label {} 0,type integer 0,message [mc {row creation order}]\
+        1,label [mc date] 1,type clock 1,message [mc {date of message occurrence}]\
+        2,label [mc time] 2,type clock 2,message [mc {time of message occurrence}]\
+        3,label [mc module] 3,type dictionary 3,message [mc {module identification}] 3,anchor left\
+        4,label [mc message] 4,type ascii 4,message [mc {message from module}] 4,anchor left\
+        switches {-m 1 --modules 1 --rows 1}\
+        pollTimes -10\
+        sort {0 decreasing}\
+    ]
     set file [open trace.htm]
     set data(helpText) [read $file]                                                           ;# initialize HTML help data from file
     close $file
