@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: utility.tcl,v 1.70 2004/01/01 11:39:06 jfontain Exp $
+# $Id: utility.tcl,v 1.85 2005/01/02 00:45:07 jfontain Exp $
 
 
 proc commaSeparatedString {words} {
@@ -12,17 +12,16 @@ proc commaSeparatedString {words} {
     return $string
 }
 
-proc startGatheringPackageDirectories {} {
+proc startGatheringPackageDirectories {} {   ;# note: the first package requirement triggers the search for packages by the Tcl core
     catch {rename source _source}
     proc source {file} {
-        if {![string equal [file tail $file] pkgIndex.tcl]} {
-            # all moodss modules must use pkgIndex.tcl files for package management
+        if {![string equal [file tail $file] pkgIndex.tcl]} {   ;# moodss modules must use pkgIndex.tcl files for package management
             return [uplevel 1 _source [list $file]]
         }
         foreach name [package names] {
-            set versions($name) [package versions $name]
+            set versions($name) [package versions $name]                ;# note: can be an empty list (for Tcl package, for example)
         }
-        uplevel 1 _source [list $file]
+        uplevel 1 _source [list $file]                                                       ;# most of the time a pkgIndex.tcl file
         set directory [file dirname $file]
         foreach name [package names] {                                                        ;# see what new packages are available
             set available [package versions $name]
@@ -32,7 +31,7 @@ proc startGatheringPackageDirectories {} {
                     if {![info exists ::package(moodss,$name)]} {
                         set ::package(directory,$name) $directory
                         set ::package(version,$name) [lindex $available end]               ;# the new package version is always last
-                    }                       ;# else there is a confirmea moodss module package by that name already so load that one
+                    }                       ;# else there is a confirmed moodss module package by that name already so load that one
                 }
             } else {                                                                              ;# first time this package is seen
                 set ::package(directory,$name) $directory
@@ -74,74 +73,13 @@ proc linesCount {string} {
     return [llength [split $string \n]]
 }
 
-if {$global::withGUI} {                                                                        ;# tkTable related utility procedures
-
-proc adjustTableColumns {table} {                               ;# automatically set column widths according to column cells content
-    upvar #0 [$table cget -variable] data
-
-    if {[array size data] == 0} return
-    update idletasks                                                 ;# make sure table and its labels is completely drawn and sized
-    set label [label .temporary]        ;# use a temporary label for precise measurements, instead of using the font measure command
-    set firstRow [$table cget -roworigin]
-    set lastRow [expr {$firstRow + [$table cget -rows]}]
-    set column [$table cget -colorigin]
-    set lastColumn [expr {$column + [$table cget -cols]}]
-    set defaultFont [$table cget -font]
-    for {} {$column < $lastColumn} {incr column} {
-        set maximum 0
-        for {set row $firstRow} {$row < $lastRow} {incr row} {
-            if {[string length [$table hidden $row,$column]] > 0} continue                         ;# take hidden cell width as null
-            if {[catch {set window [$table window cget $row,$column -window]}]} {
-                set font $defaultFont
-                if {[$table tag includes title $row,$column] && ([string length [$table tag cget title -font]] > 0)} {
-                    set font [$table tag cget title -font]
-                }
-                set text {}; catch {set text $data($row,$column)}                                              ;# data may not exist
-                $label configure -font $font -text $text
-                set width [expr {[winfo reqwidth $label] + (2 * [$table cget -padx])}]
-            } else {
-                set width [expr {[winfo reqwidth $window] + (2 * [$table window cget $row,$column -padx])}]
-            }
-            if {$width > $maximum} {
-                set maximum $width
-            }
-        }
-        $table width $column -$maximum
-    }
-    destroy $label
-}
-
-proc drawTableLimits {path lastColumn {embeddedWindowsCommand {}}} {
-    set previous [$path tag row lastrow]
-    if {[llength $previous] > 0} {                            ;# eventually reset last row aspect in case number of rows has changed
-        $path tag row {} $previous
-        if {[string length $embeddedWindowsCommand] > 0} {
-            uplevel #0 $embeddedWindowsCommand $previous {{1 0 1 0}}
-        }
-    }
-    catch {$path tag cell {} [$path tag cell lastcell]}      ;# eventually reset last cell aspect in case number of rows has changed
-    set row [$path index end row]
-    if {$row < 0} {                                                                                                  ;# no data rows
-        $path configure -borderwidth {1 0 1 1}                             ;# so that title row bottom is delimited by a thin border
-        $path window configure -1,$lastColumn -borderwidth 1
-    } else {
-        $path configure -borderwidth {1 0 1 0}                            ;# only draw a thin dark line on top and left of each cell
-        $path window configure -1,$lastColumn -borderwidth {1 1 1 0}         ;# so that the right side is delimited by a thin border
-        $path tag row lastrow $row                                               ;# so that the bottom is delimited by a thin border
-        $path tag cell lastcell [$path index end]
-        if {[string length $embeddedWindowsCommand] > 0} {
-            uplevel #0 $embeddedWindowsCommand $row {{1 0 1 1}}
-        }
-    }
-}
-
-}
-
 proc compareClocks {value1 value2} {
     return [expr {[clock scan $value1 -base 0] - [clock scan $value2 -base 0]}]
 }
 
 proc emailAddressError {string} {                                          ;# requires tcllib mime package (tested with version 1.2)
+    set string [string trim $string]
+    if {[string length $string] == 0} {return {blank address}}
     foreach list [mime::parseaddress $string] {
         array set array $list
     }
@@ -234,6 +172,7 @@ proc centerMessage {path text {background {}} {foreground {}}} {
     set label $path.centeredMessage
     if {[string length $text] == 0} {
         catch {destroy $label}                                                                                ;# label may not exist
+        set label {}
     } else {
         if {![winfo exists $label]} {
             label $label
@@ -241,6 +180,7 @@ proc centerMessage {path text {background {}} {foreground {}}} {
         $label configure -text $text -background $background -foreground $foreground
         place $label -relx 0.5 -rely 0.5 -anchor center
     }
+    return $label
 }
 
 proc 3DBorders {path background} {                                                               ;# algorithm stolen from tkUnix3d.c
@@ -285,4 +225,155 @@ proc vectors {left top width height} {                   ;# from rectangle, retu
     ]
 }
 
+if {[package vcompare $::tcl_version 8.4] < 0} {
+
+    proc setupGlobalMouseWheelBindings {} {                                                                 ;# adapted from TIP #171
+        set classes [list Text Listbox Table TreeCtrl]
+        foreach class $classes {bind $class <MouseWheel> {}}
+        if {[string equal $::tcl_platform(platform) unix]} {
+            foreach class $classes {
+                bind $class <4> {}
+                bind $class <5> {}
+            }
+        }
+        bind all <MouseWheel> [list ::tkMouseWheel %W %D %X %Y]
+        if {[string equal $::tcl_platform(platform) unix]} {
+            bind all <4> [list ::tkMouseWheel %W 120 %X %Y]
+            bind all <5> [list ::tkMouseWheel %W -120 %X %Y]
+        }
+    }
+    proc ::tkMouseWheel {fired D X Y} {
+        if {[string length [bind [winfo class $fired] <MouseWheel>]] > 0} return
+        set w [winfo containing $X $Y]
+        if {![winfo exists $w]} {catch {set w [focus]}}
+        if {[winfo exists $w]} {
+            if {[string equal [winfo class $w] Scrollbar]} {
+                catch {tkScrollByUnits $w [string index [$w cget -orient] 0] [expr {-($D / 30)}]}
+            } else {
+                ### only for the following widgets for now as composite widgets using the canvas react strangely:
+                switch [winfo class $w] {Text - Listbox - Table - TreeCtrl {} default return}
+                catch {$w yview scroll [expr {-($D / 120) * 4}] units}
+            }
+        }
+    }
+
+    proc underlineAmpersand {text} {                                                             ;# copied from tk.tcl in Tcl/Tk 8.4
+        set idx [string first "&" $text]
+        if {$idx >= 0} {
+            set underline $idx
+            # ignore "&&"
+            while {[string match "&" [string index $text [expr {$idx + 1}]]]} {
+                set base [expr {$idx + 2}]
+                set idx  [string first "&" [string range $text $base end]]
+                if {$idx < 0} {
+                    break
+                } else {
+                    set underline [expr {$underline + $idx + 1}]
+                    incr idx $base
+                }
+            }
+        }
+        if {$idx >= 0} {
+            regsub -all -- {&([^&])} $text {\1} text
+        }
+        return [list $text $idx]
+    }
+
+} else {
+
+    proc setupGlobalMouseWheelBindings {} {                                                                         ;# from TIP #171
+        set mw_classes [list Text Listbox Table TreeCtrl]
+        foreach class $mw_classes { bind $class <MouseWheel> {} }
+        if {[tk windowingsystem] eq "x11"} {
+            foreach class $mw_classes {
+                bind $class <4> {}
+                bind $class <5> {}
+            }
+        }
+        bind all <MouseWheel> [list ::tk::MouseWheel %W %D %X %Y]
+        if {[tk windowingsystem] eq "x11"} {
+            # Support for mousewheels on Linux/Unix commonly comes through
+            # mapping the wheel to the extended buttons.
+            bind all <4> [list ::tk::MouseWheel %W 120 %X %Y]
+            bind all <5> [list ::tk::MouseWheel %W -120 %X %Y]
+        }
+    }
+    proc ::tk::MouseWheel {wFired D X Y} {
+        # do not double-fire in case the class already has a binding
+        if {[bind [winfo class $wFired] <MouseWheel>] ne ""} { return }
+        # obtain the window the mouse is over
+        set w [winfo containing $X $Y]
+        # if we are outside the app, try and scroll the focus widget
+        if {![winfo exists $w]} { catch {set w [focus]} }
+        if {[winfo exists $w]} {
+            # scrollbars have different call conventions
+            if {[winfo class $w] eq "Scrollbar"} {
+                catch {tk::ScrollByUnits $w \
+                    [string index [$w cget -orient] 0] \
+                    [expr {-($D / 30)}]}
+            } else {
+                ### only for the following widgets for now as composite widgets using the canvas react strangely:
+                switch [winfo class $w] {Text - Listbox - Table - TreeCtrl {} default return}
+                catch {$w yview scroll [expr {-($D / 120) * 4}] units}
+            }
+        }
+    }
+
+    proc underlineAmpersand {text} {
+        return [::tk::UnderlineAmpersand $text]
+    }
+
+}
+
+proc dragEcho {data format} {
+    return $data
+}
+
+proc bounds {canvas} {                                              ;# returns the current canvas bounds (changes when page changes)
+    foreach {left top right bottom} [$canvas cget -scrollregion] {}                                                  ;# current page
+    # actual size may be larger than user specified size:
+    return [list\
+        $left $top\
+        [expr {$left + [maximum [winfo width $canvas] [expr {$right - $left}]]}]\
+        [expr {$top + [maximum [winfo height $canvas] [expr {$bottom - $top}]]}]\
+    ]
+}
+
+proc fenceRectangle {canvas list} {                                                                   ;# list: left top right bottom
+    foreach {xMinimum yMinimum} [pages::closestPageTopLeftCorner [lindex $list 0]] {}
+    foreach {left top right bottom} [bounds $canvas] {}
+    set xMaximum [expr {$xMinimum + ($right - $left)}]; set yMaximum [expr {$yMinimum + ($bottom - $top)}]
+    foreach {left top right bottom} $list {}
+    set x 0; set y 0
+    if {$left < $xMinimum} {
+        set x [expr {$xMinimum - $left}]
+    } elseif {$right > $xMaximum} {
+        set x [expr {$xMaximum - $right}]
+    }
+    if {$top < $yMinimum} {
+        set y [expr {$yMinimum - $top}]
+    } elseif {$bottom > $yMaximum} {
+        set y [expr {$yMaximum - $bottom}]
+    }
+    return [list $x $y]                                                                                      ;# corrections to apply
+}
+
+proc fence {canvas itemOrTag} {                          ;# make sure object remains entirely visible within the visible canvas area
+    if {([winfo width $canvas] <= 1) || ([winfo height $canvas] <= 1)} return             ;### invalid configuration event (Tk bug?)
+    foreach {x y} [fenceRectangle $canvas [$canvas bbox $itemOrTag]] {}
+    if {($x != 0) || ($y != 0)} {
+        $canvas move $itemOrTag $x $y
+    }
+}
+
+proc visibleForeground {background {path .}} {
+    foreach {red green blue} [winfo rgb $path $background] {}
+    if {($red + $green + $blue) >= (32768 * 3)} {                                                                ;# light background
+        return black
+    } else {                                                                                                      ;# dark background
+        return white
+    }
+}
+
+
 }
