@@ -1,7 +1,7 @@
-# copyright (C) 1997-2004 Jean-Luc Fontaine (mailto:jfontain@free.fr)
+# copyright (C) 1997-2005 Jean-Luc Fontaine (mailto:jfontain@free.fr)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-# $Id: viewer.tcl,v 2.56 2004/01/01 11:39:06 jfontain Exp $
+# $Id: viewer.tcl,v 2.76 2005/02/13 16:46:22 jfontain Exp $
 
 
 class viewer {                                                                             ;# handle viewers related functionalities
@@ -53,7 +53,7 @@ if {$global::withGUI} {
         foreach {array row column} $list {
             monitorCell $this $array $row $column
 if {$global::withGUI} {
-            # eventually initialize cell thresholds data:
+            # possibly initialize cell thresholds data:
             foreach {color level summary} [thresholds::cellData $array $row $column] {
                 thresholdCondition $this $array $row $column $color $level $summary
             }
@@ -72,7 +72,7 @@ if {$global::withGUI} {
     proc parse {dataCell arrayName rowName columnName typeName} {
         upvar 1 $arrayName cellArray $rowName cellRow $columnName cellColumn $typeName cellType
 
-        if {([scan $dataCell {%[^(](%u,%u)} array row column] != 3) || ($row < 0) || ($column < 0)} {
+        if {([scan $dataCell {%[^(](%lu,%u)} array row column] != 3) || ($column < 0)} {
             error "\"$dataCell\" is not a valid array cell"
         }
         set cellArray $array; set cellRow $row; set cellColumn $column; catch {set cellType [set ${array}($column,type)]}
@@ -84,7 +84,9 @@ if {$global::withGUI} {
         }
     }
 
-    # derive a label from cell using the tabular data index columns. array name must be qualified
+    # Derive a label from cell using the tabular data index columns (array name must be qualified) and return it along with the
+    # incomplete status, meaning that the label could not be fully determined, as can happen, for example, when the data cell no
+    # longer exists.
     proc label {array row column {identify {}}} {                                                     ;# allow forcing module header
         set label {}
         if {[string length $identify] == 0} {
@@ -93,10 +95,11 @@ if {$global::withGUI} {
         if {$identify} {
             set identifier [modules::identifier $array]                                         ;# see if array needs identification
             if {[string length $identifier] > 0} {               ;# data comes from a module array (could come from a summary table)
+                regsub {<0>$} $identifier {} identifier            ;# remove trailing namespace index for first instance of a module
                 set label "$identifier: "
             }
         }
-        if {[catch {set ${array}(indexColumns)} columns]} {                                                      ;# no index columns
+        if {[catch {set ${array}(indexColumns)} columns]} {     ;# no index columns (note: they are used to generate the cell label)
             set columns 0                                                                 ;# use first column as single index column
         }
         foreach index $columns {                                                                      ;# use index columns for label
@@ -107,7 +110,11 @@ if {$global::withGUI} {
             }
         }
         append label [set ${array}($column,label)]
-        return $label
+        # consider the label incomplete if it contains any question mark characters, reflecting the presence of missing data rows,
+        # void numeric data cells, or existing cells from a summary table pointing to void data. note that is valid data contains
+        # question mark characters, then the incomplete status is incorrect, which should be rare and have only a minor impact on
+        # performance (it causes the corresponding data labels to be updated at each poll in data viewers).
+        return [list $label [string match {*\?*} $label]]
     }
 
     virtual proc update {this array}                                                              ;# update display using cells data
@@ -120,22 +127,24 @@ if {$global::withGUI} {
         dataTrace::unregister $this $array
     }
 
+    # note: cells must always been returned in the same order (dictionary sorting suggested) so that record layer does not detect
+    # erroneous changes, incorrectly requiring the user to save dashboard when exiting application
     virtual proc cells {this}
 
+if {$global::withGUI} {
+
     virtual proc initializationConfiguration {this} { ;# configuration with switch / value option pairs for initialization from file
         return {}
     }
 
-if {$global::withGUI} {
-
     proc setupDropSite {this path} {                                 ;# allow dropping of data cells, viewer mutation or kill action
         set ($this,drop) [new dropSite -path $path -formats {DATACELLS VIEWER KILL} -command "viewer::handleDrop $this"]
     }
 
     proc handleDrop {this} {
-        if {![catch {set dragSite::data(DATACELLS)} data]} {
+        if {![catch {set data $dragSite::data(DATACELLS)}]} {
             view $this $data
-        } elseif {![catch {set dragSite::data(VIEWER)} data]} {
+        } elseif {![catch {set data $dragSite::data(VIEWER)}]} {
             mutate $this $data
         } elseif {[info exists dragSite::data(KILL)]} {
             delete $this                                                                                           ;# self destructs
@@ -144,7 +153,19 @@ if {$global::withGUI} {
 
     proc mutate {this class} {
         if {[string equal $class [classof $this]]} return                            ;# no need to create a viewer of the same class
-        set viewer [eval new $class $global::canvas -draggable [composite::cget $this -draggable]]
+        set draggable [composite::cget $this -draggable]
+        switch $class {
+            ::currentValueTable {                                 ;# needs to know mode (real time or database) at construction time
+                set viewer [new currentValueTable $global::canvas $global::pollTime -draggable $draggable]
+            }
+            ::canvas::iconic {
+                if {[string length [set name [canvas::iconic::chooseFile]]] == 0} return                                 ;# canceled
+                set viewer [new $class $global::canvas -draggable $draggable -static $global::static -file $name]
+            }
+            default {
+                set viewer [new $class $global::canvas -draggable $draggable]
+            }
+        }
         foreach list [composite::configure $viewer] {
             if {[string equal [lindex $list 0] -interval]} {                                      ;# viewer supports interval option
                 composite::configure $viewer -interval $global::pollTime                                  ;# so use current interval
@@ -164,14 +185,23 @@ if {$global::withGUI} {
             view $viewer $cells
         }
         if {[llength $cells] < $count} {                                                                 ;# warn user when necessary
-            lifoLabel::flash $global::messenger {some data cells no longer exist}
+            lifoLabel::flash $global::messenger [mc {some data cells no longer exist}]
+        }
+        if {[manageable $this]} {
+            foreach {x y width height} [canvasWindowManager::getGeometry $global::windowManager $widget::($this,path)] {}
+            set level [canvasWindowManager::getStackLevel $global::windowManager $widget::($this,path)]
+        } else {                                                                                          ;# must be a canvas viewer
+            set x [composite::cget $this -x]; set y [composite::cget $this -y]                            ;# a viewer is a composite
+            set width {}; set height {}; set level {}
         }
-        foreach {x y width height} [canvasWindowManager::getGeometry $global::windowManager $widget::($this,path)] {}
-        set level [canvasWindowManager::getStackLevel $global::windowManager $widget::($this,path)]
         delete $this                                                                                       ;# delete existing viewer
-        # viewer is as destroyable as previously deleted viewer
-        manageViewer $viewer 1 -static $global::static -setx $x -sety $y -setwidth $width -setheight $height -level $level\
-            -dragobject $viewer
+        if {[manageable $viewer]} {
+            # viewer is as destroyable as previously deleted viewer
+            manageViewer $viewer 1 -static $global::static -setx $x -sety $y -setwidth $width -setheight $height -level $level\
+                -dragobject $viewer
+        } else {                                                                                          ;# must be a canvas viewer
+            composite::configure $viewer -x $x -y $y
+        }
     }
 
     proc cellThresholdColor {array row column} {
@@ -193,7 +223,8 @@ if {$global::withGUI} {
         }
     }
 
-    # viewers should implement only one of the following procedures:
+    # viewers should implement at most one of the following procedures:
+    # note: viewers using a dataTable object might need none, as ::cellThresholdCondition{} may be enough to do the job
 
     # latest threshold condition (reset if summary is not defined (empty)):
     virtual proc thresholdCondition {this array row column color level summary} {}
@@ -216,8 +247,8 @@ if {$global::withGUI} {
 
     # If this is the first time a color is requested for that cell, give a new color, else give the color already used by that cell,
     # so that a cell always has the same color in all its container viewers, until it is no longer monitored by any viewer.
-    # When the cell is no longer displayed by the viewer, the color must be returned returnDisplayColor{}, so that the color can
-    # eventually be made available for other cells.
+    # When the cell is no longer displayed by the viewer, the color must be returned using returnDisplayColor{}, so that the color
+    # can be made available for other cells.
     proc getDisplayColor {cell} {
         variable colorIndex                                          ;# cell to color index mapping (index in global viewers colors)
         variable usageCount                                                   ;# number of times a cell is displayed using its color
@@ -231,7 +262,7 @@ if {$global::withGUI} {
                 incr count($index)
             }
             set color 0
-            set minimum $global::integerMaximum
+            set minimum $global::32BitIntegerMaximum
             for {set index 0} {$index < $colors} {incr index} {                                    ;# find the next least used color
                 if {$count($index) < $minimum} {
                     set minimum $count($index)
@@ -254,21 +285,30 @@ if {$global::withGUI} {
         }
     }
 
-    proc limitEntry {this path anchor x y option name font command} {
+    proc limitEntry {this path anchor x y option name font command yCommand type} {                  ;# note: Y command may be empty
         set entry [entry $path.maximum -borderwidth 0 -highlightthickness 0 -width 10 -font $font]
-        setupEntryValidation $entry {{checkUnsignedInteger %P}}                                       ;# filter on positive integers
-        lifoLabel::push $global::messenger "enter $name value (empty for automatic scale, Return to valid, Escape to abort)"
-        # when Return or Enter is pressed, pass new value as limit, else if Escape is pressed, abort:
+        switch $type {
+            float {setupEntryValidation $entry {{checkFloatingPoint %P}}; set type double}
+            signed {setupEntryValidation $entry {{check32BitSignedInteger %P}}; set type integer}
+            unsigned {setupEntryValidation $entry {{check31BitUnsignedInteger %P}}; set type integer}
+            default error
+        }
+        lifoLabel::push $global::messenger\
+            [format [mc {enter %s value (empty for automatic scale, Return to valid, Escape to abort)}] $name]
+        # when Return or Enter is pressed, pass new value (if valid type) as limit, else if Escape is pressed, abort:
         foreach key {<KP_Enter> <Return>} {
             bind $entry $key "
-                $command \[string trimleft \[%W get\] 0\]                               ;# avoid problems with invalid octal numbers
-                destroy %W
-                lifoLabel::pop $global::messenger
+                if {\[string is $type \[%W get\]\]} {
+                    $command \[%W get\]
+                    destroy %W
+                    lifoLabel::pop $global::messenger
+                }
             "
         }
         bind $entry <Escape> "destroy %W; lifoLabel::pop $global::messenger"
         $entry insert 0 [composite::cget $this $option]
         $entry selection range 0 end                                  ;# initially select all characters, which is a common behavior
+        if {[string length $yCommand] > 0} {set y [uplevel #0 $yCommand]}                                  ;# allow dynamic ordinate
         place $entry -anchor $anchor -x $x -y $y
         focus $entry
         ::update idletasks                                                                     ;# so entry is visible and grab works
@@ -276,12 +316,13 @@ if {$global::withGUI} {
     }
 
     proc wrongTypeMessage {type} {
-        lifoLabel::flash $global::messenger "cannot display data of type $type"
+        lifoLabel::flash $global::messenger [format [mc {cannot display data of type %s}] $type]
         bell
         if {![info exists (wrongTypeDrop)]} {
             set (wrongTypeDrop) {}
-            tk_messageBox -title {moodss: drag and drop} -type ok -icon info -message\
-                "Some viewers only accept a limited set of data types (if this case, this viewer cannot display data of type $type)"
+            tk_messageBox -title {moodss: drag and drop} -type ok -icon info -message [format [mc\
+                {Some viewers only accept a limited set of data types (if this case, this viewer cannot display data of type %s)}\
+            ] $type]
         }
     }
 
