@@ -1,9 +1,22 @@
-set rcsId {$Id: xifo.tcl,v 2.1 1999/08/14 21:49:18 jfontain Exp $}
+# The lifo and fifo classes (for the stooop object oriented extension)
+#
+# Copyright (c) 2002 by Jean-Luc Fontaine <jfontain@free.fr>.
+# This code may be distributed under the same terms as Tcl.
+#
+# $Id: xifo.tcl,v 2.8 2003/11/30 23:31:10 jfontain Exp $
 
-# source stooop.tcl
+
+# Here is a sample FIFO/LIFO implementation with stooop.
+# Sample test code is at the bottom of this file.
+
+
+# Uncomment the following lines for the bottom sample code to work:
+# package require stooop
 # namespace import stooop::*
 
-class xifo {
+
+::stooop::class xifo {
+
     proc xifo {this size} {
         set ($this,size) $size
         empty $this
@@ -17,14 +30,14 @@ class xifo {
     proc in {this data} {
         variable ${this}data
         tidyUp $this
-        if {[array size ${this}data]>=$($this,size)} {
+        if {[array size ${this}data] >= $($this,size)} {
             unset ${this}data($($this,first))
             incr ($this,first)
         }
         set ${this}data([incr ($this,last)]) $data
     }
 
-    proc tidyUp {this} {                                                                           ;# warning: for internal use only
+    proc tidyUp {this} {                       ;# warning: for internal use only
         variable ${this}data
         catch {
             unset ${this}data($($this,unset))
@@ -41,26 +54,29 @@ class xifo {
     }
 
     proc isEmpty {this} {
-        variable ${this}data
-        return [expr {[array size ${this}data]==0}]
+        return [expr {$($this,last) < $($this,first)}]
     }
 
-    virtual proc out {this}
+    ::stooop::virtual proc out {this}
 
-    virtual proc data {this}
+    ::stooop::virtual proc data {this}
 }
 
-class lifo {
+
+::stooop::class lifo {
+
     proc lifo {this {size 2147483647}} xifo {$size} {}
 
     proc ~lifo {this} {}
 
     proc out {this} {
         xifo::tidyUp $this
-        if {[array size xifo::${this}data]==0} {
+        if {[array size xifo::${this}data] == 0} {
             error "lifo $this out error, empty"
         }
-        set xifo::($this,unset) $xifo::($this,last)    ;# delay unsetting popped data to improve performance by avoiding a data copy
+        # delay unsetting popped data to improve performance by avoiding a data
+        # copy:
+        set xifo::($this,unset) $xifo::($this,last)
         incr xifo::($this,last) -1
         return [set xifo::${this}data($xifo::($this,unset))]
     }
@@ -68,24 +84,29 @@ class lifo {
     proc data {this} {
         set list {}
         set first $xifo::($this,first)
-        for {set index $xifo::($this,last)} {$index>=$first} {incr index -1} {
+        for {set index $xifo::($this,last)} {$index >= $first} {incr index -1} {
             lappend list [set xifo::${this}data($index)]
         }
         return $list
     }
+
 }
 
-class fifo {
+
+::stooop::class fifo {
+
     proc fifo {this {size 2147483647}} xifo {$size} {}
 
     proc ~fifo {this} {}
 
     proc out {this} {
         xifo::tidyUp $this
-        if {[array size xifo::${this}data]==0} {
+        if {[array size xifo::${this}data] == 0} {
             error "fifo $this out error, empty"
         }
-        set xifo::($this,unset) $xifo::($this,first)   ;# delay unsetting popped data to improve performance by avoiding a data copy
+        # delay unsetting popped data to improve performance by avoiding a data
+        # copy:
+        set xifo::($this,unset) $xifo::($this,first)
         incr xifo::($this,first)
         return [set xifo::${this}data($xifo::($this,unset))]
     }
@@ -93,16 +114,18 @@ class fifo {
     proc data {this} {
         set list {}
         set last $xifo::($this,last)
-        for {set index $xifo::($this,first)} {$index<=$last} {incr index} {
+        for {set index $xifo::($this,first)} {$index <= $last} {incr index} {
             lappend list [set xifo::${this}data($index)]
         }
         return $list
     }
+
 }
 
-# here are a few lines of example code:
+
+# Here are a few lines of sample code:
 #    proc exercise {id} {
-#        for {set u 0} {$u<10} {incr u} {
+#        for {set u 0} {$u < 10} {incr u} {
 #            xifo::in $id $u
 #        }
 #        puts [xifo::out $id]
@@ -111,9 +134,9 @@ class fifo {
 #        xifo::in $id [incr u]
 #        puts [xifo::data $id]
 #    }
-#    set id [new lifo 10]
+#    set id [stooop::new lifo 10]
 #    exercise $id
-#    delete $id
-#    set id [new fifo 10]
+#    stooop::delete $id
+#    set id [stooop::new fifo 10]
 #    exercise $id
-#    delete $id
+#    stooop::delete $id
